// $Id: DownloadItems.cs 25 2008-04-20 14:51:40Z arcao $
//
// Copyright (C) 2008 Martin "Arcao" Sloup <arcao@arcao.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections;
using System.Collections.Generic;
using System.Xml;

namespace OSD.Program
{
    ///<summary>
    /// TRida zastresujici vsechny polozky stahovani
    ///</summary>
    public class DownloadItems : IEnumerable<DownloadItem>, IDisposable
    {
        private readonly List<DownloadItem> list;
        private readonly Settings settings;

        /// <summary>
        /// Konstruktor tridy s udanim instance nastaveni aplikace
        /// </summary>
        /// <param name="settings">instance nastaveni aplikace</param>
        public DownloadItems(Settings settings)
        {
            list = new List<DownloadItem>();
            this.settings = settings;
        }

        /// <summary>
        /// Vraci pocet polozek v seznamu
        /// </summary>
        public int Count
        {
            get { return list.Count; }
        }

        #region Events

        /// <summary>
        /// Handler pro udalost, ktera nastane pri zmene stavu polozky (napriklad pri stahovani) 
        /// </summary>
        public event EventHandler StatusChanged;

        /// <summary>
        /// Handler pro udalost, ktera nastane pri zmene stavu stahovani v Download tride
        /// </summary>
        public event EventHandler DownloadStatusChanged;

        /// <summary>
        /// Handler pro udalost, ktera nastane pri zmene poctu stahnutych dat 
        /// </summary>
        public event EventHandler PositionChanged;

        /// <summary>
        /// Handler pro udalost, ktera nastane pri dokonceni stahovani
        /// </summary>
        public event EventHandler DownloadComplete;

        /// <summary>
        /// Handler pro udalost, ktera nastane pri neuspesnem stahnuti
        /// </summary>
        public event EventHandler DownloadFailed;

        #endregion

        #region IDisposable Members

        ///<summary>
        ///Uvolni vsechny polozky v seznamu (smaze je)
        ///</summary>
        public void Dispose()
        {
            foreach (DownloadItem di in list)
                di.Dispose();
            list.Clear();
        }

        #endregion

        /// <summary>
        /// Vykonna zadanou akci na vsech polozkach
        /// </summary>
        /// <param name="action">handler akce</param>
        public void ForEach(Action<DownloadItem> action)
        {
            list.ForEach(action);
        }

        /// <summary>
        /// Vymaze vsechny polozky ze seznamu
        /// </summary>
        public void Clear()
        {
            list.Clear();
        }

        /// <summary>
        /// Vraci pozici polozky v seznamu
        /// </summary>
        /// <param name="item"></param>
        /// <returns></returns>
        public int IndexOf(DownloadItem item)
        {
            return list.IndexOf(item);
        }

        #region Move methods

        /// <summary>
        /// Posouva polozku na indexu from na pozici v seznamu zadanou v to
        /// </summary>
        /// <param name="from">index polozky, kterou chceme presunout</param>
        /// <param name="to">index, kam bude polozka presunuta</param>
        public void Move(int from, int to)
        {
            if (from < 0 || from > Count - 1 || to < 0 || to > Count - 1)
                throw new IndexOutOfRangeException();

            DownloadItem di = this[from];
            list.Remove(this[from]);
            list.Insert(to, di);
        }

        /// <summary>
        /// Posouva zadanou polozku na pozici v seznamu zadanou v to
        /// </summary>
        /// <param name="di">polozka, ktera bude presunuta</param>
        /// <param name="to">index, kam bude polozka presunuta</param>
        public void Move(DownloadItem di, int to)
        {
            int from = IndexOf(di);
            Move(from, to);
        }

        /// <summary>
        /// Posune polozku na zadane pozici o jednu pozici vyse
        /// </summary>
        /// <param name="index">index polozky, kterou chceme presunout</param>
        public void MoveUp(int index)
        {
            if (index <= 0) return;
            Move(index, index - 1);
        }

        /// <summary>
        /// Posune zadanou polozku o jednu pozici vyse
        /// </summary>
        /// <param name="di">polozka, ktera bude presunuta</param>
        public void MoveUp(DownloadItem di)
        {
            MoveUp(IndexOf(di));
        }

        /// <summary>
        /// Posune polozku na zadane pozici o jednu pozici nize
        /// </summary>
        /// <param name="index">index polozky, kterou chceme presunout</param>
        public void MoveDown(int index)
        {
            if (index >= Count - 1) return;
            Move(index, index + 1);
        }

        /// <summary>
        /// Posune zadanou polozku o jednu pozici nize
        /// </summary>
        /// <param name="di">polozka, ktera bude presunuta</param>
        public void MoveDown(DownloadItem di)
        {
            MoveDown(IndexOf(di));
        }

        #endregion

        #region Exist methods

        /// <summary>
        /// Vraci true, pokud zadana polozka v seznamu existuje
        /// </summary>
        /// <param name="item">polozka, kterou chceme otestovat</param>
        /// <returns>true, pokud polozka v seznamu existuje, jinak false</returns>
        public bool Exist(DownloadItem item)
        {
            return list.IndexOf(item) != -1;
        }

        /// <summary>
        /// Vraci true, pokud zadane url polozky v seznamu existuje
        /// </summary>
        /// <param name="url">url polozky</param>
        /// <returns>true, pokud url polozky v seznamu existuje, jinak false</returns>
        public bool Exist(string url)
        {
            return list.Exists(delegate(DownloadItem di) { return di.Url == url; });
        }

        #endregion

        #region Events propagation methods

        private void OnStatusChanged(object sender, EventArgs e)
        {
            //predame dal
            if (StatusChanged != null) StatusChanged(sender, e);
        }

        private void OnDownloadStatusChanged(object sender, EventArgs e)
        {
            //predame dal
            if (DownloadStatusChanged != null) DownloadStatusChanged(sender, e);
        }

        private void OnPositionChanged(object sender, EventArgs e)
        {
            //predame dal
            if (PositionChanged != null) PositionChanged(sender, e);
        }

        private void OnDownloadComplete(object sender, EventArgs e)
        {
            //predame dal
            if (DownloadComplete != null) DownloadComplete(sender, e);
        }

        private void OnDownloadFailed(object sender, EventArgs e)
        {
            //predame dal
            if (DownloadFailed != null) DownloadFailed(sender, e);
        }

        #endregion

        #region Indexer

        /// <summary>
        /// Vraci polozku na zadanem indexu
        /// </summary>
        /// <param name="index">index polozky, kterou chceme ziskat</param>
        /// <returns>polozka na zadanem indexu</returns>
        public DownloadItem this[int index]
        {
            get { return list[index]; }
            //set { list[index] = value; }
        }

        #endregion

        #region Add methods

        /// <summary>
        /// Prida polozku zadanou prostrednictvim url
        /// </summary>
        /// <param name="url">url polozky</param>
        /// <returns>objekt nove pridane polozky</returns>
        public DownloadItem Add(string url)
        {
            return Add(new DownloadItem(url));
        }

        /// <summary>
        /// Prida polozku zadanou prostrednictvim url a nastavi dalsi hodnoty polozky
        /// </summary>
        /// <param name="url">url polozky</param>
        /// <param name="password">heslo polozky</param>
        /// <param name="newFilename">novy nazev polozky</param>
        /// <param name="description">popis polozky</param>
        /// <returns>objekt nove pridane polozky</returns>
        public DownloadItem Add(string url, string password, string newFilename, string description)
        {
            DownloadItem i = Add(url);
            if (i == null) return null;
            i.Password = password;
            i.NewFilename = newFilename;
            i.Description = description;

            return i;
        }

        /// <summary>
        /// Prida zadanou polozku do seznamu
        /// </summary>
        /// <param name="item"></param>
        /// <returns>objekt nove pridane polozky</returns>
        public DownloadItem Add(DownloadItem item)
        {
            if (list.IndexOf(item) != -1) return null;

            item.StatusChanged += OnStatusChanged;
            item.DownloadStatusChanged += OnDownloadStatusChanged;
            item.PositionChanged += OnPositionChanged;
            item.DownloadComplete += OnDownloadComplete;
            item.DownloadFailed += OnDownloadFailed;

            list.Add(item);

            return item;
        }

        #endregion

        #region Remove methods

        /// <summary>
        /// Odstani polozku z prislusneho indexu
        /// </summary>
        /// <param name="item">index polozky</param>
        public void Remove(int item)
        {
            if (item > Count - 1) throw new IndexOutOfRangeException();
            Remove(list[item]);
        }

        /// <summary>
        /// Ostrani zadanou polozku ze seznamu
        /// </summary>
        /// <param name="item">polozka</param>
        public void Remove(DownloadItem item)
        {
            //odstraneni prirazeni udalosti
            item.StatusChanged -= OnStatusChanged;
            item.DownloadStatusChanged -= OnDownloadStatusChanged;
            item.PositionChanged -= OnPositionChanged;
            item.DownloadComplete -= OnDownloadComplete;
            item.DownloadFailed -= OnDownloadFailed;

            item.Dispose();

            list.Remove(item);
        }

        #endregion

        #region Load/Save methods

        /// <summary>
        /// Nacte senzam polozek z nastaveni aplikace
        /// </summary>
        public void LoadList()
        {
            list.Clear();
            XmlNodeList nl = settings.getNodes("osd/files/file");
            //projedeme vsechny polozky
            foreach (XmlNode n in nl)
            {
                DownloadItemStatus status;
                string textStatus = "";
                if (n.Attributes["status"] != null) textStatus = n.Attributes["status"].Value.ToLower();
                //dle status nastavime stah stahovani
                switch (textStatus)
                {
                    case "downloaded":
                        status = DownloadItemStatus.Downloaded;
                        break;
                    case "paused":
                        status = DownloadItemStatus.Paused;
                        break;
                    case "deleted":
                        status = DownloadItemStatus.Deleted;
                        break;
                    default:
                        status = DownloadItemStatus.Queued;
                        break;
                }

                // Nacteme dalsi nastaveni
                DownloadItem di = new DownloadItem(n.SelectSingleNode("url").InnerText, status);
                if (n.SelectSingleNode("password") != null)
                    di.Password = n.SelectSingleNode("password").InnerText;
                if (n.SelectSingleNode("new_filename") != null)
                    di.NewFilename = n.SelectSingleNode("new_filename").InnerText;
                if (n.SelectSingleNode("description") != null)
                    di.Description = n.SelectSingleNode("description").InnerText;

                //pridame nocou polozku
                Add(di);
            }
        }

        /// <summary>
        /// Ulozi seznam polozek do nastaveni aplikace
        /// </summary>
        public void SaveList()
        {
            XmlNode node;

            //smazeme stary Node vcetne jeho potomku
            XmlNodeList nodeList = settings.getNodes("osd/files");
            if (nodeList.Count != 0)
            {
                node = settings.getNode("osd/files");
                node.ParentNode.RemoveChild(node);
            }

            // vytvorime "files" Node
            XmlNode filesNode = settings.getNode("osd").AppendChild(settings.XmlDocument.CreateElement("files"));

            foreach (DownloadItem item in this)
            {
                XmlElement fileElement = settings.XmlDocument.CreateElement("file");

                switch (item.Status)
                {
                    case DownloadItemStatus.Downloaded:
                        fileElement.SetAttribute("status", "downloaded");
                        break;
                    case DownloadItemStatus.Deleted:
                        fileElement.SetAttribute("status", "deleted");
                        break;
                    case DownloadItemStatus.Paused:
                        fileElement.SetAttribute("status", "paused");
                        break;
                    default:
                        break;
                }

                XmlNode fileNode = filesNode.AppendChild(fileElement);

                fileNode.AppendChild(settings.XmlDocument.CreateElement("url")).InnerText = item.Url;

                if (item.Password.Length > 0)
                    fileNode.AppendChild(settings.XmlDocument.CreateElement("password")).InnerText = item.Password;
                if (item.NewFilename.Length > 0)
                    fileNode.AppendChild(settings.XmlDocument.CreateElement("new_filename")).InnerText =
                        item.NewFilename;
                if (item.Description.Length > 0)
                    fileNode.AppendChild(settings.XmlDocument.CreateElement("description")).InnerText = item.Description;
            }
            settings.Save();
        }

        #endregion

        #region Enumerator

        ///<summary>
        ///Returns an enumerator that iterates through the collection.
        ///</summary>
        ///
        ///<returns>
        ///A <see cref="T:System.Collections.Generic.IEnumerator`1"></see> that can be used to iterate through the collection.
        ///</returns>
        ///<filterpriority>1</filterpriority>
        IEnumerator<DownloadItem> IEnumerable<DownloadItem>.GetEnumerator()
        {
            return list.GetEnumerator();
        }

        ///<summary>
        ///Returns an enumerator that iterates through a collection.
        ///</summary>
        ///
        ///<returns>
        ///An <see cref="T:System.Collections.IEnumerator"></see> object that can be used to iterate through the collection.
        ///</returns>
        ///<filterpriority>2</filterpriority>
        public IEnumerator GetEnumerator()
        {
            return ((IEnumerable<DownloadItem>) this).GetEnumerator();
        }

        #endregion
    }
}