// $Id: ObjectListView.cs 25 2008-04-20 14:51:40Z arcao $

/*
 * ObjectListView - A listview to show various aspects of a collection of objects
 *
 * Author: Phillip Piper
 * Date: 9/10/2006 11:15 AM
 *
 * Change log:
 * 2008-01-29  JPP  - Fixed bug that caused owner-drawn virtual lists to use 100% CPU 
 *                  - Added FlagRenderer to help draw bitwise-OR'ed flag values
 * 2008-01-23  JPP  - Fixed bug (introduced in v1.9) that made alternate row colour with groups not quite right
 *                  - Ensure that DesignerSerializationVisibility.Hidden is set on all non-browsable properties
 *                  - Make sure that sort indicators are shown after changing which columns are visible
 * 2008-01-21  JPP  - Added FastObjectListView
 * v1.9
 * 2008-01-18  JPP  - Added IncrementalUpdate()
 * 2008-01-16  JPP  - Right clicking on column header will allow the user to choose which columns are visible.
 *                    Set SelectColumnsOnRightClick to false to prevent this behaviour.
 *                  - Added ImagesRenderer to draw more than one images in a column
 *                  - Changed the positioning of the empty list msg to use all the client area. Thanks to Matze.
 * 2007-12-13  JPP  - Added CopySelectionToClipboard(). Ctrl-C invokes this method. Supports text 
 *                    and HTML formats.
 * 2007-12-12  JPP  - Added support for checkboxes via CheckStateGetter and CheckStatePutter properties.
 *                  - Made ObjectListView and OLVColumn into partial classes so that others can extend them.
 * 2007-12-09  JPP  - Added ability to have hidden columns, i.e. columns that the ObjectListView knows
 *                    about but that are not visible to the user. Controlled by OLVColumn.IsVisible.
 *                    Added ColumnSelectionForm to the project to show how it could be used in an application.
 * 
 * v1.8
 * 2007-11-26  JPP  - Cell editing fully functional
 * 2007-11-21  JPP  - Added SelectionChanged event. This event is triggered once when the 
 *                    selection changes, no matter how many items are selected or deselected (in
 *                    contrast to SelectedIndexChanged which is called once for every row that
 *                    is selected or deselected). Thanks to lupokehl42 (Daniel) for his suggestions and 
 *                    improvements on this idea.
 * 2007-11-19  JPP  - First take at cell editing
 * 2007-11-17  JPP  - Changed so that items within a group are not sorted if lastSortOrder == None
 *                  - Only call MakeSortIndicatorImages() if we haven't already made the sort indicators
 *                    (Corrected misspelling in the name of the method too)
 * 2007-11-06  JPP  - Added ability to have secondary sort criteria when sorting 
 *                    (SecondarySortColumn and SecondarySortOrder properties)
 *                  - Added SortGroupItemsByPrimaryColumn to allow group items to be sorted by the
 *                    primary column. Previous default was to sort by the grouping column.
 * v1.7
 * No big changes to this version but made to work with ListViewPrinter and released with it.
 * 
 * 2007-11-05  JPP  - Changed BaseRenderer to use DrawString() rather than TextRenderer, since TextRenderer
 *                    does not work when printing. 
 * v1.6
 * 2007-11-03  JPP  - Fixed some bugs in the rebuilding of DataListView.
 * 2007-10-31  JPP  - Changed to use builtin sort indicators on XP and later. This also avoids alignment
 *                    problems on Vista. (thanks to gravybod for the suggestion and example implementation)
 * 2007-10-21  JPP  - Added MinimumWidth and MaximumWidth properties to OLVColumn. 
 *                  - Added ability for BuildList() to preserve selection. Calling BuildList() directly
 *                    tries to preserve selection; calling SetObjects() does not.
 *                  - Added SelectAll() and DeselectAll() methods. Useful for working with large lists.
 * 2007-10-08  JPP  - Added GetNextItem() and GetPreviousItem(), which walk sequentially through the
 *                    listview items, even when the view is grouped.
 *                  - Added SelectedItem property
 * 2007-09-28  JPP  - Optimized aspect-to-string conversion. BuildList() 15% faster.
 *                  - Added empty implementation of RefreshObjects() to VirtualObjectListView since
 *                    RefreshObjects() cannot work on virtual lists. 
 * 2007-09-13  JPP  - Corrected bug with custom sorter in VirtualObjectListView (thanks for mpgjunky)
 * 2007-09-07  JPP  - Corrected image scaling bug in DrawAlignedImage() (thanks to krita970)
 * 2007-08-29  JPP  - Allow item count labels on groups to be set per column (thanks to cmarlow for idea)
 * 2007-08-14  JPP  - Major rework of DataListView based on Ian Griffiths's great work
 * 2007-08-11  JPP  - When empty, the control can now draw a "List Empty" message
 *                  - Added GetColumn() and GetItem() methods
 * v1.5
 * 2007-08-03  JPP  - Support animated GIFs in ImageRenderer
 *                  - Allow height of rows to be specified - EXPERIMENTAL!
 * 2007-07-26  JPP  - Optimised redrawing of owner-drawn lists by remembering the update rect
 *                  - Allow sort indicators to be turned off
 * 2007-06-30  JPP  - Added RowFormatter delegate
 *                  - Allow a different label when there is only one item in a group (thanks to cmarlow)
 * v1.4
 * 2007-04-12  JPP  - Allow owner drawn on steriods!
 *                  - Column headers now display sort indicators
 *                  - ImageGetter delegates can now return ints, strings or Images 
 *                    (Images are only visible if the list is owner drawn)
 *                  - Added OLVColumn.MakeGroupies to help with group partitioning
 *                  - All normal listview views are now supported
 *                  - Allow dotted aspect names, e.g. Owner.Workgroup.Name (thanks to OlafD)
 *                  - Added SelectedObject and SelectedObjects properties
 * v1.3
 * 2007-03-01  JPP  - Added DataListView
 *                  - Added VirtualObjectListView
 * 					- Added Freeze/Unfreeze capabilities
 *                  - Allowed sort handler to be installed
 *                  - Simplified sort comparisons: handles 95% of cases with only 6 lines of code!
 *                  - Fixed bug with alternative line colors on unsorted lists (thanks to cmarlow)
 * 2007-01-13  JPP  - Fixed bug with lastSortOrder (thanks to Kwan Fu Sit)
 *                  - Non-OLVColumns are no longer allowed
 * 2007-01-04  JPP  - Clear sorter before rebuilding list. 10x faster! (thanks to aaberg)
 *                  - Include GetField in GetAspectByName() so field values can be Invoked too.
 * 					- Fixed subtle bug in RefreshItem() that erased background colors.
 * 2006-11-01  JPP  - Added alternate line colouring
 * 2006-10-20  JPP  - Refactored all sorting comparisons and made it extendable. See ComparerManager.
 *                  - Improved IDE integration
 *                  - Made control DoubleBuffered
 *                  - Added object selection methods
 * 2006-10-13  JPP  Implemented grouping and column sorting
 * 2006-10-09  JPP  Initial version
 * 
 * CONDITIONS OF USE
 * This code may be freely used for non-commercial purposes. Commerical use requires a one-time
 * license fee obtainable from phillip_piper@bigfoot.com. In all cases, this code must be kept intact, 
 * complete with this header and conditions of use.
 * 
 * Copyright (C) 2006-2008 Phillip Piper
 * 
 * TO DO:
 * - Incremental update. List has changed slightly (a couple of new items, a couple of items removed)
 *   Do this efficiently. Maintain sort order, scroll position, everything.
 * - Extend MappedImageRender to be able to draw more than image if its Aspect returns an ICollection.
 */

using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Design;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Timer=System.Threading.Timer;

namespace BrightIdeasSoftware
{
    /// <summary>
    /// An object list displays 'aspects' of a collection of objects in a listview control.
    /// </summary>
    /// <remarks>
    /// <para>
    /// The intelligence for this control is in the columns. OLVColumns are
    /// extended so they understand how to fetch an 'aspect' from each row
    /// object. They also understand how to sort by their aspect, and
    /// how to group them.
    /// </para>
    /// <para>
    /// Aspects are extracted by giving the name of a method to be called or a
    /// property to be fetched. These names can be simple names or they can be dotted
    /// to chain property access e.g. "Owner.Address.Postcode". 
    /// Aspects can also be extracted by installing a delegate.
    /// </para>
    /// <para>
    /// Sorting by column clicking and grouping by column are handled automatically.
    /// </para>
    /// <para>
    /// This list puts sort indicators in the column headers to show the column sorting direction.
    /// If you wish to replace the standard images with your own images, put entries in the small image list
    /// with the key values "sort-indicator-up" and "sort-indicator-down".
    /// </para>
    /// <para>
    /// For these classes to build correctly, the project must have references to these assemblies:
    /// <list>
    /// <item>System.Data</item>
    /// <item>System.Design</item>
    /// <item>System.Drawing</item>
    /// <item>System.Windows.Forms (obviously)</item>
    /// </list>
    /// </para>
    /// </remarks>
    public partial class ObjectListView : ListView, ISupportInitialize
    {
        private Color alternateRowBackColor = Color.Empty; // what color background should alternate lines have?
        private SortDelegate customSorter; // callback for handling custom sort by column processing

        private string groupWithItemCountFormat;
        // when a group title has an item count, how should the label be formatted?

        private string groupWithItemCountSingularFormat;
        // when a group title has an item count of 1, how should the label be formatted?

        protected OLVColumn lastSortColumn; // which column did we last sort by
        protected SortOrder lastSortOrder; // which direction did we last sort
        private Rectangle lastUpdateRectangle; // remember the update rect from the last WM_PAINT msg
        private IEnumerable objects; // the collection of objects on show
        private bool showImagesOnSubItems; // should we try to show images on subitems?
        private bool showItemCountOnGroups; // should we show items count in group labels?
        private bool showSortIndicators; // should we show sort indicators in the column headers?
        private bool useAlternatingBackColors; // should we use different colors for alternate lines?

        /// <summary>
        /// Create an ObjectListView
        /// </summary>
        public ObjectListView() : base()
        {
            ColumnClick += new ColumnClickEventHandler(HandleColumnClick);
            ItemCheck += new ItemCheckEventHandler(HandleItemCheck);

            base.View = View.Details;
            DoubleBuffered = true; // kill nasty flickers. hiss... me hates 'em
            AlternateRowBackColor = Color.Empty;
            ShowSortIndicators = true;
        }

        #region Public properties

        /// <summary>
        /// How does a user indicate that they want to edit cells?
        /// </summary>
        public enum CellEditActivateMode
        {
            /// <summary>
            /// This list cannot be edited. F2 does nothing.
            /// </summary>
            None = 0,

            /// <summary>
            /// A single click on  a <strong>subitem</strong> will edit the value. Single clicking the primary column,
            /// selects the row just like normal. The user must press F2 to edit the primary column.
            /// </summary>
            SingleClick = 1,

            /// <summary>
            /// Double clicking a subitem or the primary column will edit that cell.
            /// F2 will edit the primary column.
            /// </summary>
            DoubleClick = 2,

            /// <summary>
            /// Pressing F2 is the only way to edit the cells. Once the primary column is being edited,
            /// the other cells in the row can be edited by pressing Tab.
            /// </summary>
            F2Only = 3
        }

        private List<OLVColumn> allColumns = new List<OLVColumn>();
        private CellEditActivateMode cellEditActivation = CellEditActivateMode.None;
        private String emptyListMsg = "";
        private Font emptyListMsgFont;
        private int freezeCount = 0;
        private int rowHeight = -1;
        private OLVColumn secondarySortColumn;
        private SortOrder secondarySortOrder = SortOrder.Ascending;
        private bool selectColumnsMenuStaysOpen = true;
        private bool selectColumnsOnRightClick = true;
        private ImageList shadowedImageList = null;
        private bool sortGroupItemsByPrimaryColumn = true;

        /// <summary>
        /// Get/set the collection of objects that this list will show
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public IEnumerable Objects
        {
            get { return objects; }
            set { SetObjects(value); }
        }

        /// <summary>
        /// Get or set all the columns that this control knows about. 
        /// Only those columns where IsVisible is true will be seen by the user.
        /// </summary>
        /// <remarks>If you want to add new columns programmatically, add them to
        /// AllColumns and then call RebuildColumns(). Normally, you do not have to
        /// deal with this property directly. Just use the IDE.</remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public List<OLVColumn> AllColumns
        {
            get
            {
                // If someone has wiped out the columns, put the list back
                if (allColumns == null)
                    allColumns = new List<OLVColumn>();

                // If we don't know the columns, use the columns from the control.
                // This handles legacy cases
                if (allColumns.Count == 0 && Columns.Count > 0)
                    for (int i = 0; i < Columns.Count; i++) allColumns.Add(GetColumn(i));
                return allColumns;
            }
            set { allColumns = value; }
        }

        /// <summary>
        /// Return the visible columns in the order they are displayed to the user
        /// </summary>
        [Browsable(false)]
        public List<OLVColumn> ColumnsInDisplayOrder
        {
            get
            {
                List<OLVColumn> columnsInDisplayOrder = new List<OLVColumn>(Columns.Count);
                for (int i = 0; i < Columns.Count; i++)
                    columnsInDisplayOrder.Add(null);
                for (int i = 0; i < Columns.Count; i++)
                {
                    OLVColumn col = GetColumn(i);
                    columnsInDisplayOrder[col.DisplayIndex] = col;
                }
                return columnsInDisplayOrder;
            }
        }

        /// <summary>
        /// If there are no items in this list view, what message should be drawn onto the control?
        /// </summary>
        [Category("Appearance"), Description("When the list has no items, show this message in the control"),
         DefaultValue("")
        ]
        public String EmptyListMsg
        {
            get { return emptyListMsg; }
            set
            {
                if (emptyListMsg != value)
                {
                    emptyListMsg = value;
                    Invalidate();
                }
            }
        }

        /// <summary>
        /// What font should the 'list empty' message be drawn in?
        /// </summary>
        [Category("Appearance"), Description("What font should the 'list empty' message be drawn in?"),
         DefaultValue(null)]
        public Font EmptyListMsgFont
        {
            get { return emptyListMsgFont; }
            set { emptyListMsgFont = value; }
        }

        /// <summary>
        /// Return the font for the 'list empty' message or a default 
        /// </summary>
        [Browsable(false)]
        public Font EmptyListMsgFontOrDefault
        {
            get
            {
                if (EmptyListMsgFont == null)
                    return new Font("Tahoma", 14);
                else
                    return EmptyListMsgFont;
            }
        }

        /// <summary>
        /// Does this listview have a message that should be drawn when the list is empty?
        /// </summary>
        [Browsable(false)]
        public bool HasEmptyListMsg
        {
            get { return !String.IsNullOrEmpty(EmptyListMsg); }
        }

        /// <summary>
        /// Should the list view show images on subitems?
        /// </summary>
        /// <remarks>
        /// <para>Under Windows, this works by sending messages to the underlying
        /// Windows control. To make this work under Mono, we would have to owner drawing the items :-(</para></remarks>
        [Category("Behavior"), Description("Should the list view show images on subitems?"), DefaultValue(false)]
        public bool ShowImagesOnSubItems
        {
            get { return showImagesOnSubItems; }
            set { showImagesOnSubItems = value; }
        }

        /// <summary>
        /// This property controls whether group labels will be suffixed with a count of items.
        /// </summary>
        /// <remarks>
        /// The format of the suffix is controlled by GroupWithItemCountFormat/GroupWithItemCountSingularFormat properties
        /// </remarks>
        [Category("Behavior"), Description("Will group titles be suffixed with a count of the items in the group?"),
         DefaultValue(false)]
        public bool ShowItemCountOnGroups
        {
            get { return showItemCountOnGroups; }
            set { showItemCountOnGroups = value; }
        }

        /// <summary>
        /// When a group title has an item count, how should the lable be formatted?
        /// </summary>
        /// <remarks>
        /// The given format string can/should have two placeholders:
        /// <list type="bullet">
        /// <item>{0} - the original group title</item>
        /// <item>{1} - the number of items in the group</item>
        /// </list>
        /// </remarks>
        /// <example>"{0} [{1} items]"</example>
        [Category("Behavior"), Description("The format to use when suffixing item counts to group titles"),
         DefaultValue(null)
        ]
        public string GroupWithItemCountFormat
        {
            get { return groupWithItemCountFormat; }
            set { groupWithItemCountFormat = value; }
        }

        /// <summary>
        /// Return this.GroupWithItemCountFormat or a reasonable default
        /// </summary>
        [Browsable(false)]
        public string GroupWithItemCountFormatOrDefault
        {
            get
            {
                if (String.IsNullOrEmpty(GroupWithItemCountFormat))
                    return "{0} [{1} items]";
                else
                    return GroupWithItemCountFormat;
            }
        }

        /// <summary>
        /// When a group title has an item count, how should the lable be formatted if
        /// there is only one item in the group?
        /// </summary>
        /// <remarks>
        /// The given format string can/should have two placeholders:
        /// <list type="bullet">
        /// <item>{0} - the original group title</item>
        /// <item>{1} - the number of items in the group (always 1)</item>
        /// </list>
        /// </remarks>
        /// <example>"{0} [{1} item]"</example>
        [Category("Behavior"), Description("The format to use when suffixing item counts to group titles"),
         DefaultValue(null)
        ]
        public string GroupWithItemCountSingularFormat
        {
            get { return groupWithItemCountSingularFormat; }
            set { groupWithItemCountSingularFormat = value; }
        }

        /// <summary>
        /// Return this.GroupWithItemCountSingularFormat or a reasonable default
        /// </summary>
        [Browsable(false)]
        public string GroupWithItemCountSingularFormatOrDefault
        {
            get
            {
                if (String.IsNullOrEmpty(GroupWithItemCountSingularFormat))
                    return "{0} [{1} item]";
                else
                    return GroupWithItemCountSingularFormat;
            }
        }

        /// <summary>
        /// Should the list give a different background color to every second row?
        /// </summary>
        /// <remarks><para>The color of the alternate rows is given by AlternateRowBackColor.</para>
        /// <para>There is a "feature" in .NET for listviews in non-full-row-select mode, where
        /// selected rows are not drawn with their correct background color.</para></remarks>
        [Category("Appearance"), Description("Should the list view use a different backcolor to alternate rows?"),
         DefaultValue(false)]
        public bool UseAlternatingBackColors
        {
            get { return useAlternatingBackColors; }
            set { useAlternatingBackColors = value; }
        }

        /// <summary>
        /// Should the list view show a bitmap in the column header to show the sort direction?
        /// </summary>
        [Category("Behavior"), Description("Should the list view show sort indicators in the column headers?"),
         DefaultValue(true)]
        public bool ShowSortIndicators
        {
            get { return showSortIndicators; }
            set { showSortIndicators = value; }
        }

        /// <summary>
        /// If every second row has a background different to the control, what color should it be?
        /// </summary>
        [Category("Appearance"), Description("If using alternate colors, what color should alterate rows be?"),
         DefaultValue(typeof (Color), "Empty")]
        public Color AlternateRowBackColor
        {
            get { return alternateRowBackColor; }
            set { alternateRowBackColor = value; }
        }

        /// <summary>
        /// Return the alternate row background color that has been set, or the default color
        /// </summary>
        [Browsable(false)]
        public Color AlternateRowBackColorOrDefault
        {
            get
            {
                if (alternateRowBackColor == Color.Empty)
                    return Color.LemonChiffon;
                else
                    return alternateRowBackColor;
            }
        }

        /// <summary>
        /// Get or set whether or not the listview is frozen. When the listview is
        /// frozen, it will not update itself.
        /// </summary>
        /// <remarks><para>The Frozen property is similar to the methods Freeze()/Unfreeze()
        /// except that changes to the Frozen property do not nest.</para></remarks>
        /// <example>objectListView1.Frozen = false; // unfreeze the control regardless of the number of Freeze() calls
        /// </example>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool Frozen
        {
            get { return freezeCount > 0; }
            set
            {
                if (value)
                    Freeze();
                else if (freezeCount > 0)
                {
                    freezeCount = 1;
                    Unfreeze();
                }
            }
        }

        /// <summary>
        /// Get/set the list of columns that should be used when the list switches to tile view.
        /// </summary>
        /// <remarks>If no list of columns has been installed, this value will default to the
        /// first column plus any column where IsTileViewColumn is true.</remarks>
        [Browsable(false), Obsolete("Use GetFilteredColumns() and OLVColumn.IsTileViewColumn instead"),
         DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public List<OLVColumn> ColumnsForTileView
        {
            get { return GetFilteredColumns(View.Tile); }
        }

        /// <summary>
        /// Get the ListViewItem that is currently selected . If no row is selected, or more than one is selected, return null.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ListViewItem SelectedItem
        {
            get
            {
                if (SelectedIndices.Count == 1)
                    return GetItem(SelectedIndices[0]);
                else
                    return null;
            }
            set
            {
                SelectedIndices.Clear();
                if (value != null)
                    SelectedIndices.Add(value.Index);
            }
        }

        /// <summary>
        /// Get the model object from the currently selected row. If no row is selected, or more than one is selected, return null.
        /// Select the row that is displaying the given model object. All other rows are deselected.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public Object SelectedObject
        {
            get { return GetSelectedObject(); }
            set { SelectObject(value); }
        }

        /// <summary>
        /// Get the model objects from the currently selected rows. If no row is selected, the returned List will be empty.
        /// When setting this value, select the rows that is displaying the given model objects. All other rows are deselected.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ArrayList SelectedObjects
        {
            get { return GetSelectedObjects(); }
            set { SelectObjects(value); }
        }

        /// <summary>
        /// Get/set the style of view that this listview is using
        /// </summary>
        /// <remarks>Switching to tile or details view installs the columns appropriate to that view.
        /// Confusingly, in tile view, every column is shown as a row of information.</remarks>
        public new View View
        {
            get { return base.View; }
            set
            {
                if (base.View == value)
                    return;

                if (Frozen)
                {
                    base.View = value;
                    return;
                }

                Freeze();

                // If we are switching to a Detail or Tile view, setup the columns needed for that view
                if (value == View.Details || value == View.Tile)
                {
                    ChangeToFilteredColumns(value);

                    if (value == View.Tile)
                        CalculateReasonableTileSize();
                }

                base.View = value;
                Unfreeze();
            }
        }

        /// <summary>
        /// Specify the height of each row in the control in pixels.
        /// </summary>
        /// <remarks><para>The row height in a listview is normally determined by the font size and the small image list size.
        /// This setting allows that calculation to be overridden (within reason: you still cannot set the line height to be 
        /// less than the line height of the font used in the control). </para>
        /// <para>Setting it to -1 means use the normal calculation method.</para>
        /// <para><bold>This feature is experiemental!</bold> Strange things may happen to your program, your spouse or your pet if you use it.</para>
        /// </remarks>
        [Category("Appearance"), DefaultValue(-1)]
        public int RowHeight
        {
            get { return rowHeight; }
            set
            {
                if (value < 1)
                    rowHeight = -1;
                else
                    rowHeight = value;
                SetupExternalImageList();
            }
        }

        /// <summary>
        /// Override the SmallImageList property so we can correctly shadow its operations.
        /// </summary>
        /// <remarks><para>If you use the <code>RowHeight</code> property to specify the row height, the SmallImageList
        /// must be fully initialised before setting/changing the RowHeight. If you add new images to the image
        /// list after setting the RowHeight, you must assign the imagelist to the control again. Something as simple 
        /// as this will work:</para>
        /// <para><code>listView1.SmallImageList = listView1.SmallImageList;</code></para>
        /// </remarks>
        public new ImageList SmallImageList
        {
            get { return shadowedImageList; }
            set
            {
                shadowedImageList = value;
                SetupExternalImageList();
            }
        }

        /// <summary>
        /// Give access to the image list that is actually being used by the control
        /// </summary>
        [Browsable(false)]
        public ImageList BaseSmallImageList
        {
            get { return base.SmallImageList; }
        }

        /// <summary>
        /// Get/set the column that will be used to resolve comparisons that are equal when sorting.
        /// </summary>
        /// <remarks>There is no user interface for this setting. It must be set programmatically.
        /// The default is the first column.</remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public OLVColumn SecondarySortColumn
        {
            get
            {
                if (secondarySortColumn == null)
                {
                    if (Columns.Count > 0)
                        return GetColumn(0);
                    else
                        return null;
                }
                else
                    return secondarySortColumn;
            }
            set { secondarySortColumn = value; }
        }

        /// <summary>
        /// When the SecondarySortColumn is used, in what order will it compare results?
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public SortOrder SecondarySortOrder
        {
            get { return secondarySortOrder; }
            set { secondarySortOrder = value; }
        }

        /// <summary>
        /// When the listview is grouped, should the items be sorted by the primary column?
        /// If this is false, the items will be sorted by the same column as they are grouped.
        /// </summary>
        [Category("Behavior"),
         Description(
             "When the listview is grouped, should the items be sorted by the primary column? If this is false, the items will be sorted by the same column as they are grouped."
             ), DefaultValue(true)]
        public bool SortGroupItemsByPrimaryColumn
        {
            get { return sortGroupItemsByPrimaryColumn; }
            set { sortGroupItemsByPrimaryColumn = value; }
        }

        /// <summary>
        /// How does the user indicate that they want to edit a cell? 
        /// None means that the listview cannot be edited.
        /// </summary>
        /// <remarks>Columns can also be marked as editable.</remarks>
        [Category("Behavior"), Description("How does the user indicate that they want to edit a cell?"),
         DefaultValue(CellEditActivateMode.None)]
        public CellEditActivateMode CellEditActivation
        {
            get { return cellEditActivation; }
            set { cellEditActivation = value; }
        }

        /// <summary>
        /// Return true if a cell edit operation is currently happening
        /// </summary>
        [Browsable(false)]
        public bool IsCellEditing
        {
            get { return cellEditor != null; }
        }

        /// <summary>
        /// When the user right clicks on the column headers, should a menu be presented which will allow
        /// them to choose which columns will be shown in the view?
        /// </summary>
        [Category("Behavior"),
         Description(
             "When the user right clicks on the column headers, should a menu be presented which will allow them to choose which columns will be shown in the view?"
             ), DefaultValue(true)]
        public bool SelectColumnsOnRightClick
        {
            get { return selectColumnsOnRightClick; }
            set { selectColumnsOnRightClick = value; }
        }

        /// <summary>
        /// When the column select menu is open, should it stay open after an item is selected?
        /// Staying open allows the user to turn more than one column on or off at a time.
        /// </summary>
        [Category("Behavior"),
         Description("When the column select menu is open, should it stay open after an item is selected?"),
         DefaultValue(true)]
        public bool SelectColumnsMenuStaysOpen
        {
            get { return selectColumnsMenuStaysOpen; }
            set { selectColumnsMenuStaysOpen = value; }
        }

        #endregion

        #region Callbacks

        private CheckStateGetterDelegate checkStateGetter;
        private CheckStatePutterDelegate checkStatePutter;
        private RowFormatterDelegate rowFormatter;

        /// <summary>
        /// This delegate can be used to sort the table in a custom fasion.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public SortDelegate CustomSorter
        {
            get { return customSorter; }
            set { customSorter = value; }
        }

        /// <summary>
        /// This delegate can be used to format a OLVListItem before it is added to the control.
        /// </summary>
        /// <remarks>
        /// <para>The model object for the row can be found through the RowObject property of the OLVListItem object.</para>
        /// <para>All subitems normally have the same style as list item, so setting the forecolor on one 
        /// subitem changes the forecolor of all subitems.
        /// To allow subitems to have different attributes, <code>ListViewItem.UseItemStyleForSubItems</code> 
        /// must be set to <code>false</code>.
        /// </para>
        /// <para>If <code>UseAlternatingBackColors</code> is true, the backcolor of the listitem will be calculated
        /// by the control and cannot be controlled by the RowFormatter delegate. In general, trying to use a RowFormatter
        /// when <code>UseAlternatingBackColors</code> is <code>true</code> does not work well.</para></remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public RowFormatterDelegate RowFormatter
        {
            get { return rowFormatter; }
            set { rowFormatter = value; }
        }

        /// <summary>
        /// This delegate will be called whenever the ObjectListView needs to know the check state
        /// of the row associated with a given model object
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public CheckStateGetterDelegate CheckStateGetter
        {
            get { return checkStateGetter; }
            set { checkStateGetter = value; }
        }

        /// <summary>
        /// This delegate will be called whenever the user tries to change the check state 
        /// of a row. The delegate should return the value that the listview should actuall
        /// use, which may be different to the one given to the delegate.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public CheckStatePutterDelegate CheckStatePutter
        {
            get { return checkStatePutter; }
            set { checkStatePutter = value; }
        }

        #endregion

        #region List commands

        /// <summary>
        /// Set the collection of objects that will be shown in this list view.
        /// </summary>
        /// <remarks>The list is updated immediately</remarks>
        /// <param name="collection">The objects to be displayed</param>
        public virtual void SetObjects(IEnumerable collection)
        {
            if (InvokeRequired)
            {
                Invoke(new SetObjectsInvoker(SetObjects), new object[] {collection});
                return;
            }
            objects = collection;
            BuildList(false);
        }

        /// <summary>
        /// Update the list to reflect the contents of the given collection, without affecting
        /// the scrolling position, selection or sort order.
        /// </summary>
        /// <param name="collection">The objects to be displayed</param>
        /// <remarks>
        /// <para>This method is about twice as slow as SetObjects().</para>
        /// <para>This method is experimental -- it may disappear in later versions of the code.</para>
        /// <para>There has to be a better way to do this! JPP 15/1/2008</para>
        /// <para>In most situations, if you need this functionality, use a FastObjectListView instead. JPP 2/2/2008</para>
        /// </remarks>
        public virtual void IncrementalUpdate(IEnumerable collection)
        {
            if (InvokeRequired)
            {
                Invoke(new SetObjectsInvoker(IncrementalUpdate), new object[] {collection});
                return;
            }

            BeginUpdate();

            ListViewItemSorter = null;
            ArrayList previousSelection = SelectedObjects;

            // Replace existing rows, creating new listviewitems if we get to the end of the list
            List<OLVListItem> newItems = new List<OLVListItem>();
            int rowIndex = 0;
            int itemCount = Items.Count;
            foreach (object model in collection)
            {
                if (rowIndex < itemCount)
                {
                    OLVListItem lvi = GetItem(rowIndex);
                    lvi.RowObject = model;
                    RefreshItem(lvi);
                }
                else
                {
                    OLVListItem lvi = new OLVListItem(model);
                    FillInValues(lvi, model);
                    newItems.Add(lvi);
                }
                rowIndex++;
            }

            // Delete any excess rows
            int numRowsToDelete = itemCount - rowIndex;
            for (int i = 0; i < numRowsToDelete; i++)
                Items.RemoveAt(rowIndex);

            Items.AddRange(newItems.ToArray());
            Sort(lastSortColumn);

            SetAllSubItemImages();

            SelectedObjects = previousSelection;

            EndUpdate();

            objects = collection;
        }

        /// <summary>
        /// Build/rebuild all the list view items in the list
        /// </summary>
        public virtual void BuildList()
        {
            BuildList(true);
        }

        /// <summary>
        /// Build/rebuild all the list view items in the list
        /// </summary>
        /// <param name="shouldPreserveSelection">If this is true, the control will try to preserve the selection, 
        /// i.e. all objects that were selected before the call will be selected after the rebuild</param>
        /// <remarks>Use this method in situations were the contents of the list is basically the same
        /// as previously.</remarks>
        public virtual void BuildList(bool shouldPreserveSelection)
        {
            if (Frozen)
                return;

            ArrayList previousSelection = new ArrayList();
            if (shouldPreserveSelection && objects != null) previousSelection = SelectedObjects;

            BeginUpdate();
            Items.Clear();
            ListViewItemSorter = null;

            if (objects != null)
            {
                // Build a list of all our items and then display them. (Building
                // a list and then doing one AddRange is about 10-15% faster than individual adds)
                List<OLVListItem> l = new List<OLVListItem>();
                foreach (object rowObject in objects)
                {
                    OLVListItem lvi = new OLVListItem(rowObject);
                    FillInValues(lvi, rowObject);
                    l.Add(lvi);
                }
                Items.AddRange(l.ToArray());
                SetAllSubItemImages();
                Sort(lastSortColumn);

                if (shouldPreserveSelection) SelectedObjects = previousSelection;
            }

            EndUpdate();
        }

        /// <summary>
        /// Sort the items by the last sort column
        /// </summary>
        public new void Sort()
        {
            Sort(lastSortColumn);
        }

        /// <summary>
        /// Organise the view items into groups, based on the last sort column or the first column
        /// if there is no last sort column
        /// </summary>
        public void BuildGroups()
        {
            BuildGroups(lastSortColumn);
        }

        /// <summary>
        /// Organise the view items into groups, based on the given column
        /// </summary>
        /// <param name="column">The column whose values should be used for sorting.</param>
        public virtual void BuildGroups(OLVColumn column)
        {
            if (column == null)
                column = GetColumn(0);

            Groups.Clear();

            // Getting the Count forces any internal cache of the ListView to be flushed. Without
            // this, iterating over the Items will not work correctly if the ListView handle
            // has not yet been created.
            int dummy = Items.Count;

            // Separate the list view items into groups, using the group key as the descrimanent
            Dictionary<object, List<OLVListItem>> map = new Dictionary<object, List<OLVListItem>>();
            foreach (OLVListItem olvi in Items)
            {
                object key = column.GetGroupKey(olvi.RowObject);
                if (key == null)
                    key = "{null}"; // null can't be used as the key for a dictionary
                if (!map.ContainsKey(key))
                    map[key] = new List<OLVListItem>();
                map[key].Add(olvi);
            }

            // Make a list of the required groups
            List<ListViewGroup> groups = new List<ListViewGroup>();
            foreach (object key in map.Keys)
            {
                ListViewGroup lvg = new ListViewGroup(column.ConvertGroupKeyToTitle(key));
                lvg.Tag = key;
                groups.Add(lvg);
            }

            // Sort the groups
            groups.Sort(new ListViewGroupComparer(lastSortOrder));

            // Put each group into the list view, and give each group its member items.
            // The order of statements is important here:
            // - the header must be calculate before the group is added to the list view,
            //   otherwise changing the header causes a nasty redraw (even in the middle of a BeginUpdate...EndUpdate pair)
            // - the group must be added before it is given items, otherwise an exception is thrown (is this documented?)
            string fmt = column.GroupWithItemCountFormatOrDefault;
            string singularFmt = column.GroupWithItemCountSingularFormatOrDefault;
            ColumnComparer itemSorter =
                new ColumnComparer((SortGroupItemsByPrimaryColumn ? GetColumn(0) : column), lastSortOrder,
                                   SecondarySortColumn,
                                   SecondarySortOrder);
            foreach (ListViewGroup group in groups)
            {
                if (ShowItemCountOnGroups)
                {
                    int count = map[group.Tag].Count;
                    group.Header = String.Format((count == 1 ? singularFmt : fmt), group.Header, count);
                }
                Groups.Add(group);
                // If there is no sort order, don't sort since the sort isn't stable
                if (lastSortOrder != SortOrder.None)
                    map[group.Tag].Sort(itemSorter);
                group.Items.AddRange(map[group.Tag].ToArray());
            }
        }

        /// <summary>
        /// Pause (or unpause) all animations in the list
        /// </summary>
        /// <param name="isPause">true to pause, false to unpause</param>
        public void PauseAnimations(bool isPause)
        {
            for (int i = 0; i < Columns.Count; i++)
            {
                OLVColumn col = GetColumn(i);
                if (col.Renderer is ImageRenderer)
                    ((ImageRenderer) col.Renderer).Paused = isPause;
            }
        }

        /// <summary>
        /// Give the listview a reasonable size of its tiles, based on the number of lines of
        /// information that each tile is going to display.
        /// </summary>
        public void CalculateReasonableTileSize()
        {
            if (Columns.Count <= 0)
                return;

            int imageHeight = (LargeImageList == null ? 16 : LargeImageList.ImageSize.Height);
            int dataHeight = (Font.Height + 1)*Columns.Count;
            int tileWidth = (TileSize.Width == 0 ? 200 : TileSize.Width);
            int tileHeight = Math.Max(TileSize.Height, Math.Max(imageHeight, dataHeight));
            TileSize = new Size(tileWidth, tileHeight);
        }

        /// <summary>
        /// Rebuild this list for the given view
        /// </summary>
        /// <param name="view"></param>
        internal void ChangeToFilteredColumns(View view)
        {
            Freeze();
            Clear();
            List<OLVColumn> cols = GetFilteredColumns(view);
            Columns.AddRange(cols.ToArray());
            if (view == View.Details)
            {
                foreach (OLVColumn x in cols)
                {
                    if (x.LastDisplayIndex == -1 || x.LastDisplayIndex > cols.Count - 1)
                        x.DisplayIndex = cols.Count - 1;
                    else
                        x.DisplayIndex = x.LastDisplayIndex;
                }
                ShowSortIndicator();
            }
            BuildList();
            Unfreeze();
        }

        /// <summary>
        /// Rebuild the columns based upon its current view and column visibility settings
        /// </summary>
        public void RebuildColumns()
        {
            ChangeToFilteredColumns(View);
        }

        /// <summary>
        /// Copy a text and html representation of the selected rows onto the clipboard.
        /// </summary>
        /// <remarks>Be careful when using this with virtual lists. If the user has selected
        /// 10,000,000 rows, this method will faithfully try to copy all of them to the clipboard.
        /// From the user's point of view, your program will appear to have hung.</remarks>
        public void CopySelectionToClipboard()
        {
            if (SelectedIndices.Count == 0)
                return;

            //THINK: Do we want to include something like this?
            //if (this.SelectedIndices.Count > 10000)
            //    return;

            // Build text and html versions of the selection
            StringBuilder sbText = new StringBuilder();
            StringBuilder sbHtml = new StringBuilder("<table>");

            foreach (int i in SelectedIndices)
            {
                OLVListItem row = GetItem(i);
                sbHtml.Append("<tr><td>");
                for (int j = 0; j < row.SubItems.Count; j++)
                {
                    if (j > 0)
                    {
                        sbText.Append("\t");
                        sbHtml.Append("</td><td>");
                    }
                    sbText.Append(row.SubItems[j].Text);
                    sbHtml.Append(row.SubItems[j].Text);
                }
                sbText.AppendLine();
                sbHtml.AppendLine("</td></tr>");
            }
            sbHtml.AppendLine("</table>");

            // Put both the text and html versions onto the clipboard
            DataObject dataObject = new DataObject();
            dataObject.SetText(sbText.ToString(), TextDataFormat.UnicodeText);
            dataObject.SetText(ConvertToHtmlFragment(sbHtml.ToString()), TextDataFormat.Html);
            Clipboard.SetDataObject(dataObject);
        }

        /// <summary>
        /// Convert the fragment of HTML into the Clipboards HTML format. 
        /// </summary>
        /// <remarks>The HTML format is found here http://msdn2.microsoft.com/en-us/library/aa767917.aspx
        /// </remarks>
        /// <param name="fragment">The HTML to put onto the clipboard. It must be valid HTML!</param>
        /// <returns>A string that can be put onto the clipboard and will be recognized as HTML</returns>
        private string ConvertToHtmlFragment(string fragment)
        {
            // Minimal implementation of HTML clipboard format
            string source = "http://www.codeproject.com/KB/list/ObjectListView.aspx";

            const String MARKER_BLOCK =
                "Version:1.0\r\n" + "StartHTML:{0,8}\r\n" + "EndHTML:{1,8}\r\n" + "StartFragment:{2,8}\r\n" +
                "EndFragment:{3,8}\r\n" + "StartSelection:{2,8}\r\n" + "EndSelection:{3,8}\r\n" + "SourceURL:{4}\r\n" +
                "{5}";

            int prefixLength = String.Format(MARKER_BLOCK, 0, 0, 0, 0, source, "").Length;

            const String DEFAULT_HTML_BODY =
                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">" +
                "<HTML><HEAD></HEAD><BODY><!--StartFragment-->{0}<!--EndFragment--></BODY></HTML>";

            string html = String.Format(DEFAULT_HTML_BODY, fragment);
            int startFragment = prefixLength + html.IndexOf(fragment);
            int endFragment = startFragment + fragment.Length;

            return
                String.Format(MARKER_BLOCK, prefixLength, prefixLength + html.Length, startFragment, endFragment, source,
                              html);
        }

        private delegate void SetObjectsInvoker(IEnumerable collection);

        #endregion

        #region Event handlers

        /// <summary>
        /// Event handler for the column click event
        /// </summary>
        protected virtual void HandleColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Toggle the sorting direction on successive clicks on the same column
            if (lastSortColumn != null && e.Column == lastSortColumn.Index)
                lastSortOrder = (lastSortOrder == SortOrder.Descending ? SortOrder.Ascending : SortOrder.Descending);
            else
                lastSortOrder = SortOrder.Ascending;

            BeginUpdate();
            Sort(e.Column);
            EndUpdate();
        }

        /// <summary>
        /// Handle when a user checks/unchecks a row
        /// </summary>
        protected void HandleItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (CheckStatePutter != null)
                e.NewValue = CheckStatePutter(GetModelObject(e.Index), e.NewValue);
        }

        #endregion

        #region Low level Windows message handling

        /// <summary>
        /// Override the basic message pump for this control
        /// </summary>
        /// <param name="m"></param>
        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {
                case 0x0F: // WM_PAINT
                    HandlePrePaint();
                    base.WndProc(ref m);
                    HandlePostPaint();
                    break;
                case 0x4E: // WM_NOTIFY
                    if (!HandleNotify(ref m))
                        base.WndProc(ref m);
                    break;
                case 0x05: // WM_SIZE
                case 0x114: // WM_HSCROLL:
                case 0x115: // WM_VSCROLL:
                    PossibleFinishCellEditing();
                    base.WndProc(ref m);
                    break;
                default:
                    base.WndProc(ref m);
                    break;
            }
        }

        #endregion

        #region Empty List Msg handling

        protected void HandlePrePaint()
        {
            // When we get a WM_PAINT msg, remember the rectangle that is being updated. 
            // We can't get this information later, since the BeginPaint call wipes it out.
            lastUpdateRectangle = NativeMethods.GetUpdateRect(this);

            // When the list is empty, we want to handle the drawing of the control by ourselves.
            // Unfortunately, there is no easy way to tell our superclass that we want to do this.
            // So we resort to guile and deception. We validate the list area of the control, which
            // effectively tells our superclass that this area does not need to be painted. 
            // Our superclass will then not paint the control, leaving us free to do so ourselves.
            // Without doing this trickery, the superclass will draw the
            // list as empty, and then moments later, we will draw the empty msg, giving a nasty flicker
            if (GetItemCount() == 0 && HasEmptyListMsg)
                NativeMethods.ValidateRect(this, ClientRectangle);
        }

        protected void HandlePostPaint()
        {
            // If the list isn't empty or there isn't an emptyList msg, do nothing
            if (GetItemCount() != 0 || !HasEmptyListMsg)
                return;

            // Draw the empty list msg centered in the client area of the control
            using (
                BufferedGraphics buffered = BufferedGraphicsManager.Current.Allocate(CreateGraphics(), ClientRectangle))
            {
                Graphics g = buffered.Graphics;
                g.Clear(BackColor);
                StringFormat sf = new StringFormat();
                sf.Alignment = StringAlignment.Center;
                sf.LineAlignment = StringAlignment.Center;
                sf.Trimming = StringTrimming.EllipsisCharacter;
                g.DrawString(EmptyListMsg, EmptyListMsgFontOrDefault, SystemBrushes.ControlDark, ClientRectangle, sf);
                buffered.Render();
            }
        }

        #endregion

        #region Column hiding and resizing handling

        private HeaderControl hdrCtrl = null;

        /// <summary>
        /// When the control is created capture the messages for the header. 
        /// </summary>
        protected override void OnCreateControl()
        {
            base.OnCreateControl();
            hdrCtrl = new HeaderControl(this);
        }

        /// <summary>
        /// In the notification messages, we handle attempts to change the width of our columns
        /// </summary>
        /// <param name="m">The msg to be processed</param>
        /// <returns>bool to indicate if the msg has been handled</returns>
        protected bool HandleNotify(ref Message m)
        {
            bool isMsgHandled = false;

            const int NM_RCLICK = -5;
            const int HDN_FIRST = (0 - 300);
            const int HDN_DIVIDERDBLCLICKA = (HDN_FIRST - 5);
            const int HDN_DIVIDERDBLCLICKW = (HDN_FIRST - 25);
            const int HDN_BEGINTRACKA = (HDN_FIRST - 6);
            const int HDN_BEGINTRACKW = (HDN_FIRST - 26);
            //const int HDN_ENDTRACKA = (HDN_FIRST - 7);
            //const int HDN_ENDTRACKW = (HDN_FIRST - 27);
            const int HDN_TRACKA = (HDN_FIRST - 8);
            const int HDN_TRACKW = (HDN_FIRST - 28);
            const int HDN_ITEMCHANGINGA = (HDN_FIRST - 0);
            const int HDN_ITEMCHANGINGW = (HDN_FIRST - 20);

            // Handle the notification, remembering to handle both ANSI and Unicode versions
            NativeMethods.NMHDR nmhdr = (NativeMethods.NMHDR) m.GetLParam(typeof (NativeMethods.NMHDR));
            //if (nmhdr.code < HDN_FIRST)
            //  System.Diagnostics.Debug.WriteLine(nmhdr.code);

            // In KB Article #183258, MS states that when a header control has the HDS_FULLDRAG style, it will receive
            // ITEMCHANGING events rather than TRACK events. Under XP SP2 (at least) this is not always true, which may be
            // why MS has withdrawn that particular KB article. It is true that the header is always given the HDS_FULLDRAG
            // style. But even while window style set, the control doesn't always received ITEMCHANGING events.
            // The controlling setting seems to be the Explorer option "Show Window Contents While Dragging"!
            // In the category of "truly bizarre side effects", if the this option is turned on, we will receive
            // ITEMCHANGING events instead of TRACK events. But if it is turned off, we receive lots of TRACK events and
            // only one ITEMCHANGING event at the very end of the process.
            // If we receive HDN_TRACK messages, it's harder to control the resizing process. If we return a result of 1, we
            // cancel the whole drag operation, not just that particular track event, which is clearly not what we want.
            // If we are willing to compile with unsafe code enabled, we can modify the size of the column in place, using the
            // commented out code below. But without unsafe code, the best we can do is allow the user to drag the column to
            // any width, and then spring it back to within bounds once they release the mouse button. UI-wise it's very ugly.
            NativeMethods.NMHEADER nmheader;
            switch (nmhdr.code)
            {
                case NM_RCLICK:
                    PossibleFinishCellEditing();
                    isMsgHandled = HandleHeaderRightClick();
                    break;

                case HDN_DIVIDERDBLCLICKA:
                case HDN_DIVIDERDBLCLICKW:
                case HDN_BEGINTRACKA:
                case HDN_BEGINTRACKW:
                    PossibleFinishCellEditing();
                    break;

                case HDN_TRACKA:
                case HDN_TRACKW:
                    nmheader = (NativeMethods.NMHEADER) m.GetLParam(typeof (NativeMethods.NMHEADER));
                    if (nmheader.iItem >= 0 && nmheader.iItem < Columns.Count)
                    {
                        //    unsafe {
                        //        NativeMethods.HDITEM *hditem = (NativeMethods.HDITEM*)nmheader.pHDITEM;
                        //        OLVColumn column = this.GetColumn(nmheader.iItem);
                        //        if (hditem->cxy < column.MiniumWidth)
                        //            hditem->cxy = column.MiniumWidth;
                        //        else if (column.MaxiumWidth != -1 && hditem->cxy > column.MaxiumWidth)
                        //            hditem->cxy = column.MaxiumWidth;
                        //    }
                    }
                    break;

                case HDN_ITEMCHANGINGA:
                case HDN_ITEMCHANGINGW:
                    PossibleFinishCellEditing();
                    nmheader = (NativeMethods.NMHEADER) m.GetLParam(typeof (NativeMethods.NMHEADER));
                    if (nmheader.iItem >= 0 && nmheader.iItem < Columns.Count)
                    {
                        NativeMethods.HDITEM hditem =
                            (NativeMethods.HDITEM)
                            Marshal.PtrToStructure(nmheader.pHDITEM, typeof (NativeMethods.HDITEM));
                        OLVColumn column = GetColumn(nmheader.iItem);
                        // Check the mask to see if the width field is valid, and if it is, make sure it's within range
                        if ((hditem.mask & 1) == 1 &&
                            (hditem.cxy < column.MinimumWidth ||
                             (column.MaximumWidth != -1 && hditem.cxy > column.MaximumWidth)))
                        {
                            m.Result = (IntPtr) 1; // prevent the change from happening
                            isMsgHandled = true;
                        }
                    }
                    break;

                default:
                    break;
            }

            return isMsgHandled;
        }

        /// <summary>
        /// The user has right clicked on the column headers. Do whatever is required
        /// </summary>
        /// <returns>Return true if this event has been handle</returns>
        protected virtual bool HandleHeaderRightClick()
        {
            if (SelectColumnsOnRightClick)
            {
                ShowColumnSelectMenu(Cursor.Position);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Show a popup menu at the given point which will allow the user to choose which columns
        /// are visible on this listview
        /// </summary>
        /// <param name="pt">Where should the menu be placed</param>
        protected void ShowColumnSelectMenu(Point pt)
        {
            ContextMenuStrip m = new ContextMenuStrip();

            m.ItemClicked += new ToolStripItemClickedEventHandler(ColumnSelectMenu_ItemClicked);
            m.Closing += new ToolStripDropDownClosingEventHandler(ColumnSelectMenu_Closing);

            List<OLVColumn> columns = new List<OLVColumn>(AllColumns);
            columns.Sort(delegate(OLVColumn x, OLVColumn y) { return String.Compare(x.Text, y.Text, true); });
            foreach (OLVColumn col in columns)
            {
                ToolStripMenuItem mi = new ToolStripMenuItem(col.Text);
                mi.Checked = col.IsVisible;
                mi.Tag = col;

                // The 'Index' property returns -1 when the column is not visible, so if the
                // column isn't visible we have to enable the item. Also the first column can't be turned off
                mi.Enabled = !col.IsVisible || (col.Index > 0);
                m.Items.Add(mi);
            }
            m.Show(pt);
        }

        private void ColumnSelectMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            ToolStripMenuItem mi = (ToolStripMenuItem) e.ClickedItem;
            OLVColumn col = (OLVColumn) mi.Tag;
            mi.Checked = !mi.Checked;
            col.IsVisible = mi.Checked;
            BeginInvoke(new MethodInvoker(RebuildColumns));
        }

        private void ColumnSelectMenu_Closing(object sender, ToolStripDropDownClosingEventArgs e)
        {
            e.Cancel = (SelectColumnsMenuStaysOpen && e.CloseReason == ToolStripDropDownCloseReason.ItemClicked);
        }

        protected override void OnColumnReordered(ColumnReorderedEventArgs e)
        {
            base.OnColumnReordered(e);

            // The internal logic of the .NET code behind a ENDDRAG event means that,
            // at this point, the DisplayIndex's of the columns are not yet as they are
            // going to be. So we have to invoke a method to run later that will remember
            // what the real DisplayIndex's are.
            BeginInvoke(new MethodInvoker(RememberDisplayIndicies));
        }

        private void RememberDisplayIndicies()
        {
            // Remember the display indexes so we can put them back at a later date
            foreach (OLVColumn x in AllColumns)
                x.LastDisplayIndex = x.DisplayIndex;
        }

        /// <summary>
        /// Class used to capture window messages for the header of the list view
        /// control.  
        /// </summary>
        /// <remarks>We only need this class in order to not change the cursor
        /// when the cursor is over the divider of a fixed width column. It
        /// really is a little too perfectionist even for me.</remarks>
        private class HeaderControl : NativeWindow
        {
            private ObjectListView parentListView = null;

            public HeaderControl(ObjectListView olv)
            {
                parentListView = olv;
                AssignHandle(NativeMethods.GetHeaderControl(olv));
            }

            protected override void WndProc(ref Message message)
            {
                const int WM_SETCURSOR = 0x0020;

                switch (message.Msg)
                {
                    case WM_SETCURSOR:
                        if (IsCursorOverLockedDivider())
                        {
                            message.Result = (IntPtr) 1; // Don't change the cursor
                            return;
                        }
                        break;
                }

                base.WndProc(ref message);
            }

            private bool IsCursorOverLockedDivider()
            {
                Point pt = parentListView.PointToClient(Cursor.Position);
                int dividerIndex = NativeMethods.GetDividerUnderPoint(Handle, pt);
                if (dividerIndex >= 0 && dividerIndex < parentListView.Columns.Count)
                {
                    OLVColumn column = parentListView.GetColumn(dividerIndex);
                    return column.IsFixedWidth;
                }
                else
                    return false;
            }
        }

        #endregion

        #region OLV accessing

        /// <summary>
        /// Return the column at the given index
        /// </summary>
        /// <param name="index">Index of the column to be returned</param>
        /// <returns>An OLVColumn</returns>
        public OLVColumn GetColumn(int index)
        {
            return (OLVColumn) Columns[index];
        }

        /// <summary>
        /// Return the column at the given title. 
        /// </summary>
        /// <param name="name">Name of the column to be returned</param>
        /// <returns>An OLVColumn</returns>
        public OLVColumn GetColumn(string name)
        {
            foreach (ColumnHeader column in Columns)
            {
                if (column.Text == name)
                    return (OLVColumn) column;
            }
            return null;
        }

        /// <summary>
        /// Return the number of items in the list
        /// </summary>
        /// <returns>the number of items in the list</returns>
        public virtual int GetItemCount()
        {
            return Items.Count;
        }

        /// <summary>
        /// Return the item at the given index
        /// </summary>
        /// <param name="name">Index of the item to be returned</param>
        /// <returns>An OLVListItem</returns>
        public virtual OLVListItem GetItem(int index)
        {
            return (OLVListItem) Items[index];
        }

        /// <summary>
        /// Return the model object at the given index
        /// </summary>
        /// <param name="name">Index of the model object to be returned</param>
        /// <returns>A model object</returns>
        public virtual object GetModelObject(int index)
        {
            return GetItem(index).RowObject;
        }

        #endregion

        #region Object manipulation

        /// <summary>
        /// Select all rows in the listview
        /// </summary>
        public void SelectAll()
        {
            NativeMethods.SelectAllItems(this);
        }

        /// <summary>
        /// Deselect all rows in the listview
        /// </summary>
        public void DeselectAll()
        {
            NativeMethods.DeselectAllItems(this);
        }

        /// <summary>
        /// Return the model object of the row that is selected or null if there is no selection or more than one selection
        /// </summary>
        /// <returns>Model object or null</returns>
        public virtual object GetSelectedObject()
        {
            if (SelectedIndices.Count == 1)
                return GetModelObject(SelectedIndices[0]);
            else
                return null;
        }

        /// <summary>
        /// Return the model objects of the rows that are selected or an empty collection if there is no selection
        /// </summary>
        /// <returns>ArrayList</returns>
        public virtual ArrayList GetSelectedObjects()
        {
            ArrayList objects = new ArrayList(SelectedIndices.Count);
            foreach (int index in SelectedIndices)
                objects.Add(GetModelObject(index));

            return objects;
        }

        /// <summary>
        /// Select the row that is displaying the given model object. All other rows are deselected.
        /// </summary>
        /// <param name="modelObject">The object to be selected or null to deselect all</param>
        public virtual void SelectObject(object modelObject)
        {
            if (SelectedItems.Count == 1 && ((OLVListItem) SelectedItems[0]).RowObject == modelObject)
                return;

            SelectedItems.Clear();

            //TODO: If this is too slow, we could keep a map of model object to ListViewItems
            foreach (ListViewItem lvi in Items)
            {
                if (((OLVListItem) lvi).RowObject == modelObject)
                {
                    lvi.Selected = true;
                    break;
                }
            }
        }

        /// <summary>
        /// Select the rows that is displaying any of the given model object. All other rows are deselected.
        /// </summary>
        /// <param name="modelObjects">A collection of model objects</param>
        public virtual void SelectObjects(IList modelObjects)
        {
            SelectedItems.Clear();

            //TODO: If this is too slow, we could keep a map of model object to ListViewItems
            foreach (ListViewItem lvi in Items)
            {
                if (modelObjects.Contains(((OLVListItem) lvi).RowObject))
                    lvi.Selected = true;
            }
        }

        /// <summary>
        /// Update the ListViewItem with the data from its associated model.
        /// </summary>
        /// <remarks>This method does not resort or regroup the view. It simply updates
        /// the displayed data of the given item</remarks>
        public virtual void RefreshItem(OLVListItem olvi)
        {
            // For some reason, clearing the subitems also wipes out the back color,
            // so we need to store it and then put it back again later
            Color c = olvi.BackColor;
            olvi.SubItems.Clear();
            FillInValues(olvi, olvi.RowObject);
            SetSubItemImages(olvi.Index, olvi, true);
            olvi.BackColor = c;
        }

        /// <summary>
        /// Update the rows that are showing the given objects
        /// </summary>
        /// <remarks>This method does not resort or regroup the view.</remarks>
        public virtual void RefreshObject(object modelObject)
        {
            RefreshObjects(new object[] {modelObject});
        }

        /// <summary>
        /// Update the rows that are showing the given objects
        /// </summary>
        /// <remarks>This method does not resort or regroup the view.</remarks>
        public virtual void RefreshObjects(IList modelObjects)
        {
            if (InvokeRequired)
            {
                Invoke(new RefreshObjectsInvoker(RefreshObjects), new object[] {modelObjects});
                return;
            }
            foreach (ListViewItem lvi in Items)
            {
                OLVListItem olvi = (OLVListItem) lvi;
                if (modelObjects.Contains(olvi.RowObject))
                    RefreshItem(olvi);
            }
        }

        /// <summary>
        /// Update the rows that are selected
        /// </summary>
        /// <remarks>This method does not resort or regroup the view.</remarks>
        public void RefreshSelectedObjects()
        {
            foreach (ListViewItem lvi in SelectedItems)
                RefreshItem((OLVListItem) lvi);
        }

        /// <summary>
        /// Find the given model object within the listview and return its index
        /// </summary>
        /// <remarks>Technically, this method will work with virtual lists, but it will
        /// probably be very slow.</remarks>
        /// <param name="modelObject">The model object to be found</param>
        /// <returns>The index of the object. -1 means the object was not present</returns>
        public int IndexOf(Object modelObject)
        {
            for (int i = 0; i < GetItemCount(); i++)
            {
                if (GetModelObject(i) == modelObject)
                    return i;
            }
            return -1;
        }

        /// <summary>
        /// Return the ListViewItem that appears immediately after the given item. 
        /// If the given item is null, the first item in the list will be returned.
        /// Return null if the given item is the last item.
        /// </summary>
        /// <param name="item">The item that is before the item that is returned, or null</param>
        /// <returns>A ListViewItem</returns>
        public ListViewItem GetNextItem(ListViewItem itemToFind)
        {
            if (ShowGroups)
            {
                bool isFound = (itemToFind == null);
                foreach (ListViewGroup group in Groups)
                {
                    foreach (ListViewItem lvi in group.Items)
                    {
                        if (isFound)
                            return lvi;
                        isFound = (lvi == itemToFind);
                    }
                }
                return null;
            }
            else
            {
                if (GetItemCount() == 0)
                    return null;
                if (itemToFind == null)
                    return GetItem(0);
                if (itemToFind.Index == GetItemCount() - 1)
                    return null;
                return GetItem(itemToFind.Index + 1);
            }
        }

        /// <summary>
        /// Return the ListViewItem that appears immediately before the given item. 
        /// If the given item is null, the last item in the list will be returned.
        /// Return null if the given item is the first item.
        /// </summary>
        /// <param name="item">The item that is before the item that is returned</param>
        /// <returns>A ListViewItem</returns>
        public ListViewItem GetPreviousItem(ListViewItem itemToFind)
        {
            if (ShowGroups)
            {
                ListViewItem previousItem = null;
                foreach (ListViewGroup group in Groups)
                {
                    foreach (ListViewItem lvi in group.Items)
                    {
                        if (lvi == itemToFind)
                            return previousItem;
                        else
                            previousItem = lvi;
                    }
                }
                if (itemToFind == null)
                    return previousItem;
                else
                    return null;
            }
            else
            {
                if (GetItemCount() == 0)
                    return null;
                if (itemToFind == null)
                    return GetItem(GetItemCount() - 1);
                if (itemToFind.Index == 0)
                    return null;
                return GetItem(itemToFind.Index - 1);
            }
        }

        private delegate void RefreshObjectsInvoker(IList modelObjects);

        #endregion

        #region Freezing

        /// <summary>
        /// Freeze the listview so that it no longer updates itself.
        /// </summary>
        /// <remarks>Freeze()/Unfreeze() calls nest correctly</remarks>
        public void Freeze()
        {
            freezeCount++;
        }

        /// <summary>
        /// Unfreeze the listview. If this call is the outermost Unfreeze(),
        /// the contents of the listview will be rebuilt.
        /// </summary>
        /// <remarks>Freeze()/Unfreeze() calls nest correctly</remarks>
        public void Unfreeze()
        {
            if (freezeCount <= 0)
                return;

            freezeCount--;
            if (freezeCount == 0)
                DoUnfreeze();
        }

        /// <summary>
        /// Do the actual work required when the listview is unfrozen
        /// </summary>
        protected virtual void DoUnfreeze()
        {
            BuildList();
        }

        #endregion

        #region Column Sorting

        private const string SORT_INDICATOR_DOWN_KEY = "sort-indicator-down";
        private const string SORT_INDICATOR_UP_KEY = "sort-indicator-up";

        /// <summary>
        /// Sort the items in the list view by the values in the given column.
        /// If ShowGroups is true, the rows will be grouped by the given column,
        /// otherwise, it will be a straight sort.
        /// </summary>
        /// <param name="columnToSortName">The name of the column whose values will be used for the sorting</param>
        public void Sort(string columnToSortName)
        {
            Sort(GetColumn(columnToSortName));
        }

        /// <summary>
        /// Sort the items in the list view by the values in the given column.
        /// If ShowGroups is true, the rows will be grouped by the given column,
        /// otherwise, it will be a straight sort.
        /// </summary>
        /// <param name="columnToSortIndex">The index of the column whose values will be used for the sorting</param>
        public void Sort(int columnToSortIndex)
        {
            if (columnToSortIndex >= 0 && columnToSortIndex < Columns.Count)
                Sort(GetColumn(columnToSortIndex));
        }

        /// <summary>
        /// Sort the items in the list view by the values in the given column.
        /// If ShowGroups is true, the rows will be grouped by the given column,
        /// otherwise, it will be a straight sort.
        /// </summary>
        /// <param name="columnToSort">The column whose values will be used for the sorting</param>
        public virtual void Sort(OLVColumn columnToSort)
        {
            if (InvokeRequired)
            {
                Invoke(new SortInvoker(Sort), new object[] {columnToSort});
                return;
            }

            if (Columns.Count < 1)
                return;

            if (columnToSort == null)
                columnToSort = GetColumn(0);

            if (lastSortOrder == SortOrder.None)
                lastSortOrder = Sorting;

            if (ShowGroups)
                BuildGroups(columnToSort);
            else if (CustomSorter != null)
                CustomSorter(columnToSort, lastSortOrder);
            else
                ListViewItemSorter =
                    new ColumnComparer(columnToSort, lastSortOrder, SecondarySortColumn, SecondarySortOrder);

            if (ShowSortIndicators)
                ShowSortIndicator(columnToSort, lastSortOrder);

            if (UseAlternatingBackColors && View == View.Details)
                PrepareAlternateBackColors();

            lastSortColumn = columnToSort;
        }

        /// <summary>
        /// Put a sort indicator next to the text of the sort column
        /// </summary>
        public void ShowSortIndicator()
        {
            if (ShowSortIndicators && lastSortOrder != SortOrder.None)
                ShowSortIndicator(lastSortColumn, lastSortOrder);
        }

        /// <summary>
        /// Put a sort indicator next to the text of the given given column
        /// </summary>
        /// <param name="columnToSort">The column to be marked</param>
        /// <param name="sortOrder">The sort order in effect on that column</param>
        protected void ShowSortIndicator(OLVColumn columnToSort, SortOrder sortOrder)
        {
            int imageIndex = -1;

            if (!NativeMethods.HasBuiltinSortIndicators())
            {
                // If we can't use builtin image, we have to make and then locate the index of the 
                // sort indicator we want to use. SortOrder.None doesn't show an image.
                if (SmallImageList == null || !SmallImageList.Images.ContainsKey(SORT_INDICATOR_UP_KEY))
                    MakeSortIndicatorImages();

                if (sortOrder == SortOrder.Ascending)
                    imageIndex = SmallImageList.Images.IndexOfKey(SORT_INDICATOR_UP_KEY);
                else if (sortOrder == SortOrder.Descending)
                    imageIndex = SmallImageList.Images.IndexOfKey(SORT_INDICATOR_DOWN_KEY);
            }

            // Set the image for each column
            for (int i = 0; i < Columns.Count; i++)
            {
                if (i == columnToSort.Index)
                    NativeMethods.SetColumnImage(this, i, sortOrder, imageIndex);
                else
                    NativeMethods.SetColumnImage(this, i, SortOrder.None, -1);
            }
        }

        /// <summary>
        /// If the sort indicator images don't already exist, this method will make and install them
        /// </summary>
        protected void MakeSortIndicatorImages()
        {
            ImageList il = SmallImageList;
            if (il == null)
            {
                il = new ImageList();
                il.ImageSize = new Size(16, 16);
            }

            // This arrangement of points works well with (16,16) images, and OK with others
            int midX = il.ImageSize.Width/2;
            int midY = (il.ImageSize.Height/2) - 1;
            int deltaX = midX - 2;
            int deltaY = deltaX/2;

            if (il.Images.IndexOfKey(SORT_INDICATOR_UP_KEY) == -1)
            {
                Point pt1 = new Point(midX - deltaX, midY + deltaY);
                Point pt2 = new Point(midX, midY - deltaY - 1);
                Point pt3 = new Point(midX + deltaX, midY + deltaY);
                il.Images.Add(SORT_INDICATOR_UP_KEY, MakeTriangleBitmap(il.ImageSize, new Point[] {pt1, pt2, pt3}));
            }

            if (il.Images.IndexOfKey(SORT_INDICATOR_DOWN_KEY) == -1)
            {
                Point pt1 = new Point(midX - deltaX, midY - deltaY);
                Point pt2 = new Point(midX, midY + deltaY);
                Point pt3 = new Point(midX + deltaX, midY - deltaY);
                il.Images.Add(SORT_INDICATOR_DOWN_KEY, MakeTriangleBitmap(il.ImageSize, new Point[] {pt1, pt2, pt3}));
            }

            SmallImageList = il;
        }

        private Bitmap MakeTriangleBitmap(Size sz, Point[] pts)
        {
            Bitmap bm = new Bitmap(sz.Width, sz.Height);
            Graphics g = Graphics.FromImage(bm);
            g.FillPolygon(new SolidBrush(Color.Gray), pts);
            return bm;
        }

        protected delegate void SortInvoker(OLVColumn columnToSort);

        #endregion

        #region Utilities

        /// <summary>
        /// Fill in the given OLVListItem with values of the given row
        /// </summary>
        /// <param name="lvi">the OLVListItem that is to be stuff with values</param>
        /// <param name="rowObject">the model object from which values will be taken</param>
        protected void FillInValues(OLVListItem lvi, object rowObject)
        {
            if (Columns.Count == 0)
                return;

            OLVColumn column = GetColumn(0);
            lvi.Text = column.GetStringValue(rowObject);
            lvi.ImageSelector = column.GetImage(rowObject);

            for (int i = 1; i < Columns.Count; i++)
            {
                column = GetColumn(i);
                lvi.SubItems.Add(new OLVListSubItem(column.GetStringValue(rowObject), column.GetImage(rowObject)));
            }

            // Give the row formatter a chance to mess with the item
            if (RowFormatter != null)
                RowFormatter(lvi);

            // Get the check state of the row, if we are showing check boxes
            if (CheckBoxes && CheckStateGetter != null)
                lvi.Checked = CheckStateGetter(lvi.RowObject);
        }

        /// <summary>
        /// Setup all subitem images on all rows
        /// </summary>
        protected void SetAllSubItemImages()
        {
            if (!ShowImagesOnSubItems)
                return;

            ForceSubItemImagesExStyle();

            for (int rowIndex = 0; rowIndex < GetItemCount(); rowIndex++)
                SetSubItemImages(rowIndex, GetItem(rowIndex));
        }

        /// <summary>
        /// Tell the underlying list control which images to show against the subitems
        /// </summary>
        /// <param name="rowIndex">the index at which the item occurs</param>
        /// <param name="item">the item whose subitems are to be set</param>
        protected void SetSubItemImages(int rowIndex, OLVListItem item)
        {
            SetSubItemImages(rowIndex, item, false);
        }

        /// <summary>
        /// Tell the underlying list control which images to show against the subitems
        /// </summary>
        /// <param name="rowIndex">the index at which the item occurs</param>
        /// <param name="item">the item whose subitems are to be set</param>
        /// <param name="shouldClearImages">will existing images be cleared if no new image is provided?</param>
        protected void SetSubItemImages(int rowIndex, OLVListItem item, bool shouldClearImages)
        {
            if (!ShowImagesOnSubItems)
                return;

            for (int i = 1; i < item.SubItems.Count; i++)
            {
                int imageIndex = GetActualImageIndex(((OLVListSubItem) item.SubItems[i]).ImageSelector);
                if (shouldClearImages || imageIndex != -1)
                    SetSubItemImage(rowIndex, i, imageIndex);
            }
        }

        /// <summary>
        /// Prepare the listview to show alternate row backcolors
        /// </summary>
        /// <remarks>We cannot rely on <code>lvi.Index</code> in this method.
        /// In a straight list, <code>lvi.Index</code> is the display index, and can be used to determine
        /// whether the row should be colored. But when organised by groups, <code>lvi.Index</code> is not
        /// useable because it still refers to the position in the overall list, not the display order.
        ///</remarks>
        protected virtual void PrepareAlternateBackColors()
        {
            Color rowBackColor = AlternateRowBackColorOrDefault;
            int i = 0;

            if (ShowGroups)
            {
                foreach (ListViewGroup group in Groups)
                {
                    foreach (ListViewItem lvi in group.Items)
                    {
                        if (i%2 == 0)
                            lvi.BackColor = BackColor;
                        else
                            lvi.BackColor = rowBackColor;
                        i++;
                    }
                }
            }
            else
            {
                foreach (ListViewItem lvi in Items)
                {
                    if (i%2 == 0)
                        lvi.BackColor = BackColor;
                    else
                        lvi.BackColor = rowBackColor;
                    i++;
                }
            }
        }

        /// <summary>
        /// Convert the given image selector to an index into our image list.
        /// Return -1 if that's not possible
        /// </summary>
        /// <param name="imageSelector"></param>
        /// <returns>Index of the image in the imageList, or -1</returns>
        public int GetActualImageIndex(Object imageSelector)
        {
            if (imageSelector == null)
                return -1;

            if (imageSelector is Int32)
                return (int) imageSelector;

            if (imageSelector is String && SmallImageList != null)
                return SmallImageList.Images.IndexOfKey((String) imageSelector);

            return -1;
        }

        /// <summary>
        /// Make sure the ListView has the extended style that says to display subitem images.
        /// </summary>
        /// <remarks>This method must be called after any .NET call that update the extended styles
        /// since they seem to erase this setting.</remarks>
        protected void ForceSubItemImagesExStyle()
        {
            NativeMethods.ForceSubItemImagesExStyle(this);
        }

        /// <summary>
        /// For the given item and subitem, make it display the given image
        /// </summary>
        /// <param name="itemIndex">row number (0 based)</param>
        /// <param name="subItemIndex">subitem (0 is the item itself)</param>
        /// <param name="imageIndex">index into the image list</param>
        protected void SetSubItemImage(int itemIndex, int subItemIndex, int imageIndex)
        {
            NativeMethods.SetSubItemImage(this, itemIndex, subItemIndex, imageIndex);
        }

        #endregion

        #region ISupportInitialize Members

        void ISupportInitialize.BeginInit()
        {
            Frozen = true;
        }

        void ISupportInitialize.EndInit()
        {
            Frozen = false;
        }

        #region Image list manipulation

        /// <summary>
        /// Update our externally visible image list so it holds the same images as our shadow list, but sized correctly
        /// </summary>
        private void SetupExternalImageList()
        {
            // If a row height hasn't been set, or an image list has been give which is the required size, just assign it
            if (rowHeight == -1 || (shadowedImageList != null && shadowedImageList.ImageSize.Height == rowHeight))
                base.SmallImageList = shadowedImageList;
            else
                base.SmallImageList = MakeResizedImageList(rowHeight, shadowedImageList);
        }

        /// <summary>
        /// Return a copy of the given source image list, where each image has been resized to be height x height in size.
        /// If source is null, an empty image list of the given size is returned
        /// </summary>
        /// <param name="height">Height and width of the new images</param>
        /// <param name="source">Source of the images (can be null)</param>
        /// <returns>A new image list</returns>
        private ImageList MakeResizedImageList(int height, ImageList source)
        {
            // Return a copy of the source image list, where each image has been resized to the given width and height
            ImageList il = new ImageList();
            il.ImageSize = new Size(height, height);

            // If there's nothing to copy, just return the new list
            if (source == null)
                return il;

            il.TransparentColor = source.TransparentColor;
            il.ColorDepth = source.ColorDepth;

            // Fill the imagelist with resized copies from the source
            for (int i = 0; i < source.Images.Count; i++)
            {
                Bitmap bm = MakeResizedImage(height, source.Images[i], source.TransparentColor);
                il.Images.Add(bm);
            }

            // Give each image the same key it has in the original
            foreach (String key in source.Images.Keys) il.Images.SetKeyName(source.Images.IndexOfKey(key), key);

            return il;
        }

        /// <summary>
        /// Return a bitmap of the given height x height, which shows the given image, centred.
        /// </summary>
        /// <param name="height">Height and width of new bitmap</param>
        /// <param name="image">Image to be centred</param>
        /// <param name="transparent">The background color</param>
        /// <returns>A new bitmap</returns>
        private Bitmap MakeResizedImage(int height, Image image, Color transparent)
        {
            Bitmap bm = new Bitmap(height, height);
            Graphics g = Graphics.FromImage(bm);
            g.Clear(transparent);
            int x = Math.Max(0, (bm.Size.Width - image.Size.Width)/2);
            int y = Math.Max(0, (bm.Size.Height - image.Size.Height)/2);
            g.DrawImage(image, x, y, image.Size.Width, image.Size.Height);
            return bm;
        }

        #endregion

        #region Owner drawing

        private int[] columnRightEdge = new int[128]; // will anyone ever want more than 128 columns??

        /// <summary>
        /// Owner draw the column header
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDrawColumnHeader(DrawListViewColumnHeaderEventArgs e)
        {
            e.DrawDefault = true;
            base.OnDrawColumnHeader(e);
        }

        /// <summary>
        /// Owner draw the item
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDrawItem(DrawListViewItemEventArgs e)
        {
            e.DrawDefault = (View != View.Details);
            base.OnDrawItem(e);
        }

        /// <summary>
        /// Owner draw a single subitem
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDrawSubItem(DrawListViewSubItemEventArgs e)
        {
            // Get the special renderer for this column. 
            // If there isn't one, use the default draw mechanism.
            OLVColumn column = GetColumn(e.ColumnIndex);
            if (column.RendererDelegate == null)
            {
                e.DrawDefault = true;
                return;
            }

            // Calculate where the subitem should be drawn
            // It should be as simple as 'e.Bounds', but it isn't :-(

            // There seems to be a bug in .NET where the left edge of the bounds of subitem 0
            // is always 0. This is normally what is required, but it's wrong when
            // someone drags column 0 to somewhere else in the listview. We could
            // drop down into Windows-ville and use LVM_GETSUBITEMRECT, but just to be different
            // we keep track of the right edge of all columns, and when subitem 0
            // isn't being shown at column 0, we make its left edge to be the right
            // edge of the previous column plus a little bit.
            //TODO: Replace with LVM_GETSUBITEMRECT
            Rectangle r = e.Bounds;
            if (e.ColumnIndex == 0 && e.Header.DisplayIndex != 0) r.X = columnRightEdge[e.Header.DisplayIndex - 1] + 1;
            else
                //TODO: Check the size of columnRightEdge and dynamically reallocate?
                columnRightEdge[e.Header.DisplayIndex] = e.Bounds.Right;

            // Optimize drawing by only redrawing subitems that touch the area that was damaged
            if (!r.IntersectsWith(lastUpdateRectangle)) return;

            // Get a graphics context for the renderer to use. 
            // But we have more complications. Virtual lists have a nasty habit of drawing column 0
            // whenever there is any mouse move events over a row, and doing it in an un-double buffered manner,
            // which results in nasty flickers! There are also some unbuffered draw when a mouse is first
            // hovered over column 0 of a normal row. So, to avoid all complications,
            // we always manually double-buffer the drawing.
            Graphics g = e.Graphics;
            BufferedGraphics buffer = null;
            bool avoidFlickerMode = true; // set this to false to see the probems with flicker
            if (avoidFlickerMode)
            {
                buffer = BufferedGraphicsManager.Current.Allocate(e.Graphics, r);
                g = buffer.Graphics;
            }

            // Finally, give the renderer a chance to draw something
            Object row = ((OLVListItem) e.Item).RowObject;
            column.RendererDelegate(e, g, r, row);

            if (buffer != null)
            {
                buffer.Render();
                buffer.Dispose();
            }
        }

        #endregion

        #region Selection Event Handling

        private bool hasIdleHandler = false;

        /// <summary>
        /// This method is called every time a row is selected or deselected. This can be
        /// a pain if the user shift-clicks 100 rows. We override this method so we can
        /// trigger one event for any number of select/deselects that come from one user action
        /// </summary>
        /// <param name="e"></param>
        protected override void OnSelectedIndexChanged(EventArgs e)
        {
            base.OnSelectedIndexChanged(e);

            // If we haven't already scheduled an event, schedule it to be triggered
            // By using idle time, we will wait until all select events for the same
            // user action have finished before triggering the event.
            if (!hasIdleHandler)
            {
                hasIdleHandler = true;
                Application.Idle += new EventHandler(Application_Idle);
            }
        }

        /// <summary>
        /// The application is idle. Trigger a SelectionChanged event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void Application_Idle(object sender, EventArgs e)
        {
            // Remove the handler before triggering the event
            Application.Idle -= new EventHandler(Application_Idle);
            hasIdleHandler = false;

            OnSelectionChanged(new EventArgs());
        }

        /// <summary>
        /// This event is triggered once per user action that changes the selection state
        /// of one or more rows. 
        /// </summary>
        [Category("Behavior"),
         Description(
             "This event is triggered once per user action that changes the selection state of one or more rows.")]
        public event EventHandler SelectionChanged;

        /// <summary>
        /// Trigger the SelectionChanged event
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnSelectionChanged(EventArgs e)
        {
            if (SelectionChanged != null)
                SelectionChanged(this, e);
        }

        #endregion

        #region Cell editing

        // NOTES:
        // 
        // - What event should we listen to?
        //
        // We can't use OnMouseClick, OnMouseDoubleClick, OnClick, or OnDoubleClick 
        // since they are not triggered for clicks on subitems without Full Row Select.
        //
        // We could use OnMouseDown, but selecting rows is done in OnMouseUp. This means
        // that if we start the editing during OnMouseDown, the editor will automatically 
        // lose focus when mouse up happens.
        //

        // We need the click count in the mouse up event, but that is always 1.
        // So we have to remember the click count from the preceding mouse down event.

        #region Delegates

        /// <summary>
        /// The callbacks for CellEditing events
        /// </summary>
        public delegate void CellEditEventHandler(object sender, CellEditEventArgs e);

        #endregion

        private CellEditEventArgs cellEditEventArgs = null;
        private Control cellEditor = null;

        private int lastMouseDownClickCount = 0;

        protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);
            lastMouseDownClickCount = e.Clicks;
        }

        /// <summary>
        /// Check to see if we need to start editing a cell
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            // If it was an unmodified left click, check to see if we should start editing
            if (ShouldStartCellEdit(e))
            {
                ListViewHitTestInfo info = HitTest(e.Location);
                if (info.Item != null && info.SubItem != null)
                {
                    int subItemIndex = info.Item.SubItems.IndexOf(info.SubItem);

                    // We don't edit the primary column by single clicks -- only subitems.
                    if (CellEditActivation != CellEditActivateMode.SingleClick || subItemIndex > 0)
                        EditSubItem((OLVListItem) info.Item, subItemIndex);
                }
            }
        }

        /// <summary>
        /// Should we start editing the cell? 
        /// </summary>
        /// <param name="e"></param>
        /// <returns></returns>
        protected bool ShouldStartCellEdit(MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left)
                return false;

            if ((ModifierKeys & (Keys.Shift | Keys.Control | Keys.Alt)) != 0)
                return false;

            if (lastMouseDownClickCount == 1 && CellEditActivation == CellEditActivateMode.SingleClick)
                return true;

            return (lastMouseDownClickCount == 2 && CellEditActivation == CellEditActivateMode.DoubleClick);
        }

        /// <summary>
        /// Handle a key press on this control. We specifically look for F2 which edits the primary column,
        /// or a Tab character during an edit operation, which tries to start editing on the next (or previous) cell.
        /// </summary>
        /// <param name="keyData"></param>
        /// <returns></returns>
        protected override bool ProcessDialogKey(Keys keyData)
        {
            bool isSimpleTabKey = ((keyData & Keys.KeyCode) == Keys.Tab) &&
                                  ((keyData & (Keys.Alt | Keys.Control)) == Keys.None);

            if (isSimpleTabKey && IsCellEditing)
            {
                // Tab key while editing
                FinishCellEdit();

                OLVListItem olvi = cellEditEventArgs.ListViewItem;
                int subItemIndex = cellEditEventArgs.SubItemIndex;
                int displayIndex = GetColumn(subItemIndex).DisplayIndex;

                // Move to the next or previous editable subitem, depending on Shift key. Wrap at the edges
                List<OLVColumn> columnsInDisplayOrder = ColumnsInDisplayOrder;
                do
                {
                    if ((keyData & Keys.Shift) == Keys.Shift)
                        displayIndex = (olvi.SubItems.Count + displayIndex - 1)%olvi.SubItems.Count;
                    else
                        displayIndex = (displayIndex + 1)%olvi.SubItems.Count;
                } while (!columnsInDisplayOrder[displayIndex].IsEditable);

                // If we found a different editable cell, start editing it
                subItemIndex = columnsInDisplayOrder[displayIndex].Index;
                if (cellEditEventArgs.SubItemIndex != subItemIndex)
                {
                    StartCellEdit(olvi, subItemIndex);
                    return true;
                }
            }

            // Treat F2 as a request to edit the primary column
            if (keyData == Keys.F2 && !IsCellEditing)
            {
                EditSubItem((OLVListItem) FocusedItem, 0);
                return true;
            }

            // We have to catch Return/Enter/Escape here since some types of controls
            // (e.g. ComboBox, UserControl) don't trigger key events that we can listen for.
            // Treat Return or Enter as committing the current edit operation
            if ((keyData == Keys.Return || keyData == Keys.Enter) && IsCellEditing)
            {
                FinishCellEdit();
                return true;
            }

            // Treat Escaoe as cancel the current edit operation
            if (keyData == Keys.Escape && IsCellEditing)
            {
                CancelCellEdit();
                return true;
            }

            // Treat Ctrl-C as Copy To Clipboard. We still allow the default processing
            if ((keyData & Keys.Control) == Keys.Control && (keyData & Keys.KeyCode) == Keys.C)
                CopySelectionToClipboard();

            return base.ProcessDialogKey(keyData);
        }

        /// <summary>
        /// Begin an edit operation on the given cell.
        /// </summary>
        /// <remarks>This performs various sanity checks and passes off the real work to StartCellEdit().</remarks>
        /// <param name="item">The row to be edited</param>
        /// <param name="subItemIndex">The index of the cell to be edited</param>
        public void EditSubItem(OLVListItem item, int subItemIndex)
        {
            if (item == null)
                return;

            if (subItemIndex < 0 && subItemIndex >= item.SubItems.Count)
                return;

            if (CellEditActivation == CellEditActivateMode.None)
                return;

            if (!GetColumn(subItemIndex).IsEditable)
                return;

            StartCellEdit(item, subItemIndex);
        }

        /// <summary>
        /// Really start an edit operation on a given cell. The parameters are assumed to be sane.
        /// </summary>
        /// <param name="item">The row to be edited</param>
        /// <param name="subItemIndex">The index of the cell to be edited</param>
        protected void StartCellEdit(OLVListItem item, int subItemIndex)
        {
            OLVColumn column = GetColumn(subItemIndex);
            Rectangle r = CalculateCellBounds(item, subItemIndex);
            Control c = GetCellEditor(item, subItemIndex);
            c.Bounds = r;

            // Try to align the control as the column is aligned. Not all controls support this property
            PropertyInfo pinfo = c.GetType().GetProperty("TextAlign");
            if (pinfo != null)
                pinfo.SetValue(c, column.TextAlign, null);

            // Give the control the value from the model
            SetControlValue(c, column.GetValue(item.RowObject), column.GetStringValue(item.RowObject));

            // Give the outside world the chance to munge with the process
            cellEditEventArgs = new CellEditEventArgs(column, c, r, item, subItemIndex);
            OnCellEditStarting(cellEditEventArgs);
            if (cellEditEventArgs.Cancel)
                return;

            // The event handler may have completely changed the control, so we need to remember it
            cellEditor = cellEditEventArgs.Control;

            // If the control isn't the height of the cell, centre it vertically. We don't
            // need to do this when in Tile view.
            if (View != View.Tile && cellEditor.Height != r.Height)
                cellEditor.Top += (r.Height - cellEditor.Height)/2;

            Controls.Add(cellEditor);
            ConfigureControl();
            PauseAnimations(true);
        }

        /// <summary>
        /// Try to give the given value to the provided control. Fall back to assigning a string
        /// if the value assignment fails.
        /// </summary>
        /// <param name="c">A control</param>
        /// <param name="value">The value to be given to the control</param>
        /// <param name="stringValue">The string to be given if the value doesn't work</param>
        protected void SetControlValue(Control c, Object value, String stringValue)
        {
            // Look for a property called "Value". We have to look twice, the first time might get an ambiguous
            PropertyInfo pinfo = null;
            try
            {
                pinfo = c.GetType().GetProperty("Value");
            }
            catch (AmbiguousMatchException)
            {
                // The lowest level class of the control must have overridden the "Value" property. 
                // We now have to specifically  look for only public instance properties declared in the lowest level class.
                pinfo =
                    c.GetType().GetProperty("Value",
                                            BindingFlags.DeclaredOnly | BindingFlags.Instance | BindingFlags.Public);
            }

            // If we found it, use it to assign a value, otherwise simply set the text
            if (pinfo == null)
                c.Text = stringValue;
            else
            {
                try
                {
                    pinfo.SetValue(c, value, null);
                }
                catch (TargetInvocationException)
                {
                    // Not a lot we can do about this one. Something went wrong in the bowels
                    // of the method. Let's take the ostrich approach and just ignore it :-)
                }
                catch (ArgumentException)
                {
                    c.Text = stringValue;
                }
            }
        }

        /// <summary>
        /// Setup the given control to be a cell editor
        /// </summary>
        protected void ConfigureControl()
        {
            cellEditor.Leave += new EventHandler(CellEditor_Leave);
            cellEditor.Select();
        }

        /// <summary>
        /// Return the value that the given control is showing
        /// </summary>
        /// <param name="c"></param>
        /// <returns></returns>
        protected Object GetControlValue(Control c)
        {
            try
            {
                return c.GetType().InvokeMember("Value", BindingFlags.GetProperty, null, c, null);
            }
            catch (MissingMethodException)
            {
                return c.Text;
            }
        }

        /// <summary>
        /// Called when the cell editor loses focus. Time to commit the change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CellEditor_Leave(object sender, EventArgs e)
        {
            FinishCellEdit();
        }

        /// <summary>
        /// Return the bounds of the given cell
        /// </summary>
        /// <param name="item">The row to be edited</param>
        /// <param name="subItemIndex">The index of the cell to be edited</param>
        /// <returns>A Rectangle</returns>
        protected Rectangle CalculateCellBounds(OLVListItem item, int subItemIndex)
        {
            // Item 0 is special. Its bounds include all subitems. To get just the bounds
            // of cell for item 0, we have to use GetItemRect().
            if (subItemIndex == 0) return GetItemRect(item.Index, ItemBoundsPortion.Label);
            else
                return item.SubItems[subItemIndex].Bounds;
        }

        /// <summary>
        /// Return a control that can be used to edit the value of the given cell.
        /// </summary>
        /// <param name="item">The row to be edited</param>
        /// <param name="subItemIndex">The index of the cell to be edited</param>
        /// <returns></returns>
        protected Control GetCellEditor(OLVListItem item, int subItemIndex)
        {
            OLVColumn column = GetColumn(subItemIndex);
            Object value = column.GetValue(item.RowObject);

            //THINK: Do we want to use a Registry instead of this cascade?
            if (value is Boolean)
                return new BooleanCellEditor();

            if (value is DateTime)
            {
                DateTimePicker c = new DateTimePicker();
                c.Format = DateTimePickerFormat.Short;
                return c;
            }
            if (value is Int32 || value is Int16 || value is Int64)
                return new IntUpDown();

            if (value is UInt16 || value is UInt32 || value is UInt64)
                return new UintUpDown();

            if (value is Single || value is Double)
                return new FloatCellEditor();

            return MakeDefaultCellEditor(column);
        }

        /// <summary>
        /// Return a TextBox that can be used as a default cell editor.
        /// </summary>
        /// <param name="column">What column does the cell belong to?</param>
        /// <returns></returns>
        private Control MakeDefaultCellEditor(OLVColumn column)
        {
            TextBox tb = new TextBox();

            // Build a list of unique values, to be used as autocomplete on the editor
            Dictionary<String, bool> alreadySeen = new Dictionary<string, bool>();
            for (int i = 0; i < Math.Min(GetItemCount(), 1000); i++)
            {
                String str = column.GetStringValue(GetModelObject(i));
                if (!alreadySeen.ContainsKey(str))
                {
                    tb.AutoCompleteCustomSource.Add(str);
                    alreadySeen[str] = true;
                }
            }

            tb.AutoCompleteSource = AutoCompleteSource.CustomSource;
            tb.AutoCompleteMode = AutoCompleteMode.Append;

            return tb;
        }

        /// <summary>
        /// Stop editing a cell and throw away any changes.
        /// </summary>
        protected void CancelCellEdit()
        {
            // Let the world know that the user has cancelled the edit operation
            cellEditEventArgs.Cancel = true;
            OnCellEditFinishing(cellEditEventArgs);

            // Now cleanup the editing process
            CleanupCellEdit();
        }

        /// <summary>
        /// If a cell edit is in progress, finish the edit
        /// </summary>
        protected void PossibleFinishCellEditing()
        {
            if (IsCellEditing)
                FinishCellEdit();
        }

        /// <summary>
        /// Finish the cell edit operation, writing changed data back to the model object
        /// </summary>
        protected void FinishCellEdit()
        {
            OnCellEditFinishing(cellEditEventArgs);

            // If someone doesn't cancel the editing process, write the value back into the model
            if (!cellEditEventArgs.Cancel)
            {
                Object value = GetControlValue(cellEditor);
                cellEditEventArgs.Column.PutValue(cellEditEventArgs.RowObject, value);
                RefreshItem(cellEditEventArgs.ListViewItem);
            }

            CleanupCellEdit();
        }

        /// <summary>
        /// Remove all trace of any existing cell edit operation
        /// </summary>
        protected void CleanupCellEdit()
        {
            cellEditor.Leave -= new EventHandler(CellEditor_Leave);
            Controls.Remove(cellEditor);
            cellEditor.Dispose(); //THINK: do we need to call this?
            cellEditor = null;
            Select();
            PauseAnimations(false);
        }

        /// <summary>
        /// Triggered when a cell is about to be edited.
        /// </summary>
        /// <remarks>Set Cancel to true to prevent the cell being edited.
        /// You can change the the Control to be something completely different.</remarks>
        [Category("Behavior")]
        public event CellEditEventHandler CellEditStarting;

        /// <summary>
        /// Triggered when a cell is about to finish being edited.
        /// </summary>
        /// <remarks>If Cancel is already true, the user is cancelling the edit operation.
        /// Set Cancel to true to prevent the value from the cell being written into the model.
        /// You cannot prevent the editing from finishing.</remarks>
        [Category("Behavior")]
        public event CellEditEventHandler CellEditFinishing;

        /// <summary>
        /// Tell the world when a cell is about to be edited.
        /// </summary>
        protected virtual void OnCellEditStarting(CellEditEventArgs e)
        {
            if (CellEditStarting != null)
                CellEditStarting(this, e);
        }

        /// <summary>
        /// Tell the world when a cell is about to finish being edited.
        /// </summary>
        protected virtual void OnCellEditFinishing(CellEditEventArgs e)
        {
            if (CellEditFinishing != null)
                CellEditFinishing(this, e);
        }

        /// <summary>
        /// Return a collection of columns that are appropriate to the given view.
        /// Only Tile and Details have columns; all other views have 0 columns.
        /// </summary>
        /// <param name="view">Which view are the columns being calculate for?</param>
        /// <returns>A list of columns</returns>
        public List<OLVColumn> GetFilteredColumns(View view)
        {
            // For both detail and tile view, the first column must be included. Normally, we would
            // use the ColumnHeader.Index property, but if the header is not currently part of a ListView
            // that property returns -1. So, we track the index of
            // the column header, and always include the first header.

            int index = 0;
            switch (view)
            {
                case View.Details:
                    return AllColumns.FindAll(delegate(OLVColumn x) { return (index++ == 0) || x.IsVisible; });
                case View.Tile:
                    return AllColumns.FindAll(delegate(OLVColumn x) { return (index++ == 0) || x.IsTileViewColumn; });
                default:
                    return new List<OLVColumn>();
                    ;
            }
        }

        #region Nested type: AutoCompleteCellEditor

        /// <summary>
        /// This editor shows and auto completes values from the given listview column.
        /// </summary>
        public class AutoCompleteCellEditor : ComboBox
        {
            public AutoCompleteCellEditor(ObjectListView lv, OLVColumn column)
            {
                DropDownStyle = ComboBoxStyle.DropDown;

                Dictionary<String, bool> alreadySeen = new Dictionary<string, bool>();
                for (int i = 0; i < Math.Min(lv.GetItemCount(), 1000); i++)
                {
                    String str = column.GetStringValue(lv.GetModelObject(i));
                    if (!alreadySeen.ContainsKey(str))
                    {
                        Items.Add(str);
                        alreadySeen[str] = true;
                    }
                }

                Sorted = true;
                AutoCompleteSource = AutoCompleteSource.ListItems;
                AutoCompleteMode = AutoCompleteMode.Append;
            }
        }

        #endregion

        #region Nested type: BooleanCellEditor

        /// <summary>
        /// This editor simply shows and edits boolean values.
        /// </summary>
        /// <remarks>You can intercept the CellEditStarting event if you want
        /// to change the characteristics of the editor. For example, by changing
        /// the labels to "No" and "Yes". The false value must come first.</remarks>
        public class BooleanCellEditor : ComboBox
        {
            public BooleanCellEditor()
            {
                DropDownStyle = ComboBoxStyle.DropDownList;
                Items.AddRange(new String[] {"False", "True"}); // needs to be localizable
            }

            public bool Value
            {
                get { return SelectedIndex == 1; }
                set
                {
                    if (value)
                        SelectedIndex = 1;
                    else
                        SelectedIndex = 0;
                }
            }
        }

        #endregion

        #region Nested type: CellEditEventArgs

        /// <summary>
        /// Let the world know that a cell edit operation is beginning or ending
        /// </summary>
        public class CellEditEventArgs : EventArgs
        {
            /// <summary>
            /// Change this to true to cancel the cell editing operation.
            /// </summary>
            /// <remarks>
            /// <para>During the CellEditStarting event, setting this to true will prevent the cell from being edited.</para>
            /// <para>During the CellEditFinishing event, if this value is already true, this indicates that the user has 
            /// cancelled the edit operation and that the handler should perform cleanup only. Setting this to true,
            /// will prevent the ObjectListView from trying to write the new value into the model object.</para>
            /// </remarks>
            public bool Cancel = false;

            private Rectangle cellBounds;
            private OLVColumn column;

            /// <summary>
            /// During the CellEditStarting event, this can be modified to be the control that you want
            /// to edit the value. You must fully configure the control before returning from the event,
            /// including its bounds and the value it is showing.
            /// During the CellEditFinishing event, you can use this to get the value that the user
            /// entered and commit that value to the model. Changing the control during the finishing
            /// event has no effect.
            /// </summary>
            public Control Control = null;

            private OLVListItem listViewItem;
            private Object rowObject;
            private int subItemIndex;
            private Object value;

            public CellEditEventArgs(OLVColumn column, Control c, Rectangle r, OLVListItem item, int subItemIndex)
            {
                Cancel = false;
                Control = c;
                this.column = column;
                cellBounds = r;
                listViewItem = item;
                rowObject = item.RowObject;
                this.subItemIndex = subItemIndex;
                value = column.GetValue(item.RowObject);
            }

            /// <summary>
            /// The column of the cell that is going to be or has been edited.
            /// </summary>
            public OLVColumn Column
            {
                get { return column; }
            }

            /// <summary>
            /// The model object of the row of the cell that is going to be or has been edited.
            /// </summary>
            public Object RowObject
            {
                get { return rowObject; }
            }

            /// <summary>
            /// The listview item of the cell that is going to be or has been edited.
            /// </summary>
            public OLVListItem ListViewItem
            {
                get { return listViewItem; }
            }

            /// <summary>
            /// The index of the cell that is going to be or has been edited.
            /// </summary>
            public int SubItemIndex
            {
                get { return subItemIndex; }
            }

            /// <summary>
            /// The data value of the cell before the edit operation began.
            /// </summary>
            public Object Value
            {
                get { return value; }
            }

            /// <summary>
            /// The bounds of the cell that is going to be or has been edited.
            /// </summary>
            public Rectangle CellBounds
            {
                get { return cellBounds; }
            }
        }

        #endregion

        #region Nested type: FloatCellEditor

        /// <summary>
        /// This editor simply shows and edits floating point values.
        /// </summary>
        /// <remarks>You can intercept the CellEditStarting event if you want
        /// to change the characteristics of the editor. For example, by increasing
        /// the number of decimal places.</remarks>
        public class FloatCellEditor : NumericUpDown
        {
            public FloatCellEditor()
            {
                DecimalPlaces = 2;
                Minimum = -9999999;
                Maximum = 9999999;
            }

            public new double Value
            {
                get { return Convert.ToDouble(base.Value); }
                set { base.Value = Convert.ToDecimal(value); }
            }
        }

        #endregion

        //-----------------------------------------------------------------------
        // Cell editors
        // These classes are simple cell editors that make it easier to get and set
        // the value that the control is showing.

        #region Nested type: IntUpDown

        /// <summary>
        /// This editor simply shows and edits integer values.
        /// </summary>
        public class IntUpDown : NumericUpDown
        {
            public IntUpDown()
            {
                DecimalPlaces = 0;
                Minimum = -9999999;
                Maximum = 9999999;
            }

            public new int Value
            {
                get { return Decimal.ToInt32(base.Value); }
                set { base.Value = new Decimal(value); }
            }
        }

        #endregion

        #region Nested type: UintUpDown

        /// <summary>
        /// This editor simply shows and edits unsigned integer values.
        /// </summary>
        public class UintUpDown : NumericUpDown
        {
            public UintUpDown()
            {
                DecimalPlaces = 0;
                Minimum = 0;
                Maximum = 9999999;
            }

            public new uint Value
            {
                get { return Decimal.ToUInt32(base.Value); }
                set { base.Value = new Decimal(value); }
            }
        }

        #endregion

        #endregion

        #region Design Time

        /// <summary>
        /// Return Columns for this list. We hide the original so we can associate
        /// a specialised editor with it.
        /// </summary>
        [Editor(typeof (OLVColumnCollectionEditor), typeof (UITypeEditor))]
        public new ColumnHeaderCollection Columns
        {
            get { return base.Columns; }
        }

        /// <summary>
        /// This class works in conjunction with the OLVColumns property to allow OLVColumns
        /// to be added to the ObjectListView.
        /// </summary>
        internal class OLVColumnCollectionEditor : CollectionEditor
        {
            public OLVColumnCollectionEditor(Type t) : base(t) {}

            protected override Type CreateCollectionItemType()
            {
                return typeof (OLVColumn);
            }

            public override object EditValue(ITypeDescriptorContext context, IServiceProvider provider, object value)
            {
                ColumnHeaderCollection cols = (ColumnHeaderCollection) value;

                // Hack to figure out which ObjectListView we are working on
                ObjectListView olv;
                if (cols.Count == 0)
                {
                    cols.Add(new OLVColumn());
                    olv = (ObjectListView) cols[0].ListView;
                    cols.Clear();
                    olv.AllColumns.Clear();
                }
                else
                    olv = (ObjectListView) cols[0].ListView;

                // Edit all the columns, not just the ones that are visible
                base.EditValue(context, provider, olv.AllColumns);

                // Calculate just the visible columns
                List<OLVColumn> newColumns = olv.GetFilteredColumns(View.Details);
                olv.Columns.Clear();
                olv.Columns.AddRange(newColumns.ToArray());

                return olv.Columns;
            }
        }

        #endregion

        #endregion
    }

    /// <summary>
    /// Wrapper for all native method calls on ListView controls
    /// </summary>
    internal class NativeMethods
    {
        private const int HDF_BITMAP = 0x2000;
        private const int HDF_BITMAP_ON_RIGHT = 0x1000;
        private const int HDF_CENTER = 0x0002;
        private const int HDF_IMAGE = 0x0800;
        private const int HDF_JUSTIFYMASK = 0x0003;
        private const int HDF_LEFT = 0x0000;
        private const int HDF_RIGHT = 0x0001;
        private const int HDF_RTLREADING = 0x0004;
        private const int HDF_SORTDOWN = 0x0200;
        private const int HDF_SORTUP = 0x0400;
        private const int HDF_STRING = 0x4000;
        private const int HDI_BITMAP = 0x0010;
        private const int HDI_FORMAT = 0x0004;
        private const int HDI_IMAGE = 0x0020;
        private const int HDI_TEXT = 0x0002;
        private const int HDI_WIDTH = 0x0001;
        private const int HDM_FIRST = 0x1200;
        private const int HDM_GETITEM = HDM_FIRST + 11;
        private const int HDM_HITTEST = HDM_FIRST + 6;
        private const int HDM_SETITEM = HDM_FIRST + 12;
        private const int LVCF_FMT = 0x0001;
        private const int LVCF_IMAGE = 0x0010;
        private const int LVCF_ORDER = 0x0020;
        private const int LVCF_SUBITEM = 0x0008;
        private const int LVCF_TEXT = 0x0004;
        private const int LVCF_WIDTH = 0x0002;
        private const int LVCFMT_BITMAP_ON_RIGHT = 0x1000;
        private const int LVCFMT_CENTER = 0x0002;
        private const int LVCFMT_COL_HAS_IMAGES = 0x8000;
        private const int LVCFMT_IMAGE = 0x0800;
        private const int LVCFMT_JUSTIFYMASK = 0x0003;
        private const int LVCFMT_LEFT = 0x0000;
        private const int LVCFMT_RIGHT = 0x0001;
        private const int LVIF_IMAGE = 0x0002;
        private const int LVIF_INDENT = 0x0010;
        private const int LVIF_NORECOMPUTE = 0x0800;
        private const int LVIF_PARAM = 0x0004;
        private const int LVIF_STATE = 0x0008;
        private const int LVIF_TEXT = 0x0001;
        private const int LVM_FIRST = 0x1000;
        private const int LVM_GETCOLUMN = LVM_FIRST + 95;
        private const int LVM_GETHEADER = LVM_FIRST + 31;
        private const int LVM_SETCOLUMN = LVM_FIRST + 96;
        private const int LVM_SETEXTENDEDLISTVIEWSTYLE = LVM_FIRST + 54;
        private const int LVM_SETITEM = LVM_FIRST + 76;
        private const int LVM_SETITEMSTATE = LVM_FIRST + 43;
        private const int LVS_EX_SUBITEMIMAGES = 0x0002;

        // Various flavours of SendMessage: plain vanilla,
        // others pass references to various structures
        [DllImport("user32.dll", CharSet=CharSet.Auto)]
        private static extern IntPtr SendMessage(IntPtr hWnd, int msg, int wParam, int lParam);

        [DllImport("user32.dll", EntryPoint="SendMessage", CharSet=CharSet.Auto)]
        private static extern IntPtr SendMessageLVItem(IntPtr hWnd, int msg, int wParam, ref LVITEM lvi);

        [DllImport("user32.dll", EntryPoint = "SendMessage", CharSet = CharSet.Auto)]
        private static extern IntPtr SendMessageLVColumn(IntPtr hWnd, int msg, int wParam, ref LVCOLUMN lvc);

        [DllImport("user32.dll", EntryPoint = "SendMessage", CharSet = CharSet.Auto)]
        private static extern IntPtr SendMessageHDItem(IntPtr hWnd, int msg, int wParam, ref HDITEM hdi);

        [DllImport("user32.dll", EntryPoint = "SendMessage", CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessageHDHITTESTINFO(IntPtr hWnd, int Msg, IntPtr wParam,
                                                             [In, Out] HDHITTESTINFO lParam);

        /// <summary>
        /// Make sure the ListView has the extended style that says to display subitem images.
        /// </summary>
        /// <remarks>This method must be called after any .NET call that update the extended styles
        /// since they seem to erase this setting.</remarks>
        /// <param name="list">The listview to send a message to</param>
        public static void ForceSubItemImagesExStyle(ListView list)
        {
            SendMessage(list.Handle, LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_SUBITEMIMAGES, LVS_EX_SUBITEMIMAGES);
        }

        /// <summary>
        /// For the given item and subitem, make it display the given image
        /// </summary>
        /// <param name="list">The listview to send a message to</param>
        /// <param name="itemIndex">row number (0 based)</param>
        /// <param name="subItemIndex">subitem (0 is the item itself)</param>
        /// <param name="imageIndex">index into the image list</param>
        public static void SetSubItemImage(ListView list, int itemIndex, int subItemIndex, int imageIndex)
        {
            LVITEM lvItem = new LVITEM();
            lvItem.mask = LVIF_IMAGE;
            lvItem.iItem = itemIndex;
            lvItem.iSubItem = subItemIndex;
            lvItem.iImage = imageIndex;
            SendMessageLVItem(list.Handle, LVM_SETITEM, 0, ref lvItem);
        }

        /// <summary>
        /// Setup the given column of the listview to show the given image to the right of the text.
        /// If the image index is -1, any previous image is cleared
        /// </summary>
        /// <param name="list">The listview to send a message to</param>
        /// <param name="columnIndex">Index of the column to modifiy</param>
        /// <param name="imageIndex">Index into the small image list</param>
        public static void SetColumnImage(ListView list, int columnIndex, SortOrder order, int imageIndex)
        {
            IntPtr hdrCntl = GetHeaderControl(list);
            if (hdrCntl.ToInt32() == 0)
                return;

            HDITEM item = new HDITEM();
            item.mask = HDI_FORMAT;
            IntPtr result = SendMessageHDItem(hdrCntl, HDM_GETITEM, columnIndex, ref item);

            item.fmt &= ~(HDF_SORTUP | HDF_SORTDOWN | HDF_IMAGE | HDF_BITMAP_ON_RIGHT);

            if (HasBuiltinSortIndicators())
            {
                if (order == SortOrder.Ascending)
                    item.fmt |= HDF_SORTUP;
                if (order == SortOrder.Descending)
                    item.fmt |= HDF_SORTDOWN;
            }
            else
            {
                item.mask |= HDI_IMAGE;
                item.fmt |= (HDF_IMAGE | HDF_BITMAP_ON_RIGHT);
                item.iImage = imageIndex;
            }

            result = SendMessageHDItem(hdrCntl, HDM_SETITEM, columnIndex, ref item);
        }

        /// <summary>
        /// Does this version of the operating system have builtin sort indicators?
        /// </summary>
        /// <returns>Are there builtin sort indicators</returns>
        /// <remarks>XP and later have these</remarks>
        public static bool HasBuiltinSortIndicators()
        {
            return OSFeature.Feature.GetVersionPresent(OSFeature.Themes) != null;
        }

        [DllImport("user32.dll", EntryPoint = "GetUpdateRect", CharSet = CharSet.Auto)]
        private static extern IntPtr GetUpdateRectInternal(IntPtr hWnd, ref Rectangle r, bool eraseBackground);

        /// <summary>
        /// Return the bounds of the update region on the given control.
        /// </summary>
        /// <remarks>The BeginPaint() system call validates the update region, effectively wiping out this information.
        /// So this call has to be made before the BeginPaint() call.</remarks>
        /// <param name="cntl">The control whose update region is be calculated</param>
        /// <returns>A rectangle</returns>
        public static Rectangle GetUpdateRect(Control cntl)
        {
            Rectangle r = new Rectangle();
            GetUpdateRectInternal(cntl.Handle, ref r, false);
            return r;
        }

        [DllImport("user32.dll", EntryPoint = "ValidateRect", CharSet = CharSet.Auto)]
        private static extern IntPtr ValidatedRectInternal(IntPtr hWnd, ref Rectangle r);

        /// <summary>
        /// Validate an area of the given control. A validated area will not be repainted at the next redraw.
        /// </summary>
        /// <param name="cntl">The control to be validated</param>
        /// <param name="r">The area of the control to be validated</param>
        public static void ValidateRect(Control cntl, Rectangle r)
        {
            ValidatedRectInternal(cntl.Handle, ref r);
        }

        /// <summary>
        /// Select all rows on the given listview
        /// </summary>
        /// <param name="list">The listview whose items are to be selected</param>
        public static void SelectAllItems(ListView list)
        {
            SetItemState(list, -1, 2, 2);
        }

        /// <summary>
        /// Deselect all rows on the given listview
        /// </summary>
        /// <param name="list">The listview whose items are to be deselected</param>
        public static void DeselectAllItems(ListView list)
        {
            SetItemState(list, -1, 2, 0);
        }

        /// <summary>
        /// Set the item state on the given item
        /// </summary>
        /// <param name="list">The listview whose item's state is to be changed</param>
        /// <param name="itemIndex">The index of the item to be changed</param>
        /// <param name="mask">Which bits of the value are to be set?</param>
        /// <param name="value">The value to be set</param>
        public static void SetItemState(ListView list, int itemIndex, int mask, int value)
        {
            LVITEM lvItem = new LVITEM();
            lvItem.stateMask = mask;
            lvItem.state = value;
            SendMessageLVItem(list.Handle, LVM_SETITEMSTATE, itemIndex, ref lvItem);
        }

        /// <summary>
        /// Return the handle to the header control on the given list
        /// </summary>
        /// <param name="list">The listview whose header control is to be returned</param>
        /// <returns>The handle to the header control</returns>
        public static IntPtr GetHeaderControl(ListView list)
        {
            return SendMessage(list.Handle, LVM_GETHEADER, 0, 0);
        }

        /// <summary>
        /// Return the index of the divider under the given point. Return -1 if no divider is under the pt
        /// </summary>
        /// <param name="list">The list we are interested in</param>
        /// <param name="pt">The client co-ords</param>
        /// <returns>The index of the divider under the point, or -1 if no divider is under that point</returns>
        public static int GetDividerUnderPoint(IntPtr handle, Point pt)
        {
            const int HHT_ONDIVIDER = 4;

            HDHITTESTINFO testInfo = new HDHITTESTINFO();
            testInfo.pt_x = pt.X;
            testInfo.pt_y = pt.Y;
            IntPtr result = SendMessageHDHITTESTINFO(handle, HDM_HITTEST, IntPtr.Zero, testInfo);
            if ((testInfo.flags & HHT_ONDIVIDER) != 0)
                return result.ToInt32();
            else
                return -1;
        }

        #region Nested type: HDHITTESTINFO

        [StructLayout(LayoutKind.Sequential)]
        public class HDHITTESTINFO
        {
            public int flags;
            public int iItem;
            public int pt_x;
            public int pt_y;
        }

        #endregion

        #region Nested type: HDITEM

        [StructLayout(LayoutKind.Sequential)]
        public struct HDITEM
        {
            public int cchTextMax;
            public int cxy;
            public int fmt;
            public IntPtr hbm;
            public int iImage;
            public int iOrder;
            public IntPtr lParam;
            public int mask;
            public IntPtr pszText;
            //if (_WIN32_IE >= 0x0500)
            public IntPtr pvFilter;
            public int type;
        }

        #endregion

        #region Nested type: LVCOLUMN

        [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
        private struct LVCOLUMN
        {
            public int cchTextMax;
            public int cx;
            public int fmt;
            // These are available in Common Controls >= 0x0300
            public int iImage;
            public int iOrder;
            public int iSubItem;
            public int mask;

            [MarshalAs(UnmanagedType.LPTStr)]
            public string pszText;
        } ;

        #endregion

        #region Nested type: LVITEM

        [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Auto)]
        private struct LVITEM
        {
            public int cchTextMax;
            public int cColumns;
            public int iGroupId;
            public int iImage;
            // These are available in Common Controls >= 0x0300
            public int iIndent;
            public int iItem;
            public int iSubItem;
            public int lParam;
            public int mask;

            [MarshalAs(UnmanagedType.LPTStr)]
            public string pszText;

            // These are available in Common Controls >= 0x056
            public IntPtr puColumns;
            public int state;
            public int stateMask;
        } ;

        #endregion

        #region Nested type: NMHDR

        /// <summary>
        /// Notify message header structure.
        /// </summary>
        [StructLayout(LayoutKind.Sequential)]
        public struct NMHDR
        {
            public int code;
            public IntPtr hwndFrom;
            public int idFrom;
        }

        #endregion

        #region Nested type: NMHEADER

        [StructLayout(LayoutKind.Sequential)]
        public struct NMHEADER
        {
            public int iButton;
            public int iItem;
            public NMHDR nhdr;
            public IntPtr pHDITEM;
        }

        #endregion
    }

    /// <summary>
    /// A virtual object list view operates in virtual mode, that is, it only gets model objects for
    /// a row when it is needed. This gives it the ability to handle very large numbers of rows with
    /// minimal resources.
    /// </summary>
    /// <remarks><para>A listview is not a great user interface for a large number of items. But if you've
    /// ever wanted to have a list with 10 million items, go ahead, knock yourself out.</para>
    /// <para>Virtual lists can never iterate their contents. That would defeat the whole purpose.</para>
    /// <para>Given the above, grouping and sorting are not possible on virtual lists. But if the backing data store has
    /// a sorting mechanism, a CustomSorter can be installed which will be called when the sorting is required.</para>
    /// <para>For the same reason, animate GIFs should not be used in virtual lists. Animated GIFs require some state
    /// information to be stored for each animation, but virtual lists specifically do not keep any state information.
    /// You really do not want to keep state information for 10 million animations!</para>
    /// </remarks>
    public class VirtualObjectListView : ObjectListView
    {
        /// <summary>
        /// Create a VirtualObjectListView
        /// </summary>
        public VirtualObjectListView() : base()
        {
            ShowGroups = false; // virtual lists can never show groups
            VirtualMode = true;
            RetrieveVirtualItem += new RetrieveVirtualItemEventHandler(HandleRetrieveVirtualItem);

            // Install a null custom sorter to turn off sorting. Who wants to fetch and sort 10 million items?
            CustomSorter = delegate(OLVColumn column, SortOrder sortOrder) { };
        }

        #region Public Properties

        /// <summary>
        /// This delegate is used to fetch a rowObject, given it's index within the list
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public RowGetterDelegate RowGetter
        {
            get { return rowGetter; }
            set { rowGetter = value; }
        }

        #endregion

        #region OLV accessing

        /// <summary>
        /// Return the number of items in the list
        /// </summary>
        /// <returns>the number of items in the list</returns>
        public override int GetItemCount()
        {
            return VirtualListSize;
        }

        /// <summary>
        /// Return the item at the given index
        /// </summary>
        /// <param name="name">Index of the item to be returned</param>
        /// <returns>An OLVListItem</returns>
        public override OLVListItem GetItem(int index)
        {
            return MakeListViewItem(index);
        }

        /// <summary>
        /// Return the model object at the given index
        /// </summary>
        /// <param name="name">Index of the model object to be returned</param>
        /// <returns>A model object</returns>
        public override object GetModelObject(int index)
        {
            return GetRowObjectAt(index);
        }

        #endregion

        #region Object manipulation

        /// <summary>
        /// Select the row that is displaying the given model object.
        /// This does nothing in virtual lists.
        /// </summary>
        /// <remarks>This is a no-op for virtual lists, since there is no way to map the model
        /// object back to the ListViewItem that represents it.</remarks>
        /// <param name="modelObject">The object that gave data</param>
        public override void SelectObject(object modelObject)
        {
            // do nothing
        }

        /// <summary>
        /// Select the rows that is displaying any of the given model object.
        /// This does nothing in virtual lists.
        /// </summary>
        /// <remarks>This is a no-op for virtual lists, since there is no way to map the model
        /// objects back to the ListViewItem that represents them.</remarks>
        /// <param name="modelObjects">A collection of model objects</param>
        public override void SelectObjects(IList modelObjects)
        {
            // do nothing
        }

        /// <summary>
        /// Update the rows that are showing the given objects
        /// </summary>
        /// <remarks>This is a no-op for virtual lists, since there is no way to map the model
        /// objects back to the ListViewItem that represents them.</remarks>
        public override void RefreshObjects(IList modelObjects)
        {
            // do nothing
        }

        #endregion

        /// <summary>
        /// Invalidate any cached information when we rebuild the list.
        /// </summary>
        public override void BuildList(bool shouldPreserveSelection)
        {
            lastRetrieveVirtualItemIndex = -1;

            // This call was in the older code, but I can't think why since virtual lists
            // can't be build in that fashion.
            //base.BuildList(false);
        }

        /// <summary>
        /// Prepare the listview to show alternate row backcolors
        /// </summary>
        /// <remarks>Alternate colored backrows can't be handle in the same way as our base class.
        /// With virtual lists, they are handled at RetrieveVirtualItem time.</remarks>
        protected override void PrepareAlternateBackColors()
        {
            // do nothing
        }

        #region Event handlers

        /// <summary>
        /// Handle a RetrieveVirtualItem
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void HandleRetrieveVirtualItem(object sender, RetrieveVirtualItemEventArgs e)
        {
            // .NET 2.0 seems to generate a lot of these events. Before drawing *each* sub-item,
            // this event is triggered 4-8 times for the same index. So we save lots of CPU time
            // by caching the last result.
            if (lastRetrieveVirtualItemIndex != e.ItemIndex)
            {
                lastRetrieveVirtualItemIndex = e.ItemIndex;
                lastRetrieveVirtualItem = MakeListViewItem(e.ItemIndex);
            }
            e.Item = lastRetrieveVirtualItem;
        }

        /// <summary>
        /// Create a OLVListItem for given row index
        /// </summary>
        /// <param name="itemIndex">The index of the row that is needed</param>
        /// <returns>An OLVListItem</returns>
        public OLVListItem MakeListViewItem(int itemIndex)
        {
            OLVListItem olvi = new OLVListItem(GetRowObjectAt(itemIndex));
            FillInValues(olvi, olvi.RowObject);
            if (UseAlternatingBackColors)
            {
                if (View == View.Details && itemIndex%2 == 1)
                    olvi.BackColor = AlternateRowBackColorOrDefault;
                else
                    olvi.BackColor = BackColor;

                // For some reason, UseItemStyleForSubItems doesn't work for the background color
                // when owner drawing the list, so we have to specifically give each subitem
                // the desired color
                if (OwnerDraw && olvi.UseItemStyleForSubItems)
                {
                    foreach (ListViewItem.ListViewSubItem si in olvi.SubItems)
                        si.BackColor = olvi.BackColor;
                }
            }
            SetSubItemImages(itemIndex, olvi);
            return olvi;
        }

        /// <summary>
        /// Return the row object for the given row index
        /// </summary>
        /// <param name="index">index of the row whose object is to be fetched</param>
        /// <returns>A model object or null if no delegate is installed</returns>
        protected virtual object GetRowObjectAt(int index)
        {
            if (RowGetter == null)
                return null;
            else
                return RowGetter(index);
        }

        #endregion

        #region Variable declaractions

        private OLVListItem lastRetrieveVirtualItem;
        protected int lastRetrieveVirtualItemIndex = -1;
        private RowGetterDelegate rowGetter;

        #endregion
    }

    /// <summary>
    /// A FastObjectListView trades function for speed. 
    /// </summary>
    /// <remarks>
    /// <para>On my mid-range laptop, this view builds a list of 10,000 objects in 0.1 seconds, 
    /// as opposed to a normal ObjectListView which takes 10-15 seconds. Lists of up to 50,000 items should be
    /// able to be handled with sub-second response times even on low end machines.</para>
    /// <para>
    /// A FastObjectListView is implemented as a virtual list with some of the virtual modes limits (e.g. no sorting)
    /// fixed through coding. There are some functions that simply cannot be provided. Specifically, a FastObjectListView cannot:
    /// <list>
    /// <item>shows groups</item>
    /// <item>use Tile view</item>
    /// <item>display images on subitems</item>
    /// </list>
    /// </para>
    /// <para>You can circumvent the limit on subitem images by making the list owner drawn, and giving the column
    /// a Renderer of BaseRenderer, e.g. <code>myColumnWithImage.Renderer = new BaseRenderer();</code> </para>
    /// </remarks>
    public class FastObjectListView : VirtualObjectListView
    {
        private Hashtable objectsToIndexMap = new Hashtable();

        #region Public properties

        private bool isSearchOnSortColumn = false;
        private ArrayList objectList = new ArrayList();

        /// <summary>
        /// Get/set the list of objects that are shown by the control.
        /// </summary>
        /// <remarks>This method preserves selection, if possible. Use <code>SetObjects()</code> if
        /// you do not want to preserve the selection. Preserving selection is the slowest part of this
        /// code and performance is O(n) where n is the number of selected rows.</remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public new ArrayList Objects
        {
            get { return objectList; }
            set
            {
                ArrayList previousSelection = SelectedObjects;
                SetObjects(value);
                SelectedObjects = previousSelection;
            }
        }

        /// <summary>
        /// When the user types into a list, should the values in the current sort column be searched to find a match?
        /// If this is false, the primary column will always be used regardless of the sort column.
        /// </summary>
        /// <remarks>When this is true, the behavior is like that of ITunes.</remarks>
        [Category("Behavior"),
         Description(
             "When the user types into a list, should the values in the current sort column be searched to find a match?"
             ),
         DefaultValue(false)]
        public bool IsSearchOnSortColumn
        {
            get { return isSearchOnSortColumn; }
            set { isSearchOnSortColumn = value; }
        }

        #endregion

        #region Commands

        /// <summary>
        /// Set the collection of objects that this control will show.
        /// </summary>
        /// <param name="collection"></param>
        public override void SetObjects(IEnumerable collection)
        {
            if (collection == null)
                objectList = new ArrayList();
            else if (collection is ICollection)
                objectList = new ArrayList((ICollection) collection);
            else
            {
                objectList = new ArrayList();
                foreach (object x in collection)
                    objectList.Add(x);
            }
            VirtualListSize = objectList.Count;
            Sort();
        }

        #endregion

        #region Event Handlers

        /// <summary>
        /// Event handler for the column click event
        /// </summary>
        /// <remarks>
        /// This differs from its base version by trying to preserve selection. The base class,
        /// being a pure virtual list, cannot maintain selection since it cannot map a 
        /// model objects to the row that is responsible for displaying it. This class can do that.
        /// </remarks>
        protected override void HandleColumnClick(object sender, ColumnClickEventArgs e)
        {
            // Toggle the sorting direction on successive clicks on the same column
            if (lastSortColumn != null && e.Column == lastSortColumn.Index)
                lastSortOrder = (lastSortOrder == SortOrder.Descending ? SortOrder.Ascending : SortOrder.Descending);
            else
                lastSortOrder = SortOrder.Ascending;

            BeginUpdate();
            ArrayList previousSelection = SelectedObjects;
            Sort(e.Column);
            SelectedObjects = previousSelection;
            EndUpdate();
        }

        private void FastObjectListView_SearchForVirtualItem(object sender, SearchForVirtualItemEventArgs e)
        {
            // The event has e.IsPrefixSearch, but as far as I can tell, this is always false (maybe that's different under Vista)
            // So we ignore IsPrefixSearch and IsTextSearch and always to a case insensitve prefix match

            int increment = (e.Direction == SearchDirectionHint.Up ? -1 : 1);
            OLVColumn column = GetColumn(0);
            if (IsSearchOnSortColumn && View == View.Details && lastSortColumn != null)
                column = lastSortColumn;

            int i;
            for (i = e.StartIndex; i >= 0 && i < objectList.Count; i += increment)
            {
                string data = column.GetStringValue(objectList[i]);
                if (data.StartsWith(e.Text, StringComparison.CurrentCultureIgnoreCase))
                {
                    e.Index = i;
                    return;
                }
            }

            // Also the LVFINDINFO has a LV_WRAP flag, but the SearchForVirtualItemEventArgs does not. Why?
            // We always wrap
            i = (increment < 0 ? objectList.Count : 0);
            while ((increment < 0 && i > e.StartIndex) || (increment > 0 && i < e.StartIndex))
            {
                string data = column.GetStringValue(objectList[i]);
                if (data.StartsWith(e.Text, StringComparison.CurrentCultureIgnoreCase))
                {
                    e.Index = i;
                    return;
                }
                i += increment;
            }
        }

        #endregion

        #region Object manipulation

        /// <summary>
        /// Select the row that is displaying the given model object.
        /// </summary>
        /// <param name="modelObject">The object that gave data</param>
        public override void SelectObject(object modelObject)
        {
            if (!objectsToIndexMap.ContainsKey(modelObject))
                return;

            int index = (int) objectsToIndexMap[modelObject];

            // If this object is already selected, we don't need to do anything
            if (SelectedIndices.Count == 1 && SelectedIndices[0] == index)
                return;

            SelectedIndices.Clear();
            if (index >= 0)
                SelectedIndices.Add(index);
        }

        /// <summary>
        /// Select the rows that is displaying any of the given model object.
        /// </summary>
        /// <param name="modelObjects">A collection of model objects</param>
        public override void SelectObjects(IList modelObjects)
        {
            SelectedIndices.Clear();

            foreach (object model in modelObjects)
            {
                if (objectsToIndexMap.ContainsKey(model))
                {
                    int index = (int) objectsToIndexMap[model];
                    if (index >= 0)
                        SelectedIndices.Add(index);
                }
            }
        }

        /// <summary>
        /// Update the rows that are showing the given objects
        /// </summary>
        public override void RefreshObjects(IList modelObjects)
        {
            Invalidate();
        }

        #endregion

        #region Implementation

        /// <summary>
        /// Return the row object for the given row index
        /// </summary>
        /// <param name="index">index of the row whose object is to be fetched</param>
        /// <returns>A model object or null if no delegate is installed</returns>
        protected override object GetRowObjectAt(int index)
        {
            return objectList[index];
        }

        internal class ModelObjectComparer : IComparer
        {
            private OLVColumn column;
            private ModelObjectComparer secondComparer;
            private SortOrder sortOrder;

            public ModelObjectComparer(OLVColumn col, SortOrder order)
            {
                column = col;
                sortOrder = order;
                secondComparer = null;
            }

            public ModelObjectComparer(OLVColumn col, SortOrder order, OLVColumn col2, SortOrder order2)
                : this(col, order)
            {
                // There is no point in secondary sorting on the same column
                if (col != col2)
                    secondComparer = new ModelObjectComparer(col2, order2);
            }

            #region IComparer Members

            public int Compare(object x, object y)
            {
                int result = 0;
                object x1 = column.GetValue(x);
                object y1 = column.GetValue(y);

                if (sortOrder == SortOrder.None)
                    return 0;

                // Handle nulls. Null values come last
                bool xIsNull = (x1 == null || x1 == DBNull.Value);
                bool yIsNull = (y1 == null || y1 == DBNull.Value);
                if (xIsNull || yIsNull)
                {
                    if (xIsNull && yIsNull)
                        result = 0;
                    else
                        result = (xIsNull ? -1 : 1);
                }
                else result = CompareValues(x1, y1);

                if (sortOrder == SortOrder.Descending)
                    result = 0 - result;

                // If the result was equality, use the secondary comparer to resolve it
                if (result == 0 && secondComparer != null)
                    result = secondComparer.Compare(x, y);

                return result;
            }

            #endregion

            public int CompareValues(object x, object y)
            {
                // Force case insensitive compares on strings
                if (x is String)
                    return String.Compare((String) x, (String) y, true);
                else
                {
                    IComparable comparable = x as IComparable;
                    if (comparable != null)
                        return comparable.CompareTo(y);
                    else
                        return 0;
                }
            }
        }

        #endregion

        public FastObjectListView()
        {
            SearchForVirtualItem += new SearchForVirtualItemEventHandler(FastObjectListView_SearchForVirtualItem);

            CustomSorter = delegate(OLVColumn column, SortOrder sortOrder)
                           {
                               lastRetrieveVirtualItemIndex = -1;
                               if (sortOrder != SortOrder.None)
                                   objectList.Sort(
                                       new ModelObjectComparer(column, sortOrder, SecondarySortColumn,
                                                               SecondarySortOrder));
                               objectsToIndexMap.Clear();
                               for (int i = 0; i < objectList.Count; i++)
                                   objectsToIndexMap[objectList[i]] = i;
                           };
        }
    }

    /// <summary>
    /// A DataListView is a ListView that can be bound to a datasource (which would normally be a DataTable or DataView).
    /// </summary>
    /// <remarks>
    /// <para>This listview keeps itself in sync with its source datatable by listening for change events.</para>
    /// <para>If the listview has no columns when given a data source, it will automatically create columns to show all of the datatables columns.
    /// This will be only the simplest view of the world, and would look more interesting with a few delegates installed.</para>
    /// <para>This listview will also automatically generate missing aspect getters to fetch the values from the data view.</para>
    /// <para>Changing data sources is possible, but error prone. Before changing data sources, the programmer is responsible for modifying/resetting
    /// the column collection to be valid for the new data source.</para>
    /// </remarks>
    public class DataListView : ObjectListView
    {
        /// <summary>
        /// Make a DataListView
        /// </summary>
        public DataListView() : base() {}

        #region Public Properties

        private string dataMember = "";
        private Object dataSource;

        /// <summary>
        /// Get or set the DataSource that will be displayed in this list view.
        /// </summary>
        /// <remarks>The DataSource should implement either <see cref="IList"/>, <see cref="IBindingList"/>, 
        /// or <see cref="IListSource"/>. Some common examples are the following types of objects:
        /// <list type="unordered">
        /// <item><see cref="DataView"/></item>
        /// <item><see cref="DataTable"/></item>
        /// <item><see cref="DataSet"/></item>
        /// <item><see cref="DataViewManager"/></item>
        /// <item><see cref="BindingSource"/></item>
        /// </list>
        /// <para>When binding to a list container (i.e. one that implements the
        /// <see cref="IListSource"/> interface, such as <see cref="DataSet"/>)
        /// you must also set the <see cref="DataMember"/> property in order
        /// to identify which particular list you would like to display. You
        /// may also set the <see cref="DataMember"/> property even when
        /// DataSource refers to a list, since <see cref="DataMember"/> can
        /// also be used to navigate relations between lists.</para>
        /// </remarks>
        [Category("Data"), TypeConverter("System.Windows.Forms.Design.DataSourceConverter, System.Design")]
        public Object DataSource
        {
            get { return dataSource; }
            set
            {
                //THINK: Should we only assign it if it is changed?
                //if (dataSource != value) {
                dataSource = value;
                RebindDataSource(true);
                //}
            }
        }

        /// <summary>
        /// Gets or sets the name of the list or table in the data source for which the DataListView is displaying data.
        /// </summary>
        /// <remarks>If the data source is not a DataSet or DataViewManager, this property has no effect</remarks>
        [Category("Data"),
         Editor("System.Windows.Forms.Design.DataMemberListEditor, System.Design", typeof (UITypeEditor)),
         DefaultValue("")]
        public string DataMember
        {
            get { return dataMember; }
            set
            {
                if (dataMember != value)
                {
                    dataMember = value;
                    RebindDataSource();
                }
            }
        }

        #endregion

        #region Initialization

        private CurrencyManager currencyManager = null;

        /// <summary>
        /// Our data source has changed. Figure out how to handle the new source
        /// </summary>
        protected void RebindDataSource()
        {
            RebindDataSource(false);
        }

        /// <summary>
        /// Our data source has changed. Figure out how to handle the new source
        /// </summary>
        protected void RebindDataSource(bool forceDataInitialization)
        {
            if (BindingContext == null)
                return;

            // Obtain the CurrencyManager for the current data source.
            CurrencyManager tempCurrencyManager = null;

            if (DataSource != null) tempCurrencyManager = (CurrencyManager) BindingContext[DataSource, DataMember];

            // Has our currency manager changed?
            if (currencyManager != tempCurrencyManager)
            {
                // Stop listening for events on our old currency manager
                if (currencyManager != null)
                {
                    currencyManager.MetaDataChanged -= new EventHandler(currencyManager_MetaDataChanged);
                    currencyManager.PositionChanged -= new EventHandler(currencyManager_PositionChanged);
                    currencyManager.ListChanged -= new ListChangedEventHandler(currencyManager_ListChanged);
                }

                currencyManager = tempCurrencyManager;

                // Start listening for events on our new currency manager
                if (currencyManager != null)
                {
                    currencyManager.MetaDataChanged += new EventHandler(currencyManager_MetaDataChanged);
                    currencyManager.PositionChanged += new EventHandler(currencyManager_PositionChanged);
                    currencyManager.ListChanged += new ListChangedEventHandler(currencyManager_ListChanged);
                }

                // Our currency manager has changed so we have to initialize a new data source
                forceDataInitialization = true;
            }

            if (forceDataInitialization)
                InitializeDataSource();
        }

        /// <summary>
        /// The data source for this control has changed. Reconfigure the control for the new source
        /// </summary>
        protected void InitializeDataSource()
        {
            if (Frozen || currencyManager == null)
                return;

            CreateColumnsFromSource();
            CreateMissingAspectGettersAndPutters();
            SetObjects(currencyManager.List);

            // If we have some data, resize the new columns based on the data available.
            if (Items.Count > 0)
            {
                foreach (ColumnHeader column in Columns)
                {
                    if (column.Width == 0)
                        AutoResizeColumn(column.Index, ColumnHeaderAutoResizeStyle.ColumnContent);
                }
            }
        }

        /// <summary>
        /// Create columns for the listview based on what properties are available in the data source
        /// </summary>
        /// <remarks>
        /// <para>This method will not replace existing columns.</para>
        /// </remarks>
        protected void CreateColumnsFromSource()
        {
            if (currencyManager == null || Columns.Count != 0)
                return;

            PropertyDescriptorCollection properties = currencyManager.GetItemProperties();
            if (properties.Count == 0)
                return;

            for (int i = 0; i < properties.Count; i++)
            {
                // Make a stack variable to hold the property so it can be used in the AspectGetter delegate
                PropertyDescriptor property = properties[i];

                // Relationships to other tables turn up as IBindibleLists. Don't make columns to show them.
                // CHECK: Is this always true? What other things could be here? Constraints? Triggers?
                if (property.PropertyType == typeof (IBindingList))
                    continue;

                // Create a column 
                OLVColumn column = new OLVColumn(property.DisplayName, property.Name);
                column.Width = 0; // zero-width since we will resize it once we have some data
                column.AspectGetter = delegate(object row) { return property.GetValue(row); };
                // If our column is a BLOB, it could be an image, so assign a renderer to draw it. 
                // CONSIDER: Is this a common enough case to warrant this code?
                if (property.PropertyType == typeof (Byte[]))
                    column.Renderer = new ImageRenderer();

                // Add it to our list
                Columns.Add(column);
            }
        }

        /// <summary>
        /// Generate aspect getters and putters for any columns that are missing them (and for which we have
        /// enough information to actually generate a getter)
        /// </summary>
        protected void CreateMissingAspectGettersAndPutters()
        {
            for (int i = 0; i < Columns.Count; i++)
            {
                OLVColumn column = GetColumn(i);
                if (column.AspectGetter == null && !String.IsNullOrEmpty(column.AspectName))
                {
                    column.AspectGetter = delegate(object row)
                                          {
                                              try
                                              {
                                                  // In most cases, rows will be DataRowView objects
                                                  return ((DataRowView) row)[column.AspectName];
                                              }
                                              catch
                                              {
                                                  return column.GetAspectByName(row);
                                              }
                                          };
                }
                if (column.IsEditable && column.AspectPutter == null && !String.IsNullOrEmpty(column.AspectName))
                {
                    column.AspectPutter = delegate(object row, object newValue)
                                          {
                                              try
                                              {
                                                  // In most cases, rows will be DataRowView objects
                                                  ((DataRowView) row)[column.AspectName] = newValue;
                                              }
                                              catch
                                              {
                                                  column.PutAspectByName(row, newValue);
                                              }
                                          };
                }
            }
        }

        #endregion

        #region Event Handlers

        private bool isChangingIndex = false;

        /// <summary>
        /// What should we do when the list is unfrozen
        /// </summary>
        protected override void DoUnfreeze()
        {
            // Clear any previous currency manager so the rebind will always work from scratch
            RebindDataSource(true);
        }

        /// <summary>
        /// Handles binding context changes
        /// </summary>
        /// <param name="e">The EventArgs that will be passed to any handlers
        /// of the BindingContextChanged event.</param>
        protected override void OnBindingContextChanged(EventArgs e)
        {
            base.OnBindingContextChanged(e);

            // If our binding context changes, we must rebind, since we will
            // have a new currency managers, even if we are still bound to the
            // same data source.
            RebindDataSource(false);
        }

        /// <summary>
        /// Handles parent binding context changes
        /// </summary>
        /// <param name="e">Unused EventArgs.</param>
        protected override void OnParentBindingContextChanged(EventArgs e)
        {
            base.OnParentBindingContextChanged(e);

            // BindingContext is an ambient property - by default it simply picks
            // up the parent control's context (unless something has explicitly
            // given us our own). So we must respond to changes in our parent's
            // binding context in the same way we would changes to our own
            // binding context.
            RebindDataSource(false);
        }

        // CurrencyManager ListChanged event handler. 
        // Deals with fine-grained changes to list items.
        // It's actually difficult to deal with these changes in a fine-grained manner.
        // If our listview is grouped, then any change may make a new group appear or
        // an old group disappear. It is rarely enough to simply update the affected row.
        private void currencyManager_ListChanged(object sender, ListChangedEventArgs e)
        {
            switch (e.ListChangedType)
            {
                    // Well, usually fine-grained... The whole list has changed utterly, so reload it.
                case ListChangedType.Reset:
                    InitializeDataSource();
                    break;

                    // A single item has changed, so just refresh that.
                    // TODO: Even in this simple case, we should probably rebuild the list.
                case ListChangedType.ItemChanged:
                    Object changedRow = currencyManager.List[e.NewIndex];
                    RefreshObject(changedRow);
                    break;

                    // A new item has appeared, so add that.
                    // We get this event twice if certain grid controls are used to add a new row to a 
                    // datatable: once when the editing of a new row begins, and once again when that 
                    // editing commits. (If the user cancels the creation of the new row, we never see 
                    // the second creation.) We detect this by seeing if this is a view on a row in a 
                    // DataTable, and if it is, testing to see if it's a new row under creation.
                case ListChangedType.ItemAdded:
                    Object newRow = currencyManager.List[e.NewIndex];
                    DataRowView drv = newRow as DataRowView;
                    if (drv == null || !drv.IsNew)
                    {
                        // Either we're not dealing with a view on a data table, or this is the commit 
                        // notification. Either way, this is the final notification, so we want to
                        // handle the new row now!
                        InitializeDataSource();
                    }
                    break;

                    // An item has gone away.
                case ListChangedType.ItemDeleted:
                    InitializeDataSource();
                    break;

                    // An item has changed its index.
                case ListChangedType.ItemMoved:
                    InitializeDataSource();
                    break;

                    // Something has changed in the metadata.
                    // CHECK: When are these events actually fired?
                case ListChangedType.PropertyDescriptorAdded:
                case ListChangedType.PropertyDescriptorChanged:
                case ListChangedType.PropertyDescriptorDeleted:
                    InitializeDataSource();
                    break;
            }
        }

        // The CurrencyManager calls this if the data source looks
        // different. We just reload everything.
        // CHECK: Do we need this if we are handle ListChanged metadata events?
        private void currencyManager_MetaDataChanged(object sender, EventArgs e)
        {
            InitializeDataSource();
        }

        // Called by the CurrencyManager when the currently selected item
        // changes. We update the ListView selection so that we stay in sync
        // with any other controls bound to the same source.
        private void currencyManager_PositionChanged(object sender, EventArgs e)
        {
            int index = currencyManager.Position;

            // Make sure the index is sane (-1 pops up from time to time)
            if (index < 0 || index >= Items.Count)
                return;

            // Avoid recursion. If we are currently changing the index, don't
            // start the process again. 
            if (isChangingIndex)
                return;

            try
            {
                isChangingIndex = true;

                // We can't use the index directly, since our listview may be sorted
                SelectedObject = currencyManager.List[index];

                // THINK: Do we always want to bring it into view?
                if (SelectedItems.Count > 0)
                    SelectedItems[0].EnsureVisible();
            }
            finally
            {
                isChangingIndex = false;
            }
        }

        // Called by Windows Forms when the currently selected index of the
        // control changes. This usually happens because the user clicked on
        // the control. In this case we want to notify the CurrencyManager so
        // that any other bound controls will remain in sync. This method will
        // also be called when we changed our index as a result of a
        // notification that originated from the CurrencyManager, and in that
        // case we avoid notifying the CurrencyManager back!
        protected override void OnSelectedIndexChanged(EventArgs e)
        {
            base.OnSelectedIndexChanged(e);

            // Prevent recursion
            if (isChangingIndex)
                return;

            // If we are bound to a datasource, and only one item is selected,
            // tell the currency manager which item is selected.
            if (SelectedIndices.Count == 1 && currencyManager != null)
            {
                try
                {
                    isChangingIndex = true;

                    // We can't use the selectedIndex directly, since our listview may be sorted.
                    // So we have to find the index of the selected object within the original list.
                    currencyManager.Position = currencyManager.List.IndexOf(SelectedObject);
                }
                finally
                {
                    isChangingIndex = false;
                }
            }
        }

        #endregion
    }

    #region Delegate declarations

    /// <summary>
    /// These delegates are used to extract an aspect from a row object
    /// </summary>
    public delegate Object AspectGetterDelegate(Object rowObject);

    /// <summary>
    /// These delegates are used to put a changed value back into a model object 
    /// </summary>
    public delegate void AspectPutterDelegate(Object rowObject, Object newValue);

    /// <summary>
    /// These delegates can be used to convert an aspect value to a display string,
    /// instead of using the default ToString()
    /// </summary>
    public delegate string AspectToStringConverterDelegate(Object value);

    /// <summary>
    /// These delegates are used to the state of the checkbox for a row object.
    /// </summary>
    /// <remarks>For reasons known only to someone in Microsoft, we can only set
    /// a boolean on the ListViewItem to indicate it's "checked-ness", but when
    /// we receive update events, we have to use a tristate CheckState. So we can
    /// be told about an indeterminate state, but we can't set it ourselves.</remarks>
    public delegate bool CheckStateGetterDelegate(Object rowObject);

    /// <summary>
    /// These delegates are used to put a changed check state back into a model object 
    /// </summary>
    public delegate CheckState CheckStatePutterDelegate(Object rowObject, CheckState newValue);

    /// <summary>
    /// These delegates are used to retrieve the object that is the key of the group to which the given row belongs.
    /// </summary>
    public delegate Object GroupKeyGetterDelegate(Object rowObject);

    /// <summary>
    /// These delegates are used to convert a group key into a title for the group
    /// </summary>
    public delegate string GroupKeyToTitleConverterDelegate(Object groupKey);

    /// <summary>
    /// These delegates are used to fetch the image selector that should be used
    /// to choose an image for this column.
    /// </summary>
    public delegate Object ImageGetterDelegate(Object rowObject);

    /// <summary>
    /// These delegates are used to draw a cell
    /// </summary>
    public delegate void RenderDelegate(DrawListViewSubItemEventArgs e, Graphics g, Rectangle r, Object rowObject);

    /// <summary>
    /// These delegates are used to fetch a row object for virtual lists
    /// </summary>
    public delegate Object RowGetterDelegate(int rowIndex);

    /// <summary>
    /// These delegates are used to format a listviewitem before it is added to the control.
    /// </summary>
    public delegate void RowFormatterDelegate(OLVListItem olvItem);

    /// <summary>
    /// These delegates are used to sort the listview in some custom fashion
    /// </summary>
    public delegate void SortDelegate(OLVColumn column, SortOrder sortOrder);

    #endregion

    #region Column

    /// <summary>
    /// An OLVColumn knows which aspect of an object it should present.
    /// </summary>
    /// <remarks>
    /// The column knows how to:
    /// <list type="bullet">
    ///	<item>extract its aspect from the row object</item>
    ///	<item>convert an aspect to a string</item>
    ///	<item>calculate the image for the row object</item>
    ///	<item>extract a group "key" from the row object</item>
    ///	<item>convert a group "key" into a title for the group</item>
    /// </list>
    /// <para>For sorting to work correctly, aspects from the same column
    /// must be of the same type, that is, the same aspect cannot sometimes
    /// return strings and other times integers.</para>
    /// </remarks>
    [Browsable(false)]
    public partial class OLVColumn : ColumnHeader
    {
        /// <summary>
        /// Create an OLVColumn
        /// </summary>
        public OLVColumn() : base() {}

        /// <summary>
        /// Initialize a column to have the given title, and show the given aspect
        /// </summary>
        /// <param name="title">The title of the column</param>
        /// <param name="aspect">The aspect to be shown in the column</param>
        public OLVColumn(string title, string aspect) : this()
        {
            Text = title;
            AspectName = aspect;
        }

        #region Public Properties

        private bool aspectGetterAutoGenerated;
        private Control cellEditor;
        private string groupWithItemCountFormat;
        private string groupWithItemCountSingularFormat;
        private bool isEditable = true;
        private bool isTileViewColumn = false;
        private bool isVisible = true;

        /// <summary>
        /// Where was this column last positioned within the Detail view columns
        /// </summary>
        /// <remarks>DisplayIndex is volatile. Once a column is removed from the control,
        /// there is no way to discover where it was in the display order. This property
        /// guards that information even when the column is not in the listview's active columns.</remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
 
        public int
            LastDisplayIndex = -1;

        private int maxWidth = -1;
        private int minWidth = -1;
        private BaseRenderer renderer;

        /// <summary>
        /// The name of the property or method that should be called to get the value to display in this column.
        /// This is only used if a ValueGetterDelegate has not been given.
        /// </summary>
        /// <remarks>This name can be dotted to chain references to properties or methods.</remarks>
        /// <example>"DateOfBirth"</example>
        /// <example>"Owner.HomeAddress.Postcode"</example>
        [Category("Behavior"),
         Description(
             "The name of the property or method that should be called to get the aspect to display in this column")]
        public string AspectName
        {
            get { return aspectName; }
            set { aspectName = value; }
        }

        /// <summary>
        /// This format string will be used to convert an aspect to its string representation.
        /// </summary>
        /// <remarks>
        /// This string is passed as the first parameter to the String.Format() method.
        /// This is only used if ToStringDelegate has not been set.</remarks>
        /// <example>"{0:C}" to convert a number to currency</example>
        [Category("Behavior"),
         Description("The format string that will be used to convert an aspect to its string representation"),
         DefaultValue(null)]
        public string AspectToStringFormat
        {
            get { return aspectToStringFormat; }
            set { aspectToStringFormat = value; }
        }

        /// <summary>
        /// Group objects by the initial letter of the aspect of the column
        /// </summary>
        /// <remarks>
        /// One common pattern is to group column by the initial letter of the value for that group.
        /// The aspect must be a string (obviously).
        /// </remarks>
        [Category("Behavior"),
         Description(
             "The name of the property or method that should be called to get the aspect to display in this column"),
         DefaultValue(false)]
        public bool UseInitialLetterForGroup
        {
            get { return useInitialLetterForGroup; }
            set { useInitialLetterForGroup = value; }
        }

        /// <summary>
        /// Get/set whether this column should be used when the view is switched to tile view.
        /// </summary>
        /// <remarks>Column 0 is always included in tileview regardless of this setting.
        /// Tile views do not work well with many "columns" of information, 2 or 3 works best.</remarks>
        [Category("Behavior"), Description("Will this column be used when the view is switched to tile view"),
         DefaultValue(false)]
        public bool IsTileViewColumn
        {
            get { return isTileViewColumn; }
            set { isTileViewColumn = value; }
        }

        /// <summary>
        /// This delegate will be used to extract a value to be displayed in this column.
        /// </summary>
        /// <remarks>
        /// If this is set, AspectName is ignored.
        /// </remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public AspectGetterDelegate AspectGetter
        {
            get { return aspectGetter; }
            set
            {
                aspectGetter = value;
                aspectGetterAutoGenerated = false;
            }
        }

        /// <summary>
        /// The delegate that will be used to translate the aspect to display in this column into a string.
        /// </summary>
        /// <remarks>If this value is set, ValueToStringFormat will be ignored.</remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public AspectToStringConverterDelegate AspectToStringConverter
        {
            get { return aspectToStringConverter; }
            set { aspectToStringConverter = value; }
        }

        /// <summary>
        /// This delegate is called to get the image selector of the image that should be shown in this column.
        /// It can return an int, string, Image or null.
        /// </summary>
        /// <remarks><para>This delegate can use these return value to identify the image:</para>
        /// <list>
        /// <item>null or -1 -- indicates no image</item>
        /// <item>an int -- the int value will be used as an index into the image list</item>
        /// <item>a String -- the string value will be used as a key into the image list</item>
        /// <item>an Image -- the Image will be drawn directly (only in OwnerDrawn mode)</item>
        /// </list>
        /// </remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public ImageGetterDelegate ImageGetter
        {
            get { return imageGetter; }
            set { imageGetter = value; }
        }

        /// <summary>
        /// This delegate is called to get the object that is the key for the group
        /// to which the given row belongs.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public GroupKeyGetterDelegate GroupKeyGetter
        {
            get { return groupKeyGetter; }
            set { groupKeyGetter = value; }
        }

        /// <summary>
        /// This delegate is called to convert a group key into a title for that group.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public GroupKeyToTitleConverterDelegate GroupKeyToTitleConverter
        {
            get { return groupKeyToTitleConverter; }
            set { groupKeyToTitleConverter = value; }
        }

        /// <summary>
        /// This delegate is called when a cell needs to be drawn in OwnerDrawn mode.
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public RenderDelegate RendererDelegate
        {
            get { return rendererDelegate; }
            set { rendererDelegate = value; }
        }

        /// <summary>
        /// Get/set the renderer that will be invoked when a cell needs to be redrawn
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public BaseRenderer Renderer
        {
            get { return renderer; }
            set
            {
                renderer = value;
                if (renderer == null)
                    RendererDelegate = null;
                else
                {
                    renderer.Column = this;
                    RendererDelegate = new RenderDelegate(renderer.HandleRendering);
                }
            }
        }

        /// <summary>
        /// Remember if this aspect getter for this column was generated internally, and can therefore
        /// be regenerated at will
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool AspectGetterAutoGenerated
        {
            get { return aspectGetterAutoGenerated; }
            set { aspectGetterAutoGenerated = value; }
        }

        /// <summary>
        /// When the listview is grouped by this column and group title has an item count, 
        /// how should the lable be formatted?
        /// </summary>
        /// <remarks>
        /// The given format string can/should have two placeholders:
        /// <list type="bullet">
        /// <item>{0} - the original group title</item>
        /// <item>{1} - the number of items in the group</item>
        /// </list>
        /// <para>If this value is not set, the values from the list view will be used</para>
        /// </remarks>
        /// <example>"{0} [{1} items]"</example>
        [Category("Behavior"), Description("The format to use when suffixing item counts to group titles"),
         DefaultValue(null)
        ]
        public string GroupWithItemCountFormat
        {
            get { return groupWithItemCountFormat; }
            set { groupWithItemCountFormat = value; }
        }

        /// <summary>
        /// Return this.GroupWithItemCountFormat or a reasonable default
        /// </summary>
        [Browsable(false)]
        public string GroupWithItemCountFormatOrDefault
        {
            get
            {
                if (String.IsNullOrEmpty(GroupWithItemCountFormat))
                {
                    // There is one rare but pathelogically possible case where the ListView can
                    // be null, so we have to provide a workable default for that rare case.
                    if (ListView == null)
                        return "{0} [{1} items]";
                    else
                        return ((ObjectListView) ListView).GroupWithItemCountFormatOrDefault;
                }
                else
                    return GroupWithItemCountFormat;
            }
        }

        /// <summary>
        /// When the listview is grouped by this column and a group title has an item count, 
        /// how should the lable be formatted if there is only one item in the group?
        /// </summary>
        /// <remarks>
        /// The given format string can/should have two placeholders:
        /// <list type="bullet">
        /// <item>{0} - the original group title</item>
        /// <item>{1} - the number of items in the group (always 1)</item>
        /// </list>
        /// <para>If this value is not set, the values from the list view will be used</para>
        /// </remarks>
        /// <example>"{0} [{1} item]"</example>
        [Category("Behavior"), Description("The format to use when suffixing item counts to group titles"),
         DefaultValue(null)
        ]
        public string GroupWithItemCountSingularFormat
        {
            get { return groupWithItemCountSingularFormat; }
            set { groupWithItemCountSingularFormat = value; }
        }

        /// <summary>
        /// Return this.GroupWithItemCountSingularFormat or a reasonable default
        /// </summary>
        [Browsable(false)]
        public string GroupWithItemCountSingularFormatOrDefault
        {
            get
            {
                if (String.IsNullOrEmpty(GroupWithItemCountSingularFormat))
                {
                    // There is one pathelogically rare but still possible case where the ListView can
                    // be null, so we have to provide a workable default for that rare case.
                    if (ListView == null)
                        return "{0} [{1} item]";
                    else
                        return ((ObjectListView) ListView).GroupWithItemCountSingularFormatOrDefault;
                }
                else
                    return GroupWithItemCountSingularFormat;
            }
        }

        /// <summary>
        /// What is the minimum width that the user can give to this column?
        /// </summary>
        /// <remarks>-1 means there is no minimum width. Give this the same value as MaximumWidth to make a fixed width column.</remarks>
        [Category("Misc"), Description("What is the minimum width to which the user can resize this column?"),
         DefaultValue(-1)]
        public int MinimumWidth
        {
            get { return minWidth; }
            set
            {
                minWidth = value;
                if (Width < minWidth)
                    Width = minWidth;
            }
        }

        /// <summary>
        /// What is the maximum width that the user can give to this column?
        /// </summary>
        /// <remarks>-1 means there is no maximum width. Give this the same value as MinimumWidth to make a fixed width column.</remarks>
        [Category("Misc"), Description("What is the maximum width to which the user can resize this column?"),
         DefaultValue(-1)]
        public int MaximumWidth
        {
            get { return maxWidth; }
            set
            {
                maxWidth = value;
                if (maxWidth != -1 && Width > maxWidth)
                    Width = maxWidth;
            }
        }

        /// <summary>
        /// Is this column a fixed width column?
        /// </summary>
        [Browsable(false)]
        public bool IsFixedWidth
        {
            get { return (MinimumWidth != -1 && MaximumWidth != -1 && MinimumWidth >= MaximumWidth); }
        }

        /// <summary>
        /// This delegate will be used to put an edited value back into the model object.
        /// </summary>
        /// <remarks>
        /// This does nothing if IsEditable == false.
        /// </remarks>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public AspectPutterDelegate AspectPutter
        {
            get { return aspectPutter; }
            set { aspectPutter = value; }
        }

        /// <summary>
        /// Can the values shown in this column be edited?
        /// </summary>
        /// <remarks>This defaults to true, since the primary means to control the editability of a listview
        /// is on the listview itself. Once a listview is editable, all the columns are too, unless the 
        /// programmer explicitly marks them as not editable</remarks>
        [Category("Misc"), Description("Can the value in this column be edited?"), DefaultValue(true)]
        public bool IsEditable
        {
            get { return isEditable; }
            set { isEditable = value; }
        }

        /// <summary>
        /// Return the control that should be used to edit cells in this column
        /// </summary>
        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        
        public Control CellEditor
        {
            get { return cellEditor; }
            set { cellEditor = value; }
        }

        /// <summary>
        /// Can this column be seen by the user?
        /// </summary>
        /// <remarks>After changing this value, you must call RebuildColumns() before the changes will be effected.</remarks>
        [Category("Misc"), Description("Can this column be seen by the user?"), DefaultValue(true)]
        public bool IsVisible
        {
            get { return isVisible; }
            set { isVisible = value; }
        }

        #endregion

        /// <summary>
        /// For a given row object, return the object that is to be displayed in this column.
        /// </summary>
        /// <param name="rowObject">The row object that is being displayed</param>
        /// <returns>An object, which is the aspect to be displayed</returns>
        public object GetValue(object rowObject)
        {
            if (aspectGetter == null)
                return GetAspectByName(rowObject);
            else
                return aspectGetter(rowObject);
        }

        /// <summary>
        /// For a given row object, extract the value indicated by the AspectName property of this column.
        /// </summary>
        /// <param name="rowObject">The row object that is being displayed</param>
        /// <returns>An object, which is the aspect named by AspectName</returns>
        public object GetAspectByName(object rowObject)
        {
            if (string.IsNullOrEmpty(aspectName))
                return null;

            //CONSIDER: Should we include NonPublic in this list?
            BindingFlags flags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance |
                                 BindingFlags.InvokeMethod |
                                 BindingFlags.GetProperty | BindingFlags.GetField;
            object source = rowObject;
            foreach (string property in aspectName.Split('.'))
            {
                try
                {
                    source = source.GetType().InvokeMember(property, flags, null, source, null);
                }
                catch (MissingMethodException)
                {
                    return String.Format("Cannot invoke '{0}' on a {1}", property, source.GetType());
                }
            }
            return source;
        }

        public void PutValue(Object rowObject, Object newValue)
        {
            if (aspectPutter == null)
                PutAspectByName(rowObject, newValue);
            else
                aspectPutter(rowObject, newValue);
        }

        public void PutAspectByName(Object rowObject, Object newValue)
        {
            if (string.IsNullOrEmpty(aspectName))
                return;

            // Navigated through the dotted path until we reach the target object.
            // We then try to set the last property on the dotted path on that target.
            // So, if rowObject is a Person, then an aspect named "HomeAddress.Postcode"
            // will first fetch the "HomeAddress" property, and then try to set the
            // "Postcode" property on the home address object.

            //CONSIDER: Should we include NonPublic in this list?
            BindingFlags flags = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance |
                                 BindingFlags.InvokeMethod |
                                 BindingFlags.GetProperty | BindingFlags.GetField;
            Object target = rowObject;
            List<String> parentProperties = new List<string>(aspectName.Split('.'));
            String lastProperty = parentProperties[parentProperties.Count - 1];
            parentProperties.RemoveAt(parentProperties.Count - 1);
            foreach (string property in parentProperties)
            {
                try
                {
                    target = target.GetType().InvokeMember(property, flags, null, target, null);
                }
                catch (MissingMethodException)
                {
                    Debug.WriteLine(String.Format("Cannot invoke '{0}' on a {1}", property, target.GetType()));
                    return;
                }
            }

            // Now try to set the value
            try
            {
                BindingFlags flags2 = BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance |
                                      BindingFlags.SetProperty | BindingFlags.SetField;
                target.GetType().InvokeMember(lastProperty, flags2, null, target, new Object[] {newValue});
            }
            catch (MissingMethodException ex)
            {
                Debug.WriteLine("Invoke PutAspectByName failed:");
                Debug.WriteLine(ex);
            }
        }

        /// <summary>
        /// For a given row object, return the string representation of the value shown in this column.
        /// </summary>
        /// <remarks>
        /// For aspects that are string (e.g. aPerson.Name), the aspect and its string representation are the same.
        /// For non-strings (e.g. aPerson.DateOfBirth), the string representation is very different.
        /// </remarks>
        /// <param name="rowObject"></param>
        /// <returns></returns>
        public string GetStringValue(object rowObject)
        {
            return ValueToString(GetValue(rowObject));
        }

        /// <summary>
        /// Convert the aspect object to its string representation.
        /// </summary>
        /// <remarks>
        /// If the column has been given a ToStringDelegate, that will be used to do
        /// the conversion, otherwise just use ToString(). Nulls are always converted
        /// to empty strings.
        /// </remarks>
        /// <param name="value">The value of the aspect that should be displayed</param>
        /// <returns>A string representation of the aspect</returns>
        public string ValueToString(object value)
        {
            // CONSIDER: Should we give aspect-to-string converters a chance to work on a null value?
            if (value == null)
                return "";

            if (aspectToStringConverter != null)
                return aspectToStringConverter(value);

            string fmt = AspectToStringFormat;
            if (String.IsNullOrEmpty(fmt))
                return value.ToString();
            else
                return String.Format(fmt, value);
        }

        /// <summary>
        /// For a given row object, return the image selector of the image that should displayed in this column.
        /// </summary>
        /// <param name="rowObject">The row object that is being displayed</param>
        /// <returns>int or string or Image. int or string will be used as index into image list. null or -1 means no image</returns>
        public Object GetImage(object rowObject)
        {
            if (imageGetter != null)
                return imageGetter(rowObject);

            if (!String.IsNullOrEmpty(ImageKey))
                return ImageKey;

            return ImageIndex;
        }

        /// <summary>
        /// For a given row object, return the object that is the key of the group that this row belongs to.
        /// </summary>
        /// <param name="rowObject">The row object that is being displayed</param>
        /// <returns>Group key object</returns>
        public object GetGroupKey(object rowObject)
        {
            if (groupKeyGetter == null)
            {
                object key = GetValue(rowObject);
                if (key is string && UseInitialLetterForGroup)
                {
                    String keyAsString = (String) key;
                    if (keyAsString.Length > 0)
                        key = keyAsString.Substring(0, 1).ToUpper();
                }
                return key;
            }
            else
                return groupKeyGetter(rowObject);
        }

        /// <summary>
        /// For a given group value, return the string that should be used as the groups title.
        /// </summary>
        /// <param name="value">The group key that is being converted to a title</param>
        /// <returns>string</returns>
        public string ConvertGroupKeyToTitle(object value)
        {
            if (groupKeyToTitleConverter == null)
                return ValueToString(value);
            else
                return groupKeyToTitleConverter(value);
        }

        #region Utilities

        /// <summary>
        /// Install delegates that will group the columns aspects into progressive partitions.
        /// If an aspect is less than value[n], it will be grouped with description[n]. 
        /// If an aspect has a value greater than the last element in "values", it will be grouped
        /// with the last element in "descriptions".
        /// </summary>
        /// <param name="values">Array of values. Values must be able to be
        /// compared to the aspect (using IComparable)</param>
        /// <param name="descriptions">The description for the matching value. The last element is the default description.
        /// If there are n values, there must be n+1 descriptions.</param>
        /// <example>
        /// this.salaryColumn.MakeGroupies(
        ///     new UInt32[] { 20000, 100000 },
        ///     new string[] { "Lowly worker",  "Middle management", "Rarified elevation"});
        /// </example>
        public void MakeGroupies<T>(T[] values, string[] descriptions)
        {
            if (values.Length + 1 != descriptions.Length)
                throw new ArgumentException("descriptions must have one more element than values.");

            // Install a delegate that returns the index of the description to be shown
            GroupKeyGetter = delegate(object row)
                             {
                                 Object aspect = GetValue(row);
                                 if (aspect == null || aspect == DBNull.Value)
                                     return -1;
                                 IComparable comparable = (IComparable) aspect;
                                 for (int i = 0; i < values.Length; i++)
                                 {
                                     if (comparable.CompareTo(values[i]) < 0)
                                         return i;
                                 }

                                 // Display the last element in the array
                                 return descriptions.Length - 1;
                             };

            // Install a delegate that simply looks up the given index in the descriptions. 
            GroupKeyToTitleConverter = delegate(object key)
                                       {
                                           if ((int) key < 0)
                                               return "";

                                           return descriptions[(int) key];
                                       };
        }

        #endregion

        #region Private Variables

        private AspectGetterDelegate aspectGetter;
        private string aspectName;
        private AspectPutterDelegate aspectPutter;
        private AspectToStringConverterDelegate aspectToStringConverter;
        private string aspectToStringFormat;
        private GroupKeyGetterDelegate groupKeyGetter;
        private GroupKeyToTitleConverterDelegate groupKeyToTitleConverter;
        private ImageGetterDelegate imageGetter;
        private RenderDelegate rendererDelegate;
        private bool useInitialLetterForGroup;

        #endregion
    }

    #endregion

    #region OLVListItem and OLVListSubItem

    /// <summary>
    /// OLVListItems are specialized ListViewItems that know which row object they came from,
    /// and the row index at which they are displayed, even when in group view mode. They
    /// also know the image they should draw against themselves
    /// </summary>
    public class OLVListItem : ListViewItem
    {
        private int displayIndex;
        private Object imageSelector;
        private object rowObject;

        /// <summary>
        /// Create a OLVListItem for the given row object
        /// </summary>
        public OLVListItem(object rowObject) : base()
        {
            this.rowObject = rowObject;
        }

        /// <summary>
        /// Create a OLVListItem for the given row object, represented by the given string and image
        /// </summary>
        public OLVListItem(object rowObject, string text, Object image) : base(text, -1)
        {
            this.rowObject = rowObject;
            imageSelector = image;
        }

        /// <summary>
        /// RowObject is the model object that is source of the data for this list item.
        /// </summary>
        public object RowObject
        {
            get { return rowObject; }
            set { rowObject = value; }
        }

        /// <summary>
        /// DisplayIndex is the index of the row where this item is displayed. For flat lists,
        /// this is the same as ListViewItem.Index, but for grouped views, it is different.
        /// </summary>
        public int DisplayIndex
        {
            get { return displayIndex; }
            set { displayIndex = value; }
        }

        /// <summary>
        /// Get or set the image that should be shown against this item
        /// </summary>
        /// <remarks><para>This can be an Image, a string or an int. A string or an int will
        /// be used as an index into the small image list.</para></remarks>
        public Object ImageSelector
        {
            get { return imageSelector; }
            set
            {
                imageSelector = value;
                if (value is Int32)
                    ImageIndex = (Int32) value;
                else if (value is String)
                    ImageKey = (String) value;
                else
                    ImageIndex = -1;
            }
        }
    }

    /// <summary>
    /// A ListViewSubItem that knows which image should be drawn against it.
    /// </summary>
    [Browsable(false)]
    public class OLVListSubItem : ListViewItem.ListViewSubItem
    {
        private ImageRenderer.AnimationState animationState;
        private Object imageSelector;

        /// <summary>
        /// Create a OLVListSubItem
        /// </summary>
        public OLVListSubItem() : base() {}

        /// <summary>
        /// Create a OLVListSubItem that shows the given string and image
        /// </summary>
        public OLVListSubItem(string text, Object image) : base()
        {
            Text = text;
            ImageSelector = image;
        }

        /// <summary>
        /// Get or set the image that should be shown against this item
        /// </summary>
        /// <remarks><para>This can be an Image, a string or an int. A string or an int will
        /// be used as an index into the small image list.</para></remarks>
        public Object ImageSelector
        {
            get { return imageSelector; }
            set { imageSelector = value; }
        }

        /// <summary>
        /// Return the state of the animatation of the image on this subitem. 
        /// Null means there is either no image, or it is not an animation
        /// </summary>
        public ImageRenderer.AnimationState AnimationState
        {
            get { return animationState; }
            set { animationState = value; }
        }
    }

    #endregion

    #region Comparers

    /// <summary>
    /// This comparer sort list view groups.
    /// It does this on the basis of the values in the Tags, if we can figure out how to compare
    /// objects of that type. Failing that, it uses a case insensitive compare on the group header.
    /// </summary>
    internal class ListViewGroupComparer : IComparer<ListViewGroup>
    {
        private SortOrder sortOrder;

        public ListViewGroupComparer(SortOrder order)
        {
            sortOrder = order;
        }

        #region IComparer<ListViewGroup> Members

        public int Compare(ListViewGroup x, ListViewGroup y)
        {
            // If we know how to compare the tags, do that.
            // Otherwise do a case insensitive compare on the group header.
            // We have explicitly catch the "almost-null" value of DBNull.Value,
            // since comparing to that value always produces a type exception.
            int result;
            IComparable comparable = x.Tag as IComparable;
            if (comparable != null && y.Tag != DBNull.Value)
                result = comparable.CompareTo(y.Tag);
            else
                result = String.Compare(x.Header, y.Header, true);

            if (sortOrder == SortOrder.Descending)
                result = 0 - result;

            return result;
        }

        #endregion
    }

    /// <summary>
    /// ColumnComparer is the workhorse for all comparison between two values of a particular column.
    /// If the column has a specific comparer, use that to compare the values. Otherwise, do
    /// a case insensitive string compare of the string representations of the values.
    /// </summary>
    /// <remarks><para>This class inherits from both IComparer and its generic counterpart
    /// so that it can be used on untyped and typed collections.</para></remarks>
    internal class ColumnComparer : IComparer, IComparer<OLVListItem>
    {
        private OLVColumn column;
        private ColumnComparer secondComparer;
        private SortOrder sortOrder;

        public ColumnComparer(OLVColumn col, SortOrder order)
        {
            column = col;
            sortOrder = order;
            secondComparer = null;
        }

        public ColumnComparer(OLVColumn col, SortOrder order, OLVColumn col2, SortOrder order2) : this(col, order)
        {
            // There is no point in secondary sorting on the same column
            if (col != col2)
                secondComparer = new ColumnComparer(col2, order2);
        }

        #region IComparer Members

        public int Compare(object x, object y)
        {
            return Compare((OLVListItem) x, (OLVListItem) y);
        }

        #endregion

        #region IComparer<OLVListItem> Members

        public int Compare(OLVListItem x, OLVListItem y)
        {
            int result = 0;
            object x1 = column.GetValue(x.RowObject);
            object y1 = column.GetValue(y.RowObject);

            if (sortOrder == SortOrder.None)
                return 0;

            // Handle nulls. Null values come last
            bool xIsNull = (x1 == null || x1 == DBNull.Value);
            bool yIsNull = (y1 == null || y1 == DBNull.Value);
            if (xIsNull || yIsNull)
            {
                if (xIsNull && yIsNull)
                    result = 0;
                else
                    result = (xIsNull ? -1 : 1);
            }
            else result = CompareValues(x1, y1);

            if (sortOrder == SortOrder.Descending)
                result = 0 - result;

            // If the result was equality, use the secondary comparer to resolve it
            if (result == 0 && secondComparer != null)
                result = secondComparer.Compare(x, y);

            return result;
        }

        #endregion

        public int CompareValues(object x, object y)
        {
            // Force case insensitive compares on strings
            if (x is String)
                return String.Compare((String) x, (String) y, true);
            else
            {
                IComparable comparable = x as IComparable;
                if (comparable != null)
                    return comparable.CompareTo(y);
                else
                    return 0;
            }
        }
    }

    #endregion

    #region Renderers

    /// <summary>
    /// Renderers are responsible for drawing a single cell within an owner drawn ObjectListView. 
    /// </summary>
    /// <remarks>
    /// <para>Methods on this class are called during the DrawSubItem event.</para>
    /// <para>Subclasses will normally override the Render method, and use the other
    /// methods as helper functions.</para>
    /// </remarks>
    [Browsable(false)]
    public class BaseRenderer
    {
        /// <summary>
        /// Make a simple renderer
        /// </summary>
        public BaseRenderer() {}

        #region Properties 

        private Object aspect;
        private bool canWrap = false;
        private OLVColumn column;
        private DrawListViewSubItemEventArgs eventArgs;
        private Font font;
        private bool isDrawBackground = true;
        private bool isItemSelected;
        private OLVListItem listItem;
        private ListViewItem.ListViewSubItem listSubItem;

        private ObjectListView objectListView;
        private Object rowObject;
        private Brush textBrush;

        /// <summary>
        /// Get/set the event that caused this renderer to be called
        /// </summary>
        public DrawListViewSubItemEventArgs Event
        {
            get { return eventArgs; }
            set { eventArgs = value; }
        }

        /// <summary>
        /// Get/set the listview for which the drawing is to be done
        /// </summary>
        public ObjectListView ListView
        {
            get { return objectListView; }
            set { objectListView = value; }
        }

        /// <summary>
        /// Get or set the OLVColumn that this renderer will draw
        /// </summary>
        public OLVColumn Column
        {
            get { return column; }
            set { column = value; }
        }

        /// <summary>
        /// Get or set the model object that this renderer should draw
        /// </summary>
        public Object RowObject
        {
            get { return rowObject; }
            set { rowObject = value; }
        }

        /// <summary>
        /// Get or set the aspect of the model object that this renderer should draw
        /// </summary>
        public Object Aspect
        {
            get
            {
                if (aspect == null)
                    aspect = column.GetValue(rowObject);
                return aspect;
            }
            set { aspect = value; }
        }

        /// <summary>
        /// Get or set the listitem that this renderer will be drawing
        /// </summary>
        public OLVListItem ListItem
        {
            get { return listItem; }
            set { listItem = value; }
        }

        /// <summary>
        /// Get or set the list subitem that this renderer will be drawing
        /// </summary>
        public ListViewItem.ListViewSubItem SubItem
        {
            get { return listSubItem; }
            set { listSubItem = value; }
        }

        /// <summary>
        /// Get the specialized OLVSubItem that this renderer is drawing
        /// </summary>
        /// <remarks>This returns null for column 0.</remarks>
        public OLVListSubItem OLVSubItem
        {
            get { return listSubItem as OLVListSubItem; }
        }

        /// <summary>
        /// Cache whether or not our item is selected
        /// </summary>
        public bool IsItemSelected
        {
            get { return isItemSelected; }
            set { isItemSelected = value; }
        }

        /// <summary>
        /// Return the font to be used for text in this cell
        /// </summary>
        /// <returns>The font of the subitem</returns>
        public Font Font
        {
            get
            {
                if (font == null)
                {
                    if (ListItem.UseItemStyleForSubItems)
                        return ListItem.Font;
                    else
                        return SubItem.Font;
                }
                else
                    return font;
            }
            set { font = value; }
        }

        /// <summary>
        /// The brush that will be used to paint the text
        /// </summary>
        public Brush TextBrush
        {
            get
            {
                if (textBrush == null)
                    return new SolidBrush(GetForegroundColor());
                else
                    return textBrush;
            }
            set { textBrush = value; }
        }

        /// <summary>
        /// Should this renderer fill in the background before drawing?
        /// </summary>
        public bool IsDrawBackground
        {
            get { return isDrawBackground; }
            set { isDrawBackground = value; }
        }

        /// <summary>
        /// Can the renderer wrap lines that do not fit completely within the cell?
        /// </summary>
        /// <remarks>This value is currently only used when printing a list view using ListViewPrinter.</remarks>
        public bool CanWrap
        {
            get { return canWrap; }
            set { canWrap = value; }
        }

        #endregion

        #region Utilities

        /// <summary>
        /// Return the image that should be drawn against this subitem
        /// </summary>
        /// <returns>An Image or null if no image should be drawn.</returns>
        public Image GetImage()
        {
            if (Column.Index == 0)
                return GetImage(ListItem.ImageSelector);
            else
                return GetImage(OLVSubItem.ImageSelector);
        }

        /// <summary>
        /// Return the actual image that should be drawn when keyed by the given image selector.
        /// An image selector can be: <list>
        /// <item>an int, giving the index into the image list</item>
        /// <item>a string, giving the image key into the image list</item>
        /// <item>an Image, being the image itself</item>
        /// </list>
        /// </summary>
        /// <param name="imageSelector">The value that indicates the image to be used</param>
        /// <returns>An Image or null</returns>
        public Image GetImage(Object imageSelector)
        {
            if (imageSelector == null)
                return null;

            ImageList il = ListView.BaseSmallImageList;
            if (il != null)
            {
                if (imageSelector is Int32)
                {
                    Int32 index = (Int32) imageSelector;
                    if (index < 0 || index >= il.Images.Count)
                        return null;
                    else
                        return il.Images[index];
                }

                if (imageSelector is String)
                {
                    if (il.Images.ContainsKey((String) imageSelector))
                        return il.Images[(String) imageSelector];
                    else
                        return null;
                }
            }

            return imageSelector as Image;
        }

        /// <summary>
        /// Return the Color that is the background color for this item's cell
        /// </summary>
        /// <returns>The background color of the subitem</returns>
        public Color GetBackgroundColor()
        {
            if (IsItemSelected && ListView.FullRowSelect)
            {
                if (ListView.Focused)
                    return SystemColors.Highlight;
                else if (!ListView.HideSelection)
                    return SystemColors.Control; //TODO: What color should this be?
            }
            if (ListItem.UseItemStyleForSubItems)
                return ListItem.BackColor;
            else
                return SubItem.BackColor;
        }

        /// <summary>
        /// Return the Color that is the background color for this item's text
        /// </summary>
        /// <returns>The background color of the subitem's text</returns>
        protected Color GetTextBackgroundColor()
        {
            if (IsItemSelected && (Column.Index == 0 || ListView.FullRowSelect))
                return SystemColors.Highlight;
            else if (ListItem.UseItemStyleForSubItems)
                return ListItem.BackColor;
            else
                return SubItem.BackColor;
        }

        /// <summary>
        /// Return the color to be used for text in this cell
        /// </summary>
        /// <returns>The text color of the subitem</returns>
        protected Color GetForegroundColor()
        {
            if (IsItemSelected && (Column.Index == 0 || ListView.FullRowSelect))
                return SystemColors.HighlightText;
            else if (ListItem.UseItemStyleForSubItems)
                return ListItem.ForeColor;
            else
                return SubItem.ForeColor;
        }

        /// <summary>
        /// Align the second rectangle with the first rectangle, 
        /// according to the alignment of the column
        /// </summary>
        /// <param name="outer">The cell's bounds</param>
        /// <param name="inner">The rectangle to be aligned within the bounds</param>
        /// <returns>An aligned rectangle</returns>
        protected Rectangle AlignRectangle(Rectangle outer, Rectangle inner)
        {
            Rectangle r = new Rectangle(outer.Location, inner.Size);

            // Centre horizontally depending on the column alignment
            if (inner.Width < outer.Width)
            {
                switch (Column.TextAlign)
                {
                    case HorizontalAlignment.Left:
                        r.X = outer.Left;
                        break;
                    case HorizontalAlignment.Center:
                        r.X = outer.Left + ((outer.Width - inner.Width)/2);
                        break;
                    case HorizontalAlignment.Right:
                        r.X = outer.Right - inner.Width - 1;
                        break;
                }
            }
            // Centre vertically too
            if (inner.Height < outer.Height)
                r.Y = outer.Top + ((outer.Height - inner.Height)/2);

            return r;
        }

        /// <summary>
        /// Draw the given image aligned horizontally within the column.
        /// </summary>
        /// <remarks>
        /// Over tall images are scaled to fit. Over-wide images are 
        /// truncated. This is by design!
        /// </remarks>
        /// <param name="g">Graphics context to use for drawing</param>
        /// <param name="r">Bounds of the cell</param>
        /// <param name="image">The image to be drawn</param>
        protected void DrawAlignedImage(Graphics g, Rectangle r, Image image)
        {
            if (image == null)
                return;

            // By default, the image goes in the top left of the rectangle
            Rectangle imageBounds = new Rectangle(r.Location, image.Size);

            // If the image is too tall to be drawn in the space provided, proportionally scale it down.
            // Too wide images are not scaled.
            if (image.Height > r.Height)
            {
                float scaleRatio = (float) r.Height/(float) image.Height;
                imageBounds.Width = (int) ((float) image.Width*scaleRatio);
                imageBounds.Height = r.Height - 1;
            }

            // Align and draw our (possibly scaled) image
            g.DrawImage(image, AlignRectangle(r, imageBounds));
        }

        /// <summary>
        /// Fill in the background of this cell
        /// </summary>
        /// <param name="g">Graphics context to use for drawing</param>
        /// <param name="r">Bounds of the cell</param>
        protected void DrawBackground(Graphics g, Rectangle r)
        {
            if (IsDrawBackground)
                g.FillRectangle(new SolidBrush(GetBackgroundColor()), r);
        }

        #endregion

        /// <summary>
        /// The delegate that is called from the list view. This is the main entry point, but
        /// subclasses should override Render instead of this method.
        /// </summary>
        /// <param name="e">The event that caused this redraw</param>
        /// <param name="g">The context that our drawing should be done using</param>
        /// <param name="r">The bounds of the cell within which the renderer can draw</param>
        /// <param name="rowObject">The model object for this row</param>
        public void HandleRendering(DrawListViewSubItemEventArgs e, Graphics g, Rectangle r, Object rowObject)
        {
            Event = e;
            ListView = (ObjectListView) Column.ListView;
            RowObject = rowObject;
            ListItem = e.Item as OLVListItem;
            SubItem = e.SubItem;
            Column = ListView.GetColumn(e.ColumnIndex);
            Aspect = null; // uncache previous result
            IsItemSelected = ListItem.Selected;
            // ((e.ItemState & ListViewItemStates.Selected) == ListViewItemStates.Selected);
            IsDrawBackground = true;
            Font = null;
            TextBrush = null;
            Render(g, r);
        }

        /// <summary>
        /// Draw our data into the given rectangle using the given graphics context.
        /// </summary>
        /// <remarks>
        /// <para>Subclasses should override this method.</para></remarks>
        /// <param name="g">The graphics context that should be used for drawing</param>
        /// <param name="r">The bounds of the subitem cell</param>
        public virtual void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            // Adjust the rectangle to match the padding used by the native mode of the ListView
            Rectangle r2 = r;
            r2.X += 4;
            r2.Width -= 4;
            DrawImageAndText(g, r2);
        }

        /// <summary>
        /// Draw our subitems image and text
        /// </summary>
        /// <param name="g">Graphics context to use for drawing</param>
        /// <param name="r">Bounds of the cell</param>
        protected void DrawImageAndText(Graphics g, Rectangle r)
        {
            DrawImageAndText(g, r, SubItem.Text, GetImage());
        }

        /// <summary>
        /// Draw the given text and optional image in the "normal" fashion
        /// </summary>
        /// <param name="g">Graphics context to use for drawing</param>
        /// <param name="r">Bounds of the cell</param>
        /// <param name="txt">The string to be drawn</param>
        /// <param name="image">The optional image to be drawn</param>
        protected void DrawImageAndText(Graphics g, Rectangle r, String txt, Image image)
        {
            // Draw the image
            if (image != null)
            {
                int top = r.Y;
                if (image.Size.Height < r.Height)
                    top += ((r.Height - image.Size.Height)/2);

                g.DrawImageUnscaled(image, r.X, top);
                r.X += image.Width;
                r.Width -= image.Width;
            }

            StringFormat fmt = new StringFormat();
            fmt.LineAlignment = StringAlignment.Center;
            fmt.Trimming = StringTrimming.EllipsisCharacter;
            if (!CanWrap)
                fmt.FormatFlags = StringFormatFlags.NoWrap;
            switch (Column.TextAlign)
            {
                case HorizontalAlignment.Center:
                    fmt.Alignment = StringAlignment.Center;
                    break;
                case HorizontalAlignment.Left:
                    fmt.Alignment = StringAlignment.Near;
                    break;
                case HorizontalAlignment.Right:
                    fmt.Alignment = StringAlignment.Far;
                    break;
            }

            // Draw the background of the text as selected, if it's the primary column
            // and it's selected and it's not in FullRowSelect mode.
            if (IsDrawBackground && IsItemSelected && Column.Index == 0 && !ListView.FullRowSelect)
            {
                SizeF size = g.MeasureString(txt, Font, r.Width, fmt);
                Rectangle r2 = AlignRectangle(r, new Rectangle(0, 0, (int) (size.Width + 1), (int) (size.Height + 1)));
                g.FillRectangle(new SolidBrush(SystemColors.Highlight), r2);
            }

            RectangleF rf = r;
            g.DrawString(txt, Font, TextBrush, rf, fmt);

            // We should put a focus rectange around the column 0 text if it's selected --
            // but we don't because:
            // - I really dislike this UI convention
            // - we are using buffered graphics, so the DrawFocusRecatangle method of the event doesn't work

            //if (this.Column.Index == 0) {
            //    Size size = TextRenderer.MeasureText(this.SubItem.Text, this.ListView.ListFont);
            //    if (r.Width > size.Width)
            //        r.Width = size.Width;
            //    this.Event.DrawFocusRectangle(r);
            //}
        }
    }

    /// <summary>
    /// This class maps a data value to an image that should be drawn for that value.
    /// </summary>
    /// <remarks><para>It is useful for drawing data that is represented as an enum or boolean.</para></remarks>
    public class MappedImageRenderer : BaseRenderer
    {
        /// <summary>
        /// Make a new empty renderer
        /// </summary>
        public MappedImageRenderer() : base()
        {
            map = new Hashtable();
        }

        /// <summary>
        /// Make a new renderer that will show the given image when the given key is the aspect value
        /// </summary>
        /// <param name="key">The data value to be matched</param>
        /// <param name="image">The image to be shown when the key is matched</param>
        public MappedImageRenderer(Object key, Object image) : this()
        {
            Add(key, image);
        }

        public MappedImageRenderer(Object key1, Object image1, Object key2, Object image2) : this()
        {
            Add(key1, image1);
            Add(key2, image2);
        }

        /// <summary>
        /// Build a renderer from the given array of keys and their matching images
        /// </summary>
        /// <param name="keysAndImages">An array of key/image pairs</param>
        public MappedImageRenderer(Object[] keysAndImages) : this()
        {
            if ((keysAndImages.GetLength(0)%2) != 0)
                throw new ArgumentException("Array must have key/image pairs");

            for (int i = 0; i < keysAndImages.GetLength(0); i += 2)
                Add(keysAndImages[i], keysAndImages[i + 1]);
        }

        /// <summary>
        /// Return a renderer that draw boolean values using the given images
        /// </summary>
        /// <param name="trueImage">Draw this when our data value is true</param>
        /// <param name="falseImage">Draw this when our data value is false</param>
        /// <returns>A Renderer</returns>
        public static MappedImageRenderer Boolean(Object trueImage, Object falseImage)
        {
            return new MappedImageRenderer(true, trueImage, false, falseImage);
        }

        /// <summary>
        /// Return a renderer that draw tristate boolean values using the given images
        /// </summary>
        /// <param name="trueImage">Draw this when our data value is true</param>
        /// <param name="falseImage">Draw this when our data value is false</param>
        /// <param name="nullImage">Draw this when our data value is null</param>
        /// <returns>A Renderer</returns>
        public static MappedImageRenderer TriState(Object trueImage, Object falseImage, Object nullImage)
        {
            return new MappedImageRenderer(new Object[] {true, trueImage, false, falseImage, null, nullImage});
        }

        /// <summary>
        /// Register the image that should be drawn when our Aspect has the data value.
        /// </summary>
        /// <param name="value">Value that the Aspect must match</param>
        /// <param name="image">An ImageSelector -- an int, string or image</param>
        public void Add(Object value, Object image)
        {
            if (value == null)
                nullImage = image;
            else
                map[value] = image;
        }

        /// <summary>
        /// Render our value
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            Image image = null;
            if (Aspect == null)
                image = GetImage(nullImage);
            else if (map.ContainsKey(Aspect))
                image = GetImage(map[Aspect]);

            DrawAlignedImage(g, r, image);
        }

        #region Private variables

        private Hashtable map; // Track the association between values and images
        private Object nullImage; // image to be drawn for null values (since null can't be a key)

        #endregion
    }

    /// <summary>
    /// Render an image that comes from our data source.
    /// </summary>
    /// <remarks>The image can be sourced from:
    /// <list>
    /// <item>a byte-array (normally when the image to be shown is
    /// stored as a value in a database)</item>
    /// <item>an int, which is treated as an index into the image list</item>
    /// <item>a string, which is treated first as a file name, and failing that as an index into the image list</item>
    /// </list>
    /// <para>If an image is an animated GIF, it's state is stored in the SubItem object.</para>
    /// <para>By default, the image renderer does not render animations (it begins life with animations paused). 
    /// To enable animations, you must call Unpause().</para>
    /// </remarks>
    public class ImageRenderer : BaseRenderer
    {
        private bool isPaused = true;

        /// <summary>
        /// Make an empty image renderer
        /// </summary>
        public ImageRenderer() : base()
        {
            tickler = new Timer(new TimerCallback(OnTimer), null, Timeout.Infinite, Timeout.Infinite);
            stopwatch = new Stopwatch();
        }

        /// <summary>
        /// Make an empty image renderer that begins life ready for animations
        /// </summary>
        public ImageRenderer(bool startAnimations) : this()
        {
            Paused = !startAnimations;
        }

        /// <summary>
        /// Should the animations in this renderer be paused?
        /// </summary>
        public bool Paused
        {
            get { return isPaused; }
            set
            {
                if (isPaused != value)
                {
                    isPaused = value;
                    if (isPaused)
                    {
                        tickler.Change(Timeout.Infinite, Timeout.Infinite);
                        stopwatch.Stop();
                    }
                    else
                    {
                        tickler.Change(1, Timeout.Infinite);
                        stopwatch.Start();
                    }
                }
            }
        }

        /// <summary>
        /// Draw our image
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);
            DrawAlignedImage(g, r, GetImageFromAspect());
        }

        /// <summary>
        /// Translate our Aspect into an image.
        /// </summary>
        /// <remarks>The strategy is:<list type="unordered">
        /// <item>If its a byte array, we treat it as an in-memory image</item>
        /// <item>If it's an int, we use that as an index into our image list</item>
        /// <item>If it's a string, we try to load a file by that name. If we can't, we use the string as an index into our image list.</item>
        ///</list></remarks>
        /// <returns>An image</returns>
        protected Image GetImageFromAspect()
        {
            if (Aspect == null || Aspect == DBNull.Value)
                return null;

            // If we've already figured out the image, don't do it again
            if (OLVSubItem != null && OLVSubItem.ImageSelector is Image)
            {
                if (OLVSubItem.AnimationState == null)
                    return (Image) OLVSubItem.ImageSelector;
                else
                    return OLVSubItem.AnimationState.image;
            }

            // Try to convert our Aspect into an Image
            // If its a byte array, we treat it as an in-memory image
            // If it's an int, we use that as an index into our image list
            // If it's a string, we try to find a file by that name. 
            //    If we can't, we use the string as an index into our image list.
            Image image = null;
            if (Aspect is Byte[])
            {
                using (MemoryStream stream = new MemoryStream((Byte[]) Aspect))
                {
                    try
                    {
                        image = Image.FromStream(stream);
                    }
                    catch (ArgumentException)
                    {
                        // ignore
                    }
                }
            }
            else if (Aspect is Int32) image = GetImage(Aspect);
            else if (Aspect is String && ((String) Aspect) != "")
            {
                try
                {
                    image = Image.FromFile((String) Aspect);
                }
                catch (FileNotFoundException)
                {
                    image = GetImage(Aspect);
                }
                catch (OutOfMemoryException)
                {
                    image = GetImage(Aspect);
                }
            }

            // If this image is an animation, initialize the animation process
            if (OLVSubItem != null && AnimationState.IsAnimation(image))
                OLVSubItem.AnimationState = new AnimationState(image);

            // Cache the image so we don't repeat this dreary process
            if (OLVSubItem != null)
                OLVSubItem.ImageSelector = image;

            return image;
        }

        /// <summary>
        /// Pause any animations
        /// </summary>
        public void Pause()
        {
            Paused = true;
        }

        /// <summary>
        /// Unpause any animations
        /// </summary>
        public void Unpause()
        {
            Paused = false;
        }

        /// <summary>
        /// This is the method that is invoked by the timer. It basically switches control to the listview thread.
        /// </summary>
        /// <param name="state">not used</param>
        public void OnTimer(Object state)
        {
            if (ListView == null || Paused)
                tickler.Change(1000, Timeout.Infinite);
            else
            {
                if (ListView.InvokeRequired)
                    ListView.Invoke(new OnTimerCallback(OnTimer), new object[] {state});
                else
                    OnTimerInThread();
            }
        }

        /// <summary>
        /// This is the OnTimer callback, but invoked in the same thread as the creator of the ListView.
        /// This method can use all of ListViews methods without creating a CrossThread exception.
        /// </summary>
        protected void OnTimerInThread()
        {
            // MAINTAINER NOTE: This method must renew the tickler. If it doesn't the animations will stop.

            // If this listview has been destroyed, we can't do anything, so we return without 
            // renewing the tickler, effectively killing all animations on this renderer
            if (ListView.IsDisposed)
                return;

            // If we're not in Detail view or our column has been removed from the list, 
            // we can't do anything at the moment, but we still renew the tickler because the view may change later.
            if (ListView.View != View.Details || Column.Index < 0)
            {
                tickler.Change(1000, Timeout.Infinite);
                return;
            }

            long elapsedMilliseconds = stopwatch.ElapsedMilliseconds;
            int subItemIndex = Column.Index;
            long nextCheckAt = elapsedMilliseconds + 1000; // wait at most one second before checking again
            Rectangle updateRect = new Rectangle(); // what part of the view must be updated to draw the changed gifs?

            // Run through all the subitems in the view for our column, and for each one that 
            // has an animation attached to it, see if the frame needs updating.
            foreach (ListViewItem lvi in ListView.Items)
            {
                // Get the gif state from the subitem. If there isn't an animation state, skip this row.
                OLVListSubItem lvsi = (OLVListSubItem) lvi.SubItems[subItemIndex];
                AnimationState state = lvsi.AnimationState;
                if (state == null || !state.IsValid)
                    continue;

                // Has this frame of the animation expired?
                if (elapsedMilliseconds >= state.currentFrameExpiresAt)
                {
                    state.AdvanceFrame(elapsedMilliseconds);

                    // Track the area of the view that needs to be redrawn to show the changed images
                    if (updateRect.IsEmpty)
                        updateRect = lvsi.Bounds;
                    else
                        updateRect = Rectangle.Union(updateRect, lvsi.Bounds);
                }

                // Remember the minimum time at which a frame is next due to change
                nextCheckAt = Math.Min(nextCheckAt, state.currentFrameExpiresAt);
            }

            // Update the part of the listview where frames have changed
            if (!updateRect.IsEmpty)
                ListView.Invalidate(updateRect);

            // Renew the tickler in time for the next frame change
            tickler.Change(nextCheckAt - elapsedMilliseconds, Timeout.Infinite);
        }

        #region Nested type: AnimationState

        /// <summary>
        /// Instances of this class kept track of the animation state of a single image.
        /// </summary>
        public class AnimationState
        {
            private const int PropertyTagFrameDelay = 0x5100;
            private const int PropertyTagLoopCount = 0x5101;
            private const int PropertyTagTypeLong = 4;
            private const int PropertyTagTypeShort = 3;

            public int currentFrame;
            public long currentFrameExpiresAt;
            public int frameCount;
            public Image image;
            public List<int> imageDuration;

            /// <summary>
            /// Create an AnimationState in a quiet state
            /// </summary>
            public AnimationState()
            {
                currentFrame = 0;
                frameCount = 0;
                imageDuration = new List<int>();
                image = null;
            }

            /// <summary>
            /// Create an animation state for the given image, which may or may not
            /// be an animation
            /// </summary>
            /// <param name="image">The image to be rendered</param>
            public AnimationState(Image image) : this()
            {
                if (!IsAnimation(image))
                    return;

                // How many frames in the animation?
                this.image = image;
                frameCount = this.image.GetFrameCount(FrameDimension.Time);

                // Find the delay between each frame. 
                // The delays are stored an array of 4-byte ints. Each int is the 
                // number of 1/100th of a second that should elapsed before the frame expires
                foreach (PropertyItem pi in this.image.PropertyItems)
                {
                    if (pi.Id == PropertyTagFrameDelay)
                    {
                        for (int i = 0; i < pi.Len; i += 4)
                        {
                            //TODO: There must be a better way to convert 4-bytes to an int
                            int delay = (pi.Value[i + 3] << 24) + (pi.Value[i + 2] << 16) + (pi.Value[i + 1] << 8) +
                                        pi.Value[i];
                            imageDuration.Add(delay*10); // store delays as milliseconds
                        }
                        break;
                    }
                }

                // There should be as many frame durations as frames
                Debug.Assert(imageDuration.Count == frameCount,
                             "There should be as many frame durations as there are frames.");
            }

            /// <summary>
            /// Does this state represent a valid animation
            /// </summary>
            public bool IsValid
            {
                get { return (image != null && frameCount > 0); }
            }

            /// <summary>
            /// Is the given image an animation
            /// </summary>
            /// <param name="image">The image to be tested</param>
            /// <returns>Is the image an animation?</returns>
            public static bool IsAnimation(Image image)
            {
                if (image == null)
                    return false;
                else
                    return (new List<Guid>(image.FrameDimensionsList)).Contains(FrameDimension.Time.Guid);
            }

            /// <summary>
            /// Advance our images current frame and calculate when it will expire
            /// </summary>
            public void AdvanceFrame(long millisecondsNow)
            {
                currentFrame = (currentFrame + 1)%frameCount;
                currentFrameExpiresAt = millisecondsNow + imageDuration[currentFrame];
                image.SelectActiveFrame(FrameDimension.Time, currentFrame);
            }
        }

        #endregion

        #region Nested type: OnTimerCallback

        protected delegate void OnTimerCallback(Object state);

        #region Private variables

        private Stopwatch stopwatch; // clock used to time the animation frame changes
        private Timer tickler; // timer used to tickle the animations

        #endregion

        #endregion
    }

    /// <summary>
    /// Render our Aspect as a progress bar
    /// </summary>
    public class BarRenderer : BaseRenderer
    {
        #region Constructors

        /// <summary>
        /// Make a BarRenderer
        /// </summary>
        public BarRenderer() : base()
        {
            Pen = new Pen(Color.Blue, 1);
            Brush = Brushes.Aquamarine;
            BackgroundBrush = Brushes.White;
            StartColor = Color.Empty;
        }

        /// <summary>
        /// Make a BarRenderer for the given range of data values
        /// </summary>
        public BarRenderer(int minimum, int maximum) : this()
        {
            MinimumValue = minimum;
            MaximumValue = maximum;
        }

        /// <summary>
        /// Make a BarRenderer using a custom bar scheme
        /// </summary>
        public BarRenderer(Pen aPen, Brush aBrush) : this()
        {
            Pen = aPen;
            Brush = aBrush;
            UseStandardBar = false;
        }

        /// <summary>
        /// Make a BarRenderer using a custom bar scheme
        /// </summary>
        public BarRenderer(int minimum, int maximum, Pen aPen, Brush aBrush) : this(minimum, maximum)
        {
            Pen = aPen;
            Brush = aBrush;
            UseStandardBar = false;
        }

        /// <summary>
        /// Make a BarRenderer that uses a horizontal gradient
        /// </summary>
        public BarRenderer(Pen aPen, Color start, Color end) : this()
        {
            Pen = aPen;
            SetGradient(start, end);
        }

        /// <summary>
        /// Make a BarRenderer that uses a horizontal gradient
        /// </summary>
        public BarRenderer(int minimum, int maximum, Pen aPen, Color start, Color end) : this(minimum, maximum)
        {
            Pen = aPen;
            SetGradient(start, end);
        }

        #endregion

        #region Public variables

        /// <summary>
        /// The brush that will be used to fill the background of the bar
        /// </summary>
        public Brush BackgroundBrush;

        /// <summary>
        /// The brush that will be used to fill the bar
        /// </summary>
        public Brush Brush;

        /// <summary>
        /// The end color when a gradient is used to fill the bar
        /// </summary>
        public Color EndColor;

        /// <summary>
        /// Regardless of how high the cell is  the progress bar will never be taller than this
        /// </summary>
        public int MaximumHeight = 16;

        /// <summary>
        /// The maximum value for the range. Values greater than this will give a full bar
        /// </summary>
        public int MaximumValue = 100;

        /// <summary>
        /// Regardless of how wide the column become the progress bar will never be wider than this
        /// </summary>
        public int MaximumWidth = 100;

        /// <summary>
        /// The minimum data value expected. Values less than this will given an empty bar
        /// </summary>
        public int MinimumValue = 0;

        /// <summary>
        /// How many pixels in from our cell border will this bar be drawn
        /// </summary>
        public int Padding = 2;

        /// <summary>
        /// The Pen that will draw the frame surrounding this bar
        /// </summary>
        public Pen Pen;

        /// <summary>
        /// The first color when a gradient is used to fill the bar
        /// </summary>
        public Color StartColor;

        /// <summary>
        /// Should this bar be drawn in the system style
        /// </summary>
        public bool UseStandardBar = true;

        #endregion

        /// <summary>
        /// Draw this progress bar using a gradient
        /// </summary>
        /// <param name="start"></param>
        /// <param name="end"></param>
        public void SetGradient(Color start, Color end)
        {
            StartColor = start;
            EndColor = end;
            UseStandardBar = false;
        }

        /// <summary>
        /// Draw our aspect
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            Rectangle frameRect = Rectangle.Inflate(r, 0 - Padding, 0 - Padding);
            frameRect.Width = Math.Min(frameRect.Width, MaximumWidth);
            frameRect.Height = Math.Min(frameRect.Height, MaximumHeight);
            frameRect = AlignRectangle(r, frameRect);

            // Convert our aspect to a numeric value
            // CONSIDER: Is this the best way to do this?
            if (!(Aspect is IConvertible))
                return;
            double aspectValue = ((IConvertible) Aspect).ToDouble(NumberFormatInfo.InvariantInfo);

            Rectangle fillRect = Rectangle.Inflate(frameRect, -1, -1);
            if (aspectValue <= MinimumValue)
                fillRect.Width = 0;
            else if (aspectValue < MaximumValue)
                fillRect.Width = (int) (fillRect.Width*(aspectValue - MinimumValue)/MaximumValue);

            if (UseStandardBar && ProgressBarRenderer.IsSupported)
            {
                ProgressBarRenderer.DrawHorizontalBar(g, frameRect);
                ProgressBarRenderer.DrawHorizontalChunks(g, fillRect);
            }
            else
            {
                g.FillRectangle(BackgroundBrush, frameRect);
                if (fillRect.Width > 0)
                {
                    fillRect.Height++;
                    if (StartColor == Color.Empty)
                        g.FillRectangle(Brush, fillRect);
                    else
                    {
                        using (
                            LinearGradientBrush gradient =
                                new LinearGradientBrush(frameRect, StartColor, EndColor, LinearGradientMode.Horizontal))
                        {
                            g.FillRectangle(gradient, fillRect);
                        }
                    }
                }
                g.DrawRectangle(Pen, frameRect);
            }
        }
    }

    /// <summary>
    /// An ImagesRenderer draws zero or more images depending on the data returned by its Aspect.
    /// </summary>
    /// <remarks><para>This renderer's Aspect must return a ICollection of ints, strings or Images,
    /// each of which will be drawn horizontally one after the other.</para></remarks>
    public class ImagesRenderer : BaseRenderer
    {
        /// <summary>
        /// Get or set the number of pixels between each image
        /// </summary>
        public int Spacing = 1;

        /// <summary>
        /// Draw our data value
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            ICollection imageSelectors = Aspect as ICollection;
            if (imageSelectors == null)
                return;

            Point pt = r.Location;
            foreach (Object selector in imageSelectors)
            {
                Image image = GetImage(selector);
                if (image != null)
                {
                    g.DrawImage(image, pt);
                    pt.X += (image.Width + Spacing);
                }
            }
        }
    }

    /// <summary>
    /// A MultiImageRenderer draws the same image a number of times based on our data value
    /// </summary>
    /// <remarks><para>The stars in the Rating column of iTunes is a good example of this type of renderer.</para></remarks>
    public class MultiImageRenderer : BaseRenderer
    {
        /// <summary>
        /// The image selector that will give the image to be drawn
        /// </summary>
        public Object ImageSelector;

        /// <summary>
        /// Values greater than or equal to this will have MaxNumberImages images drawn
        /// </summary>
        public int MaximumValue = 100;

        /// <summary>
        /// What is the maximum number of images that this renderer should draw?
        /// </summary>
        public int MaxNumberImages = 10;

        /// <summary>
        /// Values less than or equal to this will have 0 images drawn
        /// </summary>
        public int MinimumValue = 0;

        /// <summary>
        /// Get or set the number of pixels between each image
        /// </summary>
        public int Spacing = 1;

        /// <summary>
        /// Make a quiet rendererer
        /// </summary>
        public MultiImageRenderer() : base() {}

        /// <summary>
        /// Make an image renderer that will draw the indicated image, at most maxImages times.
        /// </summary>
        /// <param name="imageSelector"></param>
        /// <param name="maxImages"></param>
        /// <param name="minValue"></param>
        /// <param name="maxValue"></param>
        public MultiImageRenderer(Object imageSelector, int maxImages, int minValue, int maxValue) : this()
        {
            ImageSelector = imageSelector;
            MaxNumberImages = maxImages;
            MinimumValue = minValue;
            MaximumValue = maxValue;
        }

        /// <summary>
        /// Draw our data value
        /// </summary>
        /// <param name="g"></param>
        /// <param name="r"></param>
        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            Image image = GetImage(ImageSelector);
            if (image == null)
                return;

            // Convert our aspect to a numeric value
            // CONSIDER: Is this the best way to do this?
            if (!(Aspect is IConvertible))
                return;
            double aspectValue = ((IConvertible) Aspect).ToDouble(NumberFormatInfo.InvariantInfo);

            // Calculate how many images we need to draw to represent our aspect value
            int numberOfImages;
            if (aspectValue <= MinimumValue)
                numberOfImages = 0;
            else if (aspectValue < MaximumValue)
                numberOfImages = 1 + (int) (MaxNumberImages*(aspectValue - MinimumValue)/MaximumValue);
            else
                numberOfImages = MaxNumberImages;

            // If we need to shrink the image, what will its on-screen dimensions be?
            int imageScaledWidth = image.Width;
            int imageScaledHeight = image.Height;
            if (r.Height < image.Height)
            {
                imageScaledWidth = (int) ((float) image.Width*(float) r.Height/(float) image.Height);
                imageScaledHeight = r.Height;
            }
            // Calculate where the images should be drawn
            Rectangle imageBounds = r;
            imageBounds.Width = (MaxNumberImages*(imageScaledWidth + Spacing)) - Spacing;
            imageBounds.Height = imageScaledHeight;
            imageBounds = AlignRectangle(r, imageBounds);

            // Finally, draw the images
            for (int i = 0; i < numberOfImages; i++)
            {
                g.DrawImage(image, imageBounds.X, imageBounds.Y, imageScaledWidth, imageScaledHeight);
                imageBounds.X += (imageScaledWidth + Spacing);
            }
        }
    }

    /// <summary>
    /// A class to render a value that contains a bitwise-OR'ed collection of values.
    /// </summary>
    /// <typeparam name="T">The type of value that holds the bit-OR'ed flag</typeparam>
    public class FlagRenderer<T> : BaseRenderer where T : IConvertible
    {
        private Dictionary<UInt32, Object> imageMap = new Dictionary<UInt32, object>();
        private List<UInt32> keysInOrder = new List<UInt32>();

        /// <summary>
        /// Get or set the number of pixels between each image
        /// </summary>
        public int Spacing = 1;

        public FlagRenderer() {}

        public void Add(T key, Object imageSelector)
        {
            UInt32 k2 = ((IConvertible) key).ToUInt32(NumberFormatInfo.InvariantInfo);

            imageMap[k2] = imageSelector;
            keysInOrder.Remove(k2);
            keysInOrder.Add(k2);
        }

        public override void Render(Graphics g, Rectangle r)
        {
            DrawBackground(g, r);

            UInt32 v2 = ((IConvertible) Aspect).ToUInt32(NumberFormatInfo.InvariantInfo);

            Point pt = r.Location;
            foreach (UInt32 key in keysInOrder)
            {
                if ((v2 & key) == key)
                {
                    Image image = GetImage(imageMap[key]);
                    if (image != null)
                    {
                        g.DrawImage(image, pt);
                        pt.X += (image.Width + Spacing);
                    }
                }
            }
        }
    }

    #endregion
}